@@ -43,18 +43,10 @@ JXL_INLINE Status SetFromBytes(const Span<const uint8_t> bytes, CodecInOut* io,
                       orig_codec);
 }
 
-// Replaces "bytes" with an encoding of pixels transformed from c_current
-// color space to c_desired.
-Status Encode(const CodecInOut& io, extras::Codec codec,
-              const ColorEncoding& c_desired, size_t bits_per_sample,
-              std::vector<uint8_t>* bytes, ThreadPool* pool = nullptr);
+Status Encode(const extras::PackedPixelFile& ppf, extras::Codec codec,
+              std::vector<uint8_t>* bytes, ThreadPool* pool);
 
-// Deduces codec, calls Encode and writes to file.
-Status Encode(const CodecInOut& io, const ColorEncoding& c_desired,
-              size_t bits_per_sample, const std::string& pathname,
-              std::vector<uint8_t>* bytes, ThreadPool* pool = nullptr);
-// Same, but defaults to metadata.original color_encoding and bits_per_sample.
-Status Encode(const CodecInOut& io, const std::string& pathname,
+Status Encode(const extras::PackedPixelFile& ppf, const std::string& pathname,
               std::vector<uint8_t>* bytes, ThreadPool* pool = nullptr);
 
 }  // namespace jxl
