@@ -8,23 +8,25 @@
 
 // Decodes APNG images in memory.
 
-#include <stdint.h>
+#include <cstdint>
 
 #include "lib/extras/dec/color_hints.h"
 #include "lib/extras/packed_image.h"
-#include "lib/jxl/base/data_parallel.h"
-#include "lib/jxl/base/padded_bytes.h"
 #include "lib/jxl/base/span.h"
 #include "lib/jxl/base/status.h"
-#include "lib/jxl/codec_in_out.h"
 
 namespace jxl {
+
+struct SizeConstraints;
+
 namespace extras {
 
+bool CanDecodeAPNG();
+
 // Decodes `bytes` into `ppf`.
 Status DecodeImageAPNG(Span<const uint8_t> bytes, const ColorHints& color_hints,
-                       const SizeConstraints& constraints,
-                       PackedPixelFile* ppf);
+                       PackedPixelFile* ppf,
+                       const SizeConstraints* constraints = nullptr);
 
 }  // namespace extras
 }  // namespace jxl
