@@ -39,6 +39,13 @@ constexpr auto kJxlPrimariesNames =
                                       {"202", JXL_PRIMARIES_2100},
                                       {"DCI", JXL_PRIMARIES_P3}});
 
+constexpr auto kJxlRenderingIntentNames =
+    to_array<EnumName<JxlRenderingIntent>>(
+        {{"Per", JXL_RENDERING_INTENT_PERCEPTUAL},
+         {"Rel", JXL_RENDERING_INTENT_RELATIVE},
+         {"Sat", JXL_RENDERING_INTENT_SATURATION},
+         {"Abs", JXL_RENDERING_INTENT_ABSOLUTE}});
+
 constexpr auto kJxlTransferFunctionNames =
     to_array<EnumName<JxlTransferFunction>>(
         {{"709", JXL_TRANSFER_FUNCTION_709},
@@ -50,13 +57,6 @@ constexpr auto kJxlTransferFunctionNames =
          {"HLG", JXL_TRANSFER_FUNCTION_HLG},
          {"", JXL_TRANSFER_FUNCTION_GAMMA}});
 
-constexpr auto kJxlRenderingIntentNames =
-    to_array<EnumName<JxlRenderingIntent>>(
-        {{"Per", JXL_RENDERING_INTENT_PERCEPTUAL},
-         {"Rel", JXL_RENDERING_INTENT_RELATIVE},
-         {"Sat", JXL_RENDERING_INTENT_SATURATION},
-         {"Abs", JXL_RENDERING_INTENT_ABSOLUTE}});
-
 template <typename T, size_t N>
 Status ParseEnum(const std::string& token,
                  const std::array<EnumName<T>, N>& enum_values, T* value) {
@@ -164,7 +164,7 @@ Status ParsePrimaries(Tokenizer* tokenizer, JxlColorEncoding* c) {
   JXL_RETURN_IF_ERROR(ParseDouble(&xy_tokenizer, c->primaries_blue_xy + 1));
   c->primaries = JXL_PRIMARIES_CUSTOM;
 
-  return JXL_FAILURE("Invalid primaries %s", str.c_str());
+  return true;
 }
 
 Status ParseRenderingIntent(Tokenizer* tokenizer, JxlColorEncoding* c) {
@@ -203,12 +203,38 @@ Status ParseTransferFunction(Tokenizer* tokenizer, JxlColorEncoding* c) {
 
 Status ParseDescription(const std::string& description, JxlColorEncoding* c) {
   *c = {};
-  Tokenizer tokenizer(&description, '_');
-  JXL_RETURN_IF_ERROR(ParseColorSpace(&tokenizer, c));
-  JXL_RETURN_IF_ERROR(ParseWhitePoint(&tokenizer, c));
-  JXL_RETURN_IF_ERROR(ParsePrimaries(&tokenizer, c));
-  JXL_RETURN_IF_ERROR(ParseRenderingIntent(&tokenizer, c));
-  JXL_RETURN_IF_ERROR(ParseTransferFunction(&tokenizer, c));
+  if (description == "sRGB") {
+    c->color_space = JXL_COLOR_SPACE_RGB;
+    c->white_point = JXL_WHITE_POINT_D65;
+    c->primaries = JXL_PRIMARIES_SRGB;
+    c->transfer_function = JXL_TRANSFER_FUNCTION_SRGB;
+    c->rendering_intent = JXL_RENDERING_INTENT_PERCEPTUAL;
+  } else if (description == "DisplayP3") {
+    c->color_space = JXL_COLOR_SPACE_RGB;
+    c->white_point = JXL_WHITE_POINT_D65;
+    c->primaries = JXL_PRIMARIES_P3;
+    c->transfer_function = JXL_TRANSFER_FUNCTION_SRGB;
+    c->rendering_intent = JXL_RENDERING_INTENT_PERCEPTUAL;
+  } else if (description == "Rec2100PQ") {
+    c->color_space = JXL_COLOR_SPACE_RGB;
+    c->white_point = JXL_WHITE_POINT_D65;
+    c->primaries = JXL_PRIMARIES_2100;
+    c->transfer_function = JXL_TRANSFER_FUNCTION_PQ;
+    c->rendering_intent = JXL_RENDERING_INTENT_RELATIVE;
+  } else if (description == "Rec2100HLG") {
+    c->color_space = JXL_COLOR_SPACE_RGB;
+    c->white_point = JXL_WHITE_POINT_D65;
+    c->primaries = JXL_PRIMARIES_2100;
+    c->transfer_function = JXL_TRANSFER_FUNCTION_HLG;
+    c->rendering_intent = JXL_RENDERING_INTENT_RELATIVE;
+  } else {
+    Tokenizer tokenizer(&description, '_');
+    JXL_RETURN_IF_ERROR(ParseColorSpace(&tokenizer, c));
+    JXL_RETURN_IF_ERROR(ParseWhitePoint(&tokenizer, c));
+    JXL_RETURN_IF_ERROR(ParsePrimaries(&tokenizer, c));
+    JXL_RETURN_IF_ERROR(ParseRenderingIntent(&tokenizer, c));
+    JXL_RETURN_IF_ERROR(ParseTransferFunction(&tokenizer, c));
+  }
   return true;
 }
 
