@@ -15,14 +15,19 @@
 #include "lib/jxl/base/data_parallel.h"
 #include "lib/jxl/base/span.h"
 #include "lib/jxl/base/status.h"
-#include "lib/jxl/codec_in_out.h"
 
 namespace jxl {
+
+struct SizeConstraints;
+
 namespace extras {
 
+bool CanDecodeGIF();
+
 // Decodes `bytes` into `ppf`. color_hints are ignored.
 Status DecodeImageGIF(Span<const uint8_t> bytes, const ColorHints& color_hints,
-                      const SizeConstraints& constraints, PackedPixelFile* ppf);
+                      PackedPixelFile* ppf,
+                      const SizeConstraints* constraints = nullptr);
 
 }  // namespace extras
 }  // namespace jxl
