@@ -19,7 +19,7 @@ HWY_BEFORE_NAMESPACE();
 namespace jxl {
 namespace HWY_NAMESPACE {
 
-static constexpr float rec2020_luminances[3] = {0.2627f, 0.6780f, 0.0593f};
+static constexpr Vector3 rec2020_luminances{0.2627f, 0.6780f, 0.0593f};
 
 Status ToneMapFrame(const std::pair<float, float> display_nits,
                     ImageBundle* const ib, ThreadPool* const pool) {
@@ -44,23 +44,25 @@ Status ToneMapFrame(const std::pair<float, float> display_nits,
        ib->metadata()->IntensityTarget()},
       display_nits, rec2020_luminances);
 
-  return RunOnPool(
-      pool, 0, ib->ysize(), ThreadPool::NoInit,
-      [&](const uint32_t y, size_t /* thread */) {
-        float* const JXL_RESTRICT row_r = ib->color()->PlaneRow(0, y);
-        float* const JXL_RESTRICT row_g = ib->color()->PlaneRow(1, y);
-        float* const JXL_RESTRICT row_b = ib->color()->PlaneRow(2, y);
-        for (size_t x = 0; x < ib->xsize(); x += Lanes(df)) {
-          V red = Load(df, row_r + x);
-          V green = Load(df, row_g + x);
-          V blue = Load(df, row_b + x);
-          tone_mapper.ToneMap(&red, &green, &blue);
-          Store(red, df, row_r + x);
-          Store(green, df, row_g + x);
-          Store(blue, df, row_b + x);
-        }
-      },
-      "ToneMap");
+  const auto process_row = [&](const uint32_t y,
+                               size_t /* thread */) -> Status {
+    float* const JXL_RESTRICT row_r = ib->color()->PlaneRow(0, y);
+    float* const JXL_RESTRICT row_g = ib->color()->PlaneRow(1, y);
+    float* const JXL_RESTRICT row_b = ib->color()->PlaneRow(2, y);
+    for (size_t x = 0; x < ib->xsize(); x += Lanes(df)) {
+      V red = Load(df, row_r + x);
+      V green = Load(df, row_g + x);
+      V blue = Load(df, row_b + x);
+      tone_mapper.ToneMap(&red, &green, &blue);
+      Store(red, df, row_r + x);
+      Store(green, df, row_g + x);
+      Store(blue, df, row_b + x);
+    }
+    return true;
+  };
+  JXL_RETURN_IF_ERROR(RunOnPool(pool, 0, ib->ysize(), ThreadPool::NoInit,
+                                process_row, "ToneMap"));
+  return true;
 }
 
 Status GamutMapFrame(ImageBundle* const ib, float preserve_saturation,
@@ -77,24 +79,24 @@ Status GamutMapFrame(ImageBundle* const ib, float preserve_saturation,
   JXL_RETURN_IF_ERROR(
       ib->TransformTo(linear_rec2020, *JxlGetDefaultCms(), pool));
 
-  JXL_RETURN_IF_ERROR(RunOnPool(
-      pool, 0, ib->ysize(), ThreadPool::NoInit,
-      [&](const uint32_t y, size_t /* thread*/) {
-        float* const JXL_RESTRICT row_r = ib->color()->PlaneRow(0, y);
-        float* const JXL_RESTRICT row_g = ib->color()->PlaneRow(1, y);
-        float* const JXL_RESTRICT row_b = ib->color()->PlaneRow(2, y);
-        for (size_t x = 0; x < ib->xsize(); x += Lanes(df)) {
-          V red = Load(df, row_r + x);
-          V green = Load(df, row_g + x);
-          V blue = Load(df, row_b + x);
-          GamutMap(&red, &green, &blue, rec2020_luminances,
-                   preserve_saturation);
-          Store(red, df, row_r + x);
-          Store(green, df, row_g + x);
-          Store(blue, df, row_b + x);
-        }
-      },
-      "GamutMap"));
+  const auto process_row = [&](const uint32_t y, size_t /* thread*/) -> Status {
+    float* const JXL_RESTRICT row_r = ib->color()->PlaneRow(0, y);
+    float* const JXL_RESTRICT row_g = ib->color()->PlaneRow(1, y);
+    float* const JXL_RESTRICT row_b = ib->color()->PlaneRow(2, y);
+    for (size_t x = 0; x < ib->xsize(); x += Lanes(df)) {
+      V red = Load(df, row_r + x);
+      V green = Load(df, row_g + x);
+      V blue = Load(df, row_b + x);
+      GamutMap(&red, &green, &blue, rec2020_luminances, preserve_saturation);
+      Store(red, df, row_r + x);
+      Store(green, df, row_g + x);
+      Store(blue, df, row_b + x);
+    }
+    return true;
+  };
+
+  JXL_RETURN_IF_ERROR(RunOnPool(pool, 0, ib->ysize(), ThreadPool::NoInit,
+                                process_row, "GamutMap"));
 
   return true;
 }
