@@ -6,11 +6,12 @@
 #ifndef LIB_JXL_COMPRESSED_DC_H_
 #define LIB_JXL_COMPRESSED_DC_H_
 
-#include <stddef.h>
-#include <stdint.h>
+#include <jxl/memory_manager.h>
 
 #include "lib/jxl/ac_context.h"
 #include "lib/jxl/base/data_parallel.h"
+#include "lib/jxl/base/rect.h"
+#include "lib/jxl/base/status.h"
 #include "lib/jxl/frame_header.h"
 #include "lib/jxl/image.h"
 #include "lib/jxl/modular/modular_image.h"
@@ -21,12 +22,13 @@
 namespace jxl {
 
 // Smooth DC in already-smooth areas, to counteract banding.
-void AdaptiveDCSmoothing(const float* dc_factors, Image3F* dc,
-                         ThreadPool* pool);
+Status AdaptiveDCSmoothing(JxlMemoryManager* memory_manager,
+                           const float* dc_factors, Image3F* dc,
+                           ThreadPool* pool);
 
 void DequantDC(const Rect& r, Image3F* dc, ImageB* quant_dc, const Image& in,
                const float* dc_factors, float mul, const float* cfl_factors,
-               YCbCrChromaSubsampling chroma_subsampling,
+               const YCbCrChromaSubsampling& chroma_subsampling,
                const BlockCtxMap& bctx);
 
 }  // namespace jxl
