@@ -3,9 +3,12 @@
 // Use of this source code is governed by a BSD-style
 // license that can be found in the LICENSE file.
 
-#include "lib/jxl/convolve.h"
+#include <atomic>
 
+#include "lib/jxl/base/rect.h"
+#include "lib/jxl/base/status.h"
 #include "lib/jxl/convolve-inl.h"
+#include "lib/jxl/convolve.h"
 
 namespace jxl {
 
@@ -111,36 +114,39 @@ void SlowSymmetric3Row(const ImageF& in, const int64_t iy, const int64_t xsize,
 
 }  // namespace
 
-void SlowSymmetric3(const ImageF& in, const Rect& rect,
-                    const WeightsSymmetric3& weights, ThreadPool* pool,
-                    ImageF* JXL_RESTRICT out) {
+Status SlowSymmetric3(const ImageF& in, const Rect& rect,
+                      const WeightsSymmetric3& weights, ThreadPool* pool,
+                      ImageF* JXL_RESTRICT out) {
   const int64_t xsize = static_cast<int64_t>(rect.xsize());
   const int64_t ysize = static_cast<int64_t>(rect.ysize());
   const int64_t kRadius = 1;
 
-  JXL_CHECK(RunOnPool(
-      pool, 0, static_cast<uint32_t>(ysize), ThreadPool::NoInit,
-      [&](const uint32_t task, size_t /*thread*/) {
-        const int64_t iy = task;
-        float* JXL_RESTRICT out_row = out->Row(static_cast<size_t>(iy));
+  const auto process_row = [&](const uint32_t task,
+                               size_t /*thread*/) -> Status {
+    const int64_t iy = task;
+    float* JXL_RESTRICT out_row = out->Row(static_cast<size_t>(iy));
 
-        if (iy < kRadius || iy >= ysize - kRadius) {
-          SlowSymmetric3Row<WrapMirror>(in, iy, xsize, ysize, weights, out_row);
-        } else {
-          SlowSymmetric3Row<WrapUnchanged>(in, iy, xsize, ysize, weights,
-                                           out_row);
-        }
-      },
-      "SlowSymmetric3"));
+    if (iy < kRadius || iy >= ysize - kRadius) {
+      SlowSymmetric3Row<WrapMirror>(in, iy, xsize, ysize, weights, out_row);
+    } else {
+      SlowSymmetric3Row<WrapUnchanged>(in, iy, xsize, ysize, weights, out_row);
+    }
+    return true;
+  };
+  JXL_RETURN_IF_ERROR(RunOnPool(pool, 0, static_cast<uint32_t>(ysize),
+                                ThreadPool::NoInit, process_row,
+                                "SlowSymmetric3"));
+  return true;
 }
 
 namespace {
 
 // Separable kernels, any radius.
-float SlowSeparablePixel(const ImageF& in, const Rect& rect, const int64_t x,
-                         const int64_t y, const int64_t radius,
-                         const float* JXL_RESTRICT horz_weights,
-                         const float* JXL_RESTRICT vert_weights) {
+StatusOr<float> SlowSeparablePixel(const ImageF& in, const Rect& rect,
+                                   const int64_t x, const int64_t y,
+                                   const int64_t radius,
+                                   const float* JXL_RESTRICT horz_weights,
+                                   const float* JXL_RESTRICT vert_weights) {
   const size_t xsize = in.xsize();
   const size_t ysize = in.ysize();
   const WrapMirror wrap;
@@ -149,12 +155,12 @@ float SlowSeparablePixel(const ImageF& in, const Rect& rect, const int64_t x,
   for (int dy = -radius; dy <= radius; ++dy) {
     const float wy = vert_weights[std::abs(dy) * 4];
     const size_t sy = wrap(rect.y0() + y + dy, ysize);
-    JXL_CHECK(sy < ysize);
+    JXL_ENSURE(sy < ysize);
     const float* const JXL_RESTRICT row = in.ConstRow(sy);
     for (int dx = -radius; dx <= radius; ++dx) {
       const float wx = horz_weights[std::abs(dx) * 4];
       const size_t sx = wrap(rect.x0() + x + dx, xsize);
-      JXL_CHECK(sx < xsize);
+      JXL_ENSURE(sx < xsize);
       mul += row[sx] * wx * wy;
     }
   }
@@ -162,37 +168,41 @@ float SlowSeparablePixel(const ImageF& in, const Rect& rect, const int64_t x,
 }
 
 template <int R, typename Weights>
-void SlowSeparable(const ImageF& in, const Rect& in_rect,
-                   const Weights& weights, ThreadPool* pool, ImageF* out,
-                   const Rect& out_rect) {
-  JXL_ASSERT(in_rect.xsize() == out_rect.xsize());
-  JXL_ASSERT(in_rect.ysize() == out_rect.ysize());
-  JXL_ASSERT(in_rect.IsInside(Rect(in)));
-  JXL_ASSERT(out_rect.IsInside(Rect(*out)));
+Status SlowSeparable(const ImageF& in, const Rect& in_rect,
+                     const Weights& weights, ThreadPool* pool, ImageF* out,
+                     const Rect& out_rect) {
+  JXL_ENSURE(in_rect.xsize() == out_rect.xsize());
+  JXL_ENSURE(in_rect.ysize() == out_rect.ysize());
+  JXL_ENSURE(in_rect.IsInside(Rect(in)));
+  JXL_ENSURE(out_rect.IsInside(Rect(*out)));
   const float* horz_weights = &weights.horz[0];
   const float* vert_weights = &weights.vert[0];
 
-  const size_t ysize = in_rect.ysize();
-  JXL_CHECK(RunOnPool(
-      pool, 0, static_cast<uint32_t>(ysize), ThreadPool::NoInit,
-      [&](const uint32_t task, size_t /*thread*/) {
-        const int64_t y = task;
+  const auto process_row = [&](const uint32_t task,
+                               size_t /*thread*/) -> Status {
+    const int64_t y = task;
 
-        float* const JXL_RESTRICT row_out = out_rect.Row(out, y);
-        for (size_t x = 0; x < in_rect.xsize(); ++x) {
-          row_out[x] = SlowSeparablePixel(in, in_rect, x, y, /*radius=*/R,
-                                          horz_weights, vert_weights);
-        }
-      },
-      "SlowSeparable"));
+    float* const JXL_RESTRICT row_out = out_rect.Row(out, y);
+    for (size_t x = 0; x < in_rect.xsize(); ++x) {
+      JXL_ASSIGN_OR_RETURN(row_out[x],
+                           SlowSeparablePixel(in, in_rect, x, y, /*radius=*/R,
+                                              horz_weights, vert_weights));
+    }
+    return true;
+  };
+  const size_t ysize = in_rect.ysize();
+  JXL_RETURN_IF_ERROR(RunOnPool(pool, 0, static_cast<uint32_t>(ysize),
+                                ThreadPool::NoInit, process_row,
+                                "SlowSeparable"));
+  return true;
 }
 
 }  // namespace
 
-void SlowSeparable5(const ImageF& in, const Rect& in_rect,
-                    const WeightsSeparable5& weights, ThreadPool* pool,
-                    ImageF* out, const Rect& out_rect) {
-  SlowSeparable<2>(in, in_rect, weights, pool, out, out_rect);
+Status SlowSeparable5(const ImageF& in, const Rect& in_rect,
+                      const WeightsSeparable5& weights, ThreadPool* pool,
+                      ImageF* out, const Rect& out_rect) {
+  return SlowSeparable<2>(in, in_rect, weights, pool, out, out_rect);
 }
 
 }  // namespace jxl
