@@ -5,13 +5,13 @@
 
 #include "lib/jxl/dec_group.h"
 
-#include <stdint.h>
-#include <string.h>
-
 #include <algorithm>
+#include <cstdint>
+#include <cstring>
 #include <memory>
 #include <utility>
 
+#include "lib/jxl/chroma_from_luma.h"
 #include "lib/jxl/frame_header.h"
 
 #undef HWY_TARGET_INCLUDE
@@ -24,16 +24,14 @@
 #include "lib/jxl/base/bits.h"
 #include "lib/jxl/base/common.h"
 #include "lib/jxl/base/printf_macros.h"
+#include "lib/jxl/base/rect.h"
 #include "lib/jxl/base/status.h"
 #include "lib/jxl/coeff_order.h"
 #include "lib/jxl/common.h"  // kMaxNumPasses
-#include "lib/jxl/convolve.h"
-#include "lib/jxl/dct_scales.h"
 #include "lib/jxl/dec_cache.h"
 #include "lib/jxl/dec_transforms-inl.h"
 #include "lib/jxl/dec_xyb.h"
 #include "lib/jxl/entropy_coder.h"
-#include "lib/jxl/epf.h"
 #include "lib/jxl/quant_weights.h"
 #include "lib/jxl/quantizer-inl.h"
 #include "lib/jxl/quantizer.h"
@@ -141,7 +139,7 @@ void DequantLane(Vec<D> scaled_dequant_x, Vec<D> scaled_dequant_y,
 template <ACType ac_type>
 void DequantBlock(const AcStrategy& acs, float inv_global_scale, int quant,
                   float x_dm_multiplier, float b_dm_multiplier, Vec<D> x_cc_mul,
-                  Vec<D> b_cc_mul, size_t kind, size_t size,
+                  Vec<D> b_cc_mul, AcStrategyType kind, size_t size,
                   const Quantizer& quantizer, size_t covered_blocks,
                   const size_t* sbx,
                   const float* JXL_RESTRICT* JXL_RESTRICT dc_row,
@@ -167,14 +165,16 @@ void DequantBlock(const AcStrategy& acs, float inv_global_scale, int quant,
   }
 }
 
-Status DecodeGroupImpl(GetBlock* JXL_RESTRICT get_block,
+Status DecodeGroupImpl(const FrameHeader& frame_header,
+                       GetBlock* JXL_RESTRICT get_block,
                        GroupDecCache* JXL_RESTRICT group_dec_cache,
                        PassesDecoderState* JXL_RESTRICT dec_state,
                        size_t thread, size_t group_idx,
                        RenderPipelineInput& render_pipeline_input,
-                       ImageBundle* decoded, DrawMode draw) {
+                       jpeg::JPEGData* jpeg_data, DrawMode draw) {
   // TODO(veluca): investigate cache usage in this function.
-  const Rect block_rect = dec_state->shared->BlockGroupRect(group_idx);
+  const Rect block_rect =
+      dec_state->shared->frame_dim.BlockGroupRect(group_idx);
   const AcStrategyImage& ac_strategy = dec_state->shared->ac_strategy;
 
   const size_t xsize_blocks = block_rect.xsize();
@@ -184,8 +184,7 @@ Status DecodeGroupImpl(GetBlock* JXL_RESTRICT get_block,
 
   const float inv_global_scale = dec_state->shared->quantizer.InvGlobalScale();
 
-  const YCbCrChromaSubsampling& cs =
-      dec_state->shared->frame_header.chroma_subsampling;
+  const YCbCrChromaSubsampling& cs = frame_header.chroma_subsampling;
 
   const auto kJpegDctMin = Set(di16_full, -4095);
   const auto kJpegDctMax = Set(di16_full, 4095);
@@ -211,13 +210,14 @@ Status DecodeGroupImpl(GetBlock* JXL_RESTRICT get_block,
   std::array<int, 3> dcoff = {};
 
   // TODO(veluca): all of this should be done only once per image.
-  if (decoded->IsJPEG()) {
-    if (!dec_state->shared->cmap.IsJPEGCompatible()) {
+  const ColorCorrelation& color_correlation = dec_state->shared->cmap.base();
+  if (jpeg_data) {
+    if (!color_correlation.IsJPEGCompatible()) {
       return JXL_FAILURE("The CfL map is not JPEG-compatible");
     }
-    jpeg_is_gray = (decoded->jpeg_data->components.size() == 1);
-    jpeg_c_map = JpegOrder(dec_state->shared->frame_header.color_transform,
-                           jpeg_is_gray);
+    jpeg_is_gray = (jpeg_data->components.size() == 1);
+    JXL_ENSURE(frame_header.color_transform != ColorTransform::kXYB);
+    jpeg_c_map = JpegOrder(frame_header.color_transform, jpeg_is_gray);
     const std::vector<QuantEncoding>& qe =
         dec_state->shared->matrices.encodings();
     if (qe.empty() || qe[0].mode != QuantEncoding::Mode::kQuantModeRAW ||
@@ -225,15 +225,16 @@ Status DecodeGroupImpl(GetBlock* JXL_RESTRICT get_block,
       return JXL_FAILURE(
           "Quantization table is not a JPEG quantization table.");
     }
+    JXL_ENSURE(qe[0].qraw.qtable->size() == 3 * 8 * 8);
+    int* qtable = qe[0].qraw.qtable->data();
     for (size_t c = 0; c < 3; c++) {
-      if (dec_state->shared->frame_header.color_transform ==
-          ColorTransform::kNone) {
-        dcoff[c] = 1024 / (*qe[0].qraw.qtable)[64 * c];
+      if (frame_header.color_transform == ColorTransform::kNone) {
+        dcoff[c] = 1024 / qtable[64 * c];
       }
       for (size_t i = 0; i < 64; i++) {
         // Transpose the matrix, as it will be used on the transposed block.
-        int n = qe[0].qraw.qtable->at(64 + i);
-        int d = qe[0].qraw.qtable->at(64 * c + i);
+        int n = qtable[64 + i];
+        int d = qtable[64 * c + i];
         if (n <= 0 || d <= 0 || n >= 65536 || d >= 65536) {
           return JXL_FAILURE("Invalid JPEG quantization table");
         }
@@ -281,10 +282,10 @@ Status DecodeGroupImpl(GetBlock* JXL_RESTRICT get_block,
     float* JXL_RESTRICT idct_row[3];
     int16_t* JXL_RESTRICT jpeg_row[3];
     for (size_t c = 0; c < 3; c++) {
-      idct_row[c] = render_pipeline_input.GetBuffer(c).second.Row(
-          render_pipeline_input.GetBuffer(c).first, sby[c] * kBlockDim);
-      if (decoded->IsJPEG()) {
-        auto& component = decoded->jpeg_data->components[jpeg_c_map[c]];
+      const auto& buffer = render_pipeline_input.GetBuffer(c);
+      idct_row[c] = buffer.second.Row(buffer.first, sby[c] * kBlockDim);
+      if (jpeg_data) {
+        auto& component = jpeg_data->components[jpeg_c_map[c]];
         jpeg_row[c] =
             component.coeffs.data() +
             (component.width_in_blocks * (r[c].y0() + sby[c]) + r[c].x0()) *
@@ -296,10 +297,8 @@ Status DecodeGroupImpl(GetBlock* JXL_RESTRICT get_block,
     for (size_t tx = 0; tx < DivCeil(xsize_blocks, kColorTileDimInBlocks);
          tx++) {
       size_t abs_tx = tx + block_rect.x0() / kColorTileDimInBlocks;
-      auto x_cc_mul =
-          Set(d, dec_state->shared->cmap.YtoXRatio(row_cmap[0][abs_tx]));
-      auto b_cc_mul =
-          Set(d, dec_state->shared->cmap.YtoBRatio(row_cmap[2][abs_tx]));
+      auto x_cc_mul = Set(d, color_correlation.YtoXRatio(row_cmap[0][abs_tx]));
+      auto b_cc_mul = Set(d, color_correlation.YtoBRatio(row_cmap[2][abs_tx]));
       // Increment bx by llf_x because those iterations would otherwise
       // immediately continue (!IsFirstBlock). Reduces mispredictions.
       for (; bx < xsize_blocks && bx < (tx + 1) * kColorTileDimInBlocks;) {
@@ -325,7 +324,7 @@ Status DecodeGroupImpl(GetBlock* JXL_RESTRICT get_block,
         } else {
           // No point in reading from bitstream without accumulating and not
           // drawing.
-          JXL_ASSERT(draw == kDraw);
+          JXL_ENSURE(draw == kDraw);
           if (ac_type == ACType::k16) {
             memset(group_dec_cache->dec_group_qblock16, 0,
                    size * 3 * sizeof(int16_t));
@@ -348,8 +347,8 @@ Status DecodeGroupImpl(GetBlock* JXL_RESTRICT get_block,
           continue;
         }
 
-        if (JXL_UNLIKELY(decoded->IsJPEG())) {
-          if (acs.Strategy() != AcStrategy::Type::DCT) {
+        if (JXL_UNLIKELY(jpeg_data)) {
+          if (acs.Strategy() != AcStrategyType::DCT) {
             return JXL_FAILURE(
                 "Can only decode to JPEG if only DCT-8 is used.");
           }
@@ -366,7 +365,7 @@ Status DecodeGroupImpl(GetBlock* JXL_RESTRICT get_block,
             int16_t* JXL_RESTRICT jpeg_pos =
                 jpeg_row[c] + sbx[c] * kDCTBlockSize;
             // JPEG XL is transposed, JPEG is not.
-            auto transposed_dct = qblock[c].ptr32;
+            auto* transposed_dct = qblock[c].ptr32;
             Transpose8x8InPlace(transposed_dct);
             // No CfL - no need to store the y block converted to integers.
             if (!cs.Is444() ||
@@ -386,8 +385,8 @@ Status DecodeGroupImpl(GetBlock* JXL_RESTRICT get_block,
               }
             } else {
               // transposed_dct_y contains the y channel block, transposed.
-              const auto scale = Set(
-                  di, dec_state->shared->cmap.RatioJPEG(row_cmap[c][abs_tx]));
+              const auto scale =
+                  Set(di, ColorCorrelation::RatioJPEG(row_cmap[c][abs_tx]));
               const auto round = Set(di, 1 << (kCFLFixedPointPrecision - 1));
               for (int i = 0; i < 64; i += Lanes(d)) {
                 auto in = Load(di, transposed_dct + i);
@@ -418,7 +417,7 @@ Status DecodeGroupImpl(GetBlock* JXL_RESTRICT get_block,
           // Dequantize and add predictions.
           dequant_block(
               acs, inv_global_scale, row_quant[bx], dec_state->x_dm_multiplier,
-              dec_state->b_dm_multiplier, x_cc_mul, b_cc_mul, acs.RawStrategy(),
+              dec_state->b_dm_multiplier, x_cc_mul, b_cc_mul, acs.Strategy(),
               size, dec_state->shared->quantizer,
               acs.covered_blocks_y() * acs.covered_blocks_x(), sbx, dc_rows,
               dc_stride,
@@ -481,8 +480,10 @@ Status DecodeACVarBlock(size_t ctx_offset, size_t log2_covered_blocks,
 
   size_t nzeros =
       decoder->ReadHybridUintInlined<uses_lz77>(nzero_ctx, br, context_map);
-  if (nzeros + covered_blocks > size) {
-    return JXL_FAILURE("Invalid AC: nzeros too large");
+  if (nzeros > size - covered_blocks) {
+    return JXL_FAILURE("Invalid AC: nzeros %" PRIuS " too large for %" PRIuS
+                       " 8x8 blocks",
+                       nzeros, covered_blocks);
   }
   for (size_t y = 0; y < acs.covered_blocks_y(); y++) {
     for (size_t x = 0; x < acs.covered_blocks_x(); x++) {
@@ -516,9 +517,10 @@ Status DecodeACVarBlock(size_t ctx_offset, size_t log2_covered_blocks,
     nzeros -= prev;
   }
   if (JXL_UNLIKELY(nzeros != 0)) {
-    return JXL_FAILURE("Invalid AC: nzeros not 0. Block (%" PRIuS ", %" PRIuS
+    return JXL_FAILURE("Invalid AC: nzeros at end of block is %" PRIuS
+                       ", should be 0. Block (%" PRIuS ", %" PRIuS
                        "), channel %" PRIuS,
-                       bx, by, c);
+                       nzeros, bx, by, c);
   }
 
   return true;
@@ -574,13 +576,14 @@ struct GetBlockFromBitstream : public GetBlock {
     return true;
   }
 
-  Status Init(BitReader* JXL_RESTRICT* JXL_RESTRICT readers, size_t num_passes,
+  Status Init(const FrameHeader& frame_header,
+              BitReader* JXL_RESTRICT* JXL_RESTRICT readers, size_t num_passes,
               size_t group_idx, size_t histo_selector_bits, const Rect& rect,
               GroupDecCache* JXL_RESTRICT group_dec_cache,
               PassesDecoderState* dec_state, size_t first_pass) {
     for (size_t i = 0; i < 3; i++) {
-      hshift[i] = dec_state->shared->frame_header.chroma_subsampling.HShift(i);
-      vshift[i] = dec_state->shared->frame_header.chroma_subsampling.VShift(i);
+      hshift[i] = frame_header.chroma_subsampling.HShift(i);
+      vshift[i] = frame_header.chroma_subsampling.VShift(i);
     }
     this->coeff_order_size = dec_state->shared->coeff_order_size;
     this->coeff_orders =
@@ -588,8 +591,7 @@ struct GetBlockFromBitstream : public GetBlock {
     this->context_map = dec_state->context_map.data() + first_pass;
     this->readers = readers;
     this->num_passes = num_passes;
-    this->shift_for_pass =
-        dec_state->shared->frame_header.passes.shift + first_pass;
+    this->shift_for_pass = frame_header.passes.shift + first_pass;
     this->group_dec_cache = group_dec_cache;
     this->rect = rect;
     block_ctx_map = &dec_state->shared->block_ctx_map;
@@ -607,12 +609,14 @@ struct GetBlockFromBitstream : public GetBlock {
       }
       ctx_offset[pass] = cur_histogram * block_ctx_map->NumACContexts();
 
-      decoders[pass] =
-          ANSSymbolReader(&dec_state->code[pass + first_pass], readers[pass]);
+      JXL_ASSIGN_OR_RETURN(
+          decoders[pass],
+          ANSSymbolReader::Create(&dec_state->code[pass + first_pass],
+                                  readers[pass]));
     }
     nzeros_stride = group_dec_cache->num_nzeroes[0].PixelsPerRow();
     for (size_t i = 0; i < num_passes; i++) {
-      JXL_ASSERT(
+      JXL_ENSURE(
           nzeros_stride ==
           static_cast<size_t>(group_dec_cache->num_nzeroes[i].PixelsPerRow()));
     }
@@ -646,7 +650,7 @@ struct GetBlockFromEncoder : public GetBlock {
   Status LoadBlock(size_t bx, size_t by, const AcStrategy& acs, size_t size,
                    size_t log2_covered_blocks, ACPtr block[3],
                    ACType ac_type) override {
-    JXL_DASSERT(ac_type == ACType::k32);
+    JXL_ENSURE(ac_type == ACType::k32);
     for (size_t c = 0; c < 3; c++) {
       // for each pass
       for (size_t i = 0; i < quantized_ac->size(); i++) {
@@ -661,61 +665,70 @@ struct GetBlockFromEncoder : public GetBlock {
     return true;
   }
 
-  GetBlockFromEncoder(const std::vector<std::unique_ptr<ACImage>>& ac,
-                      size_t group_idx, const uint32_t* shift_for_pass)
-      : quantized_ac(&ac), shift_for_pass(shift_for_pass) {
+  static StatusOr<GetBlockFromEncoder> Create(
+      const std::vector<std::unique_ptr<ACImage>>& ac, size_t group_idx,
+      const uint32_t* shift_for_pass) {
+    GetBlockFromEncoder result(ac, group_idx, shift_for_pass);
     // TODO(veluca): not supported with chroma subsampling.
-    for (size_t i = 0; i < quantized_ac->size(); i++) {
-      JXL_CHECK((*quantized_ac)[i]->Type() == ACType::k32);
+    for (size_t i = 0; i < ac.size(); i++) {
+      JXL_ENSURE(ac[i]->Type() == ACType::k32);
       for (size_t c = 0; c < 3; c++) {
-        rows[i][c] = (*quantized_ac)[i]->PlaneRow(c, group_idx, 0).ptr32;
+        result.rows[i][c] = ac[i]->PlaneRow(c, group_idx, 0).ptr32;
       }
     }
+    return result;
   }
 
   const std::vector<std::unique_ptr<ACImage>>* JXL_RESTRICT quantized_ac;
   size_t offset = 0;
   const int32_t* JXL_RESTRICT rows[kMaxNumPasses][3];
   const uint32_t* shift_for_pass = nullptr;  // not owned
+
+ private:
+  GetBlockFromEncoder(const std::vector<std::unique_ptr<ACImage>>& ac,
+                      size_t group_idx, const uint32_t* shift_for_pass)
+      : quantized_ac(&ac), shift_for_pass(shift_for_pass) {}
 };
 
 HWY_EXPORT(DecodeGroupImpl);
 
 }  // namespace
 
-Status DecodeGroup(BitReader* JXL_RESTRICT* JXL_RESTRICT readers,
+Status DecodeGroup(const FrameHeader& frame_header,
+                   BitReader* JXL_RESTRICT* JXL_RESTRICT readers,
                    size_t num_passes, size_t group_idx,
                    PassesDecoderState* JXL_RESTRICT dec_state,
                    GroupDecCache* JXL_RESTRICT group_dec_cache, size_t thread,
                    RenderPipelineInput& render_pipeline_input,
-                   ImageBundle* JXL_RESTRICT decoded, size_t first_pass,
+                   jpeg::JPEGData* JXL_RESTRICT jpeg_data, size_t first_pass,
                    bool force_draw, bool dc_only, bool* should_run_pipeline) {
-  DrawMode draw = (num_passes + first_pass ==
-                   dec_state->shared->frame_header.passes.num_passes) ||
-                          force_draw
-                      ? kDraw
-                      : kDontDraw;
+  JxlMemoryManager* memory_manager = dec_state->memory_manager();
+  DrawMode draw =
+      (num_passes + first_pass == frame_header.passes.num_passes) || force_draw
+          ? kDraw
+          : kDontDraw;
 
   if (should_run_pipeline) {
     *should_run_pipeline = draw != kDontDraw;
   }
 
   if (draw == kDraw && num_passes == 0 && first_pass == 0) {
-    group_dec_cache->InitDCBufferOnce();
-    const YCbCrChromaSubsampling& cs =
-        dec_state->shared->frame_header.chroma_subsampling;
+    JXL_RETURN_IF_ERROR(group_dec_cache->InitDCBufferOnce(memory_manager));
+    const YCbCrChromaSubsampling& cs = frame_header.chroma_subsampling;
     for (size_t c : {0, 1, 2}) {
       size_t hs = cs.HShift(c);
       size_t vs = cs.VShift(c);
       // We reuse filter_input_storage here as it is not currently in use.
-      const Rect src_rect_precs = dec_state->shared->BlockGroupRect(group_idx);
+      const Rect src_rect_precs =
+          dec_state->shared->frame_dim.BlockGroupRect(group_idx);
       const Rect src_rect =
           Rect(src_rect_precs.x0() >> hs, src_rect_precs.y0() >> vs,
                src_rect_precs.xsize() >> hs, src_rect_precs.ysize() >> vs);
       const Rect copy_rect(kRenderPipelineXOffset, 2, src_rect.xsize(),
                            src_rect.ysize());
-      CopyImageToWithPadding(src_rect, dec_state->shared->dc->Plane(c), 2,
-                             copy_rect, &group_dec_cache->dc_buffer);
+      JXL_RETURN_IF_ERROR(
+          CopyImageToWithPadding(src_rect, dec_state->shared->dc->Plane(c), 2,
+                                 copy_rect, &group_dec_cache->dc_buffer));
       // Mirrorpad. Interleaving left and right padding ensures that padding
       // works out correctly even for images with DC size of 1.
       for (size_t y = 0; y < src_rect.ysize() + 4; y++) {
@@ -734,9 +747,10 @@ Status DecodeGroup(BitReader* JXL_RESTRICT* JXL_RESTRICT readers,
           }
         }
       }
-      Rect dst_rect = render_pipeline_input.GetBuffer(c).second;
-      ImageF* upsampling_dst = render_pipeline_input.GetBuffer(c).first;
-      JXL_ASSERT(dst_rect.IsInside(*upsampling_dst));
+      const auto& buffer = render_pipeline_input.GetBuffer(c);
+      Rect dst_rect = buffer.second;
+      ImageF* upsampling_dst = buffer.first;
+      JXL_ENSURE(dst_rect.IsInside(*upsampling_dst));
 
       RenderPipelineStage::RowInfo input_rows(1, std::vector<float*>(5));
       RenderPipelineStage::RowInfo output_rows(1, std::vector<float*>(8));
@@ -744,7 +758,7 @@ Status DecodeGroup(BitReader* JXL_RESTRICT* JXL_RESTRICT readers,
            y++) {
         for (ssize_t iy = 0; iy < 5; iy++) {
           input_rows[0][iy] = group_dec_cache->dc_buffer.Row(
-              Mirror(ssize_t(y) + iy - 2,
+              Mirror(static_cast<ssize_t>(y) + iy - 2,
                      dec_state->shared->dc->Plane(c).ysize() >> vs) +
               2 - src_rect.y0());
         }
@@ -754,9 +768,9 @@ Status DecodeGroup(BitReader* JXL_RESTRICT* JXL_RESTRICT readers,
               kRenderPipelineXOffset;
         }
         // Arguments set to 0/nullptr are not used.
-        dec_state->upsampler8x->ProcessRow(input_rows, output_rows,
-                                           /*xextra=*/0, src_rect.xsize(), 0, 0,
-                                           thread);
+        JXL_RETURN_IF_ERROR(dec_state->upsampler8x->ProcessRow(
+            input_rows, output_rows,
+            /*xextra=*/0, src_rect.xsize(), 0, 0, thread));
       }
     }
     return true;
@@ -764,21 +778,21 @@ Status DecodeGroup(BitReader* JXL_RESTRICT* JXL_RESTRICT readers,
 
   size_t histo_selector_bits = 0;
   if (dc_only) {
-    JXL_ASSERT(num_passes == 0);
+    JXL_ENSURE(num_passes == 0);
   } else {
-    JXL_ASSERT(dec_state->shared->num_histograms > 0);
+    JXL_ENSURE(dec_state->shared->num_histograms > 0);
     histo_selector_bits = CeilLog2Nonzero(dec_state->shared->num_histograms);
   }
 
   auto get_block = jxl::make_unique<GetBlockFromBitstream>();
-  JXL_RETURN_IF_ERROR(
-      get_block->Init(readers, num_passes, group_idx, histo_selector_bits,
-                      dec_state->shared->BlockGroupRect(group_idx),
-                      group_dec_cache, dec_state, first_pass));
+  JXL_RETURN_IF_ERROR(get_block->Init(
+      frame_header, readers, num_passes, group_idx, histo_selector_bits,
+      dec_state->shared->frame_dim.BlockGroupRect(group_idx), group_dec_cache,
+      dec_state, first_pass));
 
   JXL_RETURN_IF_ERROR(HWY_DYNAMIC_DISPATCH(DecodeGroupImpl)(
-      get_block.get(), group_dec_cache, dec_state, thread, group_idx,
-      render_pipeline_input, decoded, draw));
+      frame_header, get_block.get(), group_dec_cache, dec_state, thread,
+      group_idx, render_pipeline_input, jpeg_data, draw));
 
   for (size_t pass = 0; pass < num_passes; pass++) {
     if (!get_block->decoders[pass].CheckANSFinalState()) {
@@ -788,23 +802,27 @@ Status DecodeGroup(BitReader* JXL_RESTRICT* JXL_RESTRICT readers,
   return true;
 }
 
-Status DecodeGroupForRoundtrip(const std::vector<std::unique_ptr<ACImage>>& ac,
+Status DecodeGroupForRoundtrip(const FrameHeader& frame_header,
+                               const std::vector<std::unique_ptr<ACImage>>& ac,
                                size_t group_idx,
                                PassesDecoderState* JXL_RESTRICT dec_state,
                                GroupDecCache* JXL_RESTRICT group_dec_cache,
                                size_t thread,
                                RenderPipelineInput& render_pipeline_input,
-                               ImageBundle* JXL_RESTRICT decoded,
+                               jpeg::JPEGData* JXL_RESTRICT jpeg_data,
                                AuxOut* aux_out) {
-  GetBlockFromEncoder get_block(ac, group_idx,
-                                dec_state->shared->frame_header.passes.shift);
-  group_dec_cache->InitOnce(
+  JxlMemoryManager* memory_manager = dec_state->memory_manager();
+  JXL_ASSIGN_OR_RETURN(
+      GetBlockFromEncoder get_block,
+      GetBlockFromEncoder::Create(ac, group_idx, frame_header.passes.shift));
+  JXL_RETURN_IF_ERROR(group_dec_cache->InitOnce(
+      memory_manager,
       /*num_passes=*/0,
-      /*used_acs=*/(1u << AcStrategy::kNumValidStrategies) - 1);
+      /*used_acs=*/(1u << AcStrategy::kNumValidStrategies) - 1));
 
   return HWY_DYNAMIC_DISPATCH(DecodeGroupImpl)(
-      &get_block, group_dec_cache, dec_state, thread, group_idx,
-      render_pipeline_input, decoded, kDraw);
+      frame_header, &get_block, group_dec_cache, dec_state, thread, group_idx,
+      render_pipeline_input, jpeg_data, kDraw);
 }
 
 }  // namespace jxl
