@@ -5,10 +5,9 @@
 
 #include "lib/jxl/dec_modular.h"
 
-#include <stdint.h>
+#include <jxl/memory_manager.h>
 
-#include <atomic>
-#include <sstream>
+#include <cstdint>
 #include <vector>
 
 #include "lib/jxl/frame_header.h"
@@ -18,10 +17,9 @@
 #include <hwy/foreach_target.h>
 #include <hwy/highway.h>
 
-#include "lib/jxl/alpha.h"
 #include "lib/jxl/base/compiler_specific.h"
 #include "lib/jxl/base/printf_macros.h"
-#include "lib/jxl/base/span.h"
+#include "lib/jxl/base/rect.h"
 #include "lib/jxl/base/status.h"
 #include "lib/jxl/compressed_dc.h"
 #include "lib/jxl/epf.h"
@@ -105,14 +103,14 @@ void SingleFromSingleAccurate(const size_t xsize,
 
 // convert custom [bits]-bit float (with [exp_bits] exponent bits) stored as int
 // back to binary32 float
-void int_to_float(const pixel_type* const JXL_RESTRICT row_in,
-                  float* const JXL_RESTRICT row_out, const size_t xsize,
-                  const int bits, const int exp_bits) {
+Status int_to_float(const pixel_type* const JXL_RESTRICT row_in,
+                    float* const JXL_RESTRICT row_out, const size_t xsize,
+                    const int bits, const int exp_bits) {
+  static_assert(sizeof(pixel_type) == sizeof(float));
   if (bits == 32) {
-    JXL_ASSERT(sizeof(pixel_type) == sizeof(float));
-    JXL_ASSERT(exp_bits == 8);
+    JXL_ENSURE(exp_bits == 8);
     memcpy(row_out, row_in, xsize * sizeof(float));
-    return;
+    return true;
   }
   int exp_bias = (1 << (exp_bits - 1)) - 1;
   int sign_shift = bits - 1;
@@ -145,39 +143,43 @@ void int_to_float(const pixel_type* const JXL_RESTRICT row_in,
     // broke up the arbitrary float into its parts, now reassemble into
     // binary32
     exp += 127;
-    JXL_ASSERT(exp >= 0);
+    JXL_ENSURE(exp >= 0);
     f = (signbit ? 0x80000000 : 0);
     f |= (exp << 23);
     f |= mantissa;
     memcpy(&row_out[x], &f, 4);
   }
+  return true;
 }
 
+#if JXL_DEBUG_V_LEVEL >= 1
 std::string ModularStreamId::DebugString() const {
   std::ostringstream os;
-  os << (kind == kGlobalData   ? "ModularGlobal"
-         : kind == kVarDCTDC   ? "VarDCTDC"
-         : kind == kModularDC  ? "ModularDC"
-         : kind == kACMetadata ? "ACMeta"
-         : kind == kQuantTable ? "QuantTable"
-         : kind == kModularAC  ? "ModularAC"
-                               : "");
-  if (kind == kVarDCTDC || kind == kModularDC || kind == kACMetadata ||
-      kind == kModularAC) {
+  os << (kind == GlobalData   ? "ModularGlobal"
+         : kind == VarDCTDC   ? "VarDCTDC"
+         : kind == ModularDC  ? "ModularDC"
+         : kind == ACMetadata ? "ACMeta"
+         : kind == QuantTable ? "QuantTable"
+         : kind == ModularAC  ? "ModularAC"
+                              : "");
+  if (kind == VarDCTDC || kind == ModularDC || kind == ACMetadata ||
+      kind == ModularAC) {
     os << " group " << group_id;
   }
-  if (kind == kModularAC) {
+  if (kind == ModularAC) {
     os << " pass " << pass_id;
   }
-  if (kind == kQuantTable) {
+  if (kind == QuantTable) {
     os << " " << quant_table_id;
   }
   return os.str();
 }
+#endif
 
 Status ModularFrameDecoder::DecodeGlobalInfo(BitReader* reader,
                                              const FrameHeader& frame_header,
                                              bool allow_truncated_group) {
+  JxlMemoryManager* memory_manager = this->memory_manager();
   bool decode_color = frame_header.encoding == FrameEncoding::kModular;
   const auto& metadata = frame_header.nonserialized_metadata->m;
   bool is_gray = metadata.color_encoding.IsGray();
@@ -187,7 +189,7 @@ Status ModularFrameDecoder::DecodeGlobalInfo(BitReader* reader,
   }
   do_color = decode_color;
   size_t nb_extra = metadata.extra_channel_info.size();
-  bool has_tree = reader->ReadBits(1);
+  bool has_tree = static_cast<bool>(reader->ReadBits(1));
   if (!allow_truncated_group ||
       reader->TotalBitsConsumed() < reader->TotalBytes() * kBitsPerByte) {
     if (has_tree) {
@@ -195,9 +197,10 @@ Status ModularFrameDecoder::DecodeGlobalInfo(BitReader* reader,
           std::min(static_cast<size_t>(1 << 22),
                    1024 + frame_dim.xsize * frame_dim.ysize *
                               (nb_chans + nb_extra) / 16);
-      JXL_RETURN_IF_ERROR(DecodeTree(reader, &tree, tree_size_limit));
       JXL_RETURN_IF_ERROR(
-          DecodeHistograms(reader, (tree.size() + 1) / 2, &code, &context_map));
+          DecodeTree(memory_manager, reader, &tree, tree_size_limit));
+      JXL_RETURN_IF_ERROR(DecodeHistograms(
+          memory_manager, reader, (tree.size() + 1) / 2, &code, &context_map));
     }
   }
   if (!do_color) nb_chans = 0;
@@ -214,8 +217,10 @@ Status ModularFrameDecoder::DecodeGlobalInfo(BitReader* reader,
     }
   }
 
-  Image gi(frame_dim.xsize, frame_dim.ysize, metadata.bit_depth.bits_per_sample,
-           nb_chans + nb_extra);
+  JXL_ASSIGN_OR_RETURN(
+      Image gi,
+      Image::Create(memory_manager, frame_dim.xsize, frame_dim.ysize,
+                    metadata.bit_depth.bits_per_sample, nb_chans + nb_extra));
 
   all_same_shift = true;
   if (frame_header.color_transform == ColorTransform::kYCbCr) {
@@ -226,7 +231,7 @@ Status ModularFrameDecoder::DecodeGlobalInfo(BitReader* reader,
           DivCeil(frame_dim.xsize, 1 << gi.channel[c].hshift);
       size_t ysize_shifted =
           DivCeil(frame_dim.ysize, 1 << gi.channel[c].vshift);
-      gi.channel[c].shrink(xsize_shifted, ysize_shifted);
+      JXL_RETURN_IF_ERROR(gi.channel[c].shrink(xsize_shifted, ysize_shifted));
       if (gi.channel[c].hshift != gi.channel[0].hshift ||
           gi.channel[c].vshift != gi.channel[0].vshift)
         all_same_shift = false;
@@ -235,8 +240,9 @@ Status ModularFrameDecoder::DecodeGlobalInfo(BitReader* reader,
 
   for (size_t ec = 0, c = nb_chans; ec < nb_extra; ec++, c++) {
     size_t ecups = frame_header.extra_channel_upsampling[ec];
-    gi.channel[c].shrink(DivCeil(frame_dim.xsize_upsampled, ecups),
-                         DivCeil(frame_dim.ysize_upsampled, ecups));
+    JXL_RETURN_IF_ERROR(
+        gi.channel[c].shrink(DivCeil(frame_dim.xsize_upsampled, ecups),
+                             DivCeil(frame_dim.ysize_upsampled, ecups)));
     gi.channel[c].hshift = gi.channel[c].vshift =
         CeilLog2Nonzero(ecups) - CeilLog2Nonzero(frame_header.upsampling);
     if (gi.channel[c].hshift != gi.channel[0].hshift ||
@@ -293,18 +299,19 @@ void ModularFrameDecoder::MaybeDropFullImage() {
 }
 
 Status ModularFrameDecoder::DecodeGroup(
-    const Rect& rect, BitReader* reader, int minShift, int maxShift,
-    const ModularStreamId& stream, bool zerofill, PassesDecoderState* dec_state,
-    RenderPipelineInput* render_pipeline_input, bool allow_truncated,
-    bool* should_run_pipeline) {
+    const FrameHeader& frame_header, const Rect& rect, BitReader* reader,
+    int minShift, int maxShift, const ModularStreamId& stream, bool zerofill,
+    PassesDecoderState* dec_state, RenderPipelineInput* render_pipeline_input,
+    bool allow_truncated, bool* should_run_pipeline) {
   JXL_DEBUG_V(6, "Decoding %s with rect %s and shift bracket %d..%d %s",
               stream.DebugString().c_str(), Description(rect).c_str(), minShift,
               maxShift, zerofill ? "using zerofill" : "");
-  JXL_DASSERT(stream.kind == ModularStreamId::kModularDC ||
-              stream.kind == ModularStreamId::kModularAC);
+  JXL_ENSURE(stream.kind == ModularStreamId::Kind::ModularDC ||
+             stream.kind == ModularStreamId::Kind::ModularAC);
   const size_t xsize = rect.xsize();
   const size_t ysize = rect.ysize();
-  Image gi(xsize, ysize, full_image.bitdepth, 0);
+  JXL_ASSIGN_OR_RETURN(Image gi, Image::Create(memory_manager_, xsize, ysize,
+                                               full_image.bitdepth, 0));
   // start at the first bigger-than-groupsize non-metachannel
   size_t c = full_image.nb_meta_channels;
   for (; c < full_image.channel.size(); c++) {
@@ -326,7 +333,8 @@ Status ModularFrameDecoder::DecodeGroup(
         memset(row_out, 0, r.xsize() * sizeof(*row_out));
       }
     } else {
-      Channel gc(r.xsize(), r.ysize());
+      JXL_ASSIGN_OR_RETURN(
+          Channel gc, Channel::Create(memory_manager_, r.xsize(), r.ysize()));
       if (zerofill) ZeroFillImage(&gc.plane);
       gc.hshift = fc.hshift;
       gc.vshift = fc.vshift;
@@ -338,7 +346,6 @@ Status ModularFrameDecoder::DecodeGroup(
   // problems later (in ModularImageToDecodedRect).
   if (gi.channel.empty()) {
     if (dec_state && should_run_pipeline) {
-      const auto& frame_header = dec_state->shared->frame_header;
       const auto* metadata = frame_header.nonserialized_metadata;
       if (do_color || metadata->m.num_extra_channels > 0) {
         // Signal to FrameDecoder that we do not have some of the required input
@@ -359,13 +366,13 @@ Status ModularFrameDecoder::DecodeGroup(
   }
   // Undo global transforms that have been pushed to the group level
   if (!use_full_image) {
-    JXL_ASSERT(render_pipeline_input);
-    for (auto t : global_transform) {
+    JXL_ENSURE(render_pipeline_input);
+    for (const auto& t : global_transform) {
       JXL_RETURN_IF_ERROR(t.Inverse(gi, global_header.wp_header));
     }
-    JXL_RETURN_IF_ERROR(ModularImageToDecodedRect(gi, dec_state, nullptr,
-                                                  *render_pipeline_input,
-                                                  Rect(0, 0, gi.w, gi.h)));
+    JXL_RETURN_IF_ERROR(ModularImageToDecodedRect(
+        frame_header, gi, dec_state, nullptr, *render_pipeline_input,
+        Rect(0, 0, gi.w, gi.h)));
     return true;
   }
   int gic = 0;
@@ -377,25 +384,31 @@ Status ModularFrameDecoder::DecodeGroup(
     Rect r(rect.x0() >> fc.hshift, rect.y0() >> fc.vshift,
            rect.xsize() >> fc.hshift, rect.ysize() >> fc.vshift, fc.w, fc.h);
     if (r.xsize() == 0 || r.ysize() == 0) continue;
-    JXL_ASSERT(use_full_image);
-    CopyImageTo(/*rect_from=*/Rect(0, 0, r.xsize(), r.ysize()),
-                /*from=*/gi.channel[gic].plane,
-                /*rect_to=*/r, /*to=*/&fc.plane);
+    JXL_ENSURE(use_full_image);
+    JXL_RETURN_IF_ERROR(
+        CopyImageTo(/*rect_from=*/Rect(0, 0, r.xsize(), r.ysize()),
+                    /*from=*/gi.channel[gic].plane,
+                    /*rect_to=*/r, /*to=*/&fc.plane));
     gic++;
   }
   return true;
 }
 
-Status ModularFrameDecoder::DecodeVarDCTDC(size_t group_id, BitReader* reader,
+Status ModularFrameDecoder::DecodeVarDCTDC(const FrameHeader& frame_header,
+                                           size_t group_id, BitReader* reader,
                                            PassesDecoderState* dec_state) {
-  const Rect r = dec_state->shared->DCGroupRect(group_id);
+  JxlMemoryManager* memory_manager = dec_state->memory_manager();
+  const Rect r = dec_state->shared->frame_dim.DCGroupRect(group_id);
+  JXL_DEBUG_V(6, "Decoding VarDCT DC with rect %s", Description(r).c_str());
   // TODO(eustas): investigate if we could reduce the impact of
   //               EvalRationalPolynomial; generally speaking, the limit is
   //               2**(128/(3*magic)), where 128 comes from IEEE 754 exponent,
   //               3 comes from XybToRgb that cubes the values, and "magic" is
   //               the sum of all other contributions. 2**18 is known to lead
   //               to NaN on input found by fuzzing (see commit message).
-  Image image(r.xsize(), r.ysize(), full_image.bitdepth, 3);
+  JXL_ASSIGN_OR_RETURN(Image image,
+                       Image::Create(memory_manager, r.xsize(), r.ysize(),
+                                     full_image.bitdepth, 3));
   size_t stream_id = ModularStreamId::VarDCTDC(group_id).ID(frame_dim);
   reader->Refill();
   size_t extra_precision = reader->ReadFixedBits<2>();
@@ -403,50 +416,62 @@ Status ModularFrameDecoder::DecodeVarDCTDC(size_t group_id, BitReader* reader,
   ModularOptions options;
   for (size_t c = 0; c < 3; c++) {
     Channel& ch = image.channel[c < 2 ? c ^ 1 : c];
-    ch.w >>= dec_state->shared->frame_header.chroma_subsampling.HShift(c);
-    ch.h >>= dec_state->shared->frame_header.chroma_subsampling.VShift(c);
-    ch.shrink();
+    ch.w >>= frame_header.chroma_subsampling.HShift(c);
+    ch.h >>= frame_header.chroma_subsampling.VShift(c);
+    JXL_RETURN_IF_ERROR(ch.shrink());
   }
   if (!ModularGenericDecompress(
           reader, image, /*header=*/nullptr, stream_id, &options,
           /*undo_transforms=*/true, &tree, &code, &context_map)) {
-    return JXL_FAILURE("Failed to decode modular DC group");
+    return JXL_FAILURE("Failed to decode VarDCT DC group (DC group id %d)",
+                       static_cast<int>(group_id));
   }
   DequantDC(r, &dec_state->shared_storage.dc_storage,
             &dec_state->shared_storage.quant_dc, image,
             dec_state->shared->quantizer.MulDC(), mul,
-            dec_state->shared->cmap.DCFactors(),
-            dec_state->shared->frame_header.chroma_subsampling,
-            dec_state->shared->block_ctx_map);
+            dec_state->shared->cmap.base().DCFactors(),
+            frame_header.chroma_subsampling, dec_state->shared->block_ctx_map);
   return true;
 }
 
-Status ModularFrameDecoder::DecodeAcMetadata(size_t group_id, BitReader* reader,
+Status ModularFrameDecoder::DecodeAcMetadata(const FrameHeader& frame_header,
+                                             size_t group_id, BitReader* reader,
                                              PassesDecoderState* dec_state) {
-  const Rect r = dec_state->shared->DCGroupRect(group_id);
+  JxlMemoryManager* memory_manager = dec_state->memory_manager();
+  const Rect r = dec_state->shared->frame_dim.DCGroupRect(group_id);
+  JXL_DEBUG_V(6, "Decoding AcMetadata with rect %s", Description(r).c_str());
   size_t upper_bound = r.xsize() * r.ysize();
   reader->Refill();
   size_t count = reader->ReadBits(CeilLog2Nonzero(upper_bound)) + 1;
   size_t stream_id = ModularStreamId::ACMetadata(group_id).ID(frame_dim);
   // YToX, YToB, ACS + QF, EPF
-  Image image(r.xsize(), r.ysize(), full_image.bitdepth, 4);
+  JXL_ASSIGN_OR_RETURN(Image image,
+                       Image::Create(memory_manager, r.xsize(), r.ysize(),
+                                     full_image.bitdepth, 4));
   static_assert(kColorTileDimInBlocks == 8, "Color tile size changed");
   Rect cr(r.x0() >> 3, r.y0() >> 3, (r.xsize() + 7) >> 3, (r.ysize() + 7) >> 3);
-  image.channel[0] = Channel(cr.xsize(), cr.ysize(), 3, 3);
-  image.channel[1] = Channel(cr.xsize(), cr.ysize(), 3, 3);
-  image.channel[2] = Channel(count, 2, 0, 0);
+  JXL_ASSIGN_OR_RETURN(
+      image.channel[0],
+      Channel::Create(memory_manager, cr.xsize(), cr.ysize(), 3, 3));
+  JXL_ASSIGN_OR_RETURN(
+      image.channel[1],
+      Channel::Create(memory_manager, cr.xsize(), cr.ysize(), 3, 3));
+  JXL_ASSIGN_OR_RETURN(image.channel[2],
+                       Channel::Create(memory_manager, count, 2, 0, 0));
   ModularOptions options;
   if (!ModularGenericDecompress(
           reader, image, /*header=*/nullptr, stream_id, &options,
           /*undo_transforms=*/true, &tree, &code, &context_map)) {
     return JXL_FAILURE("Failed to decode AC metadata");
   }
-  ConvertPlaneAndClamp(Rect(image.channel[0].plane), image.channel[0].plane, cr,
-                       &dec_state->shared_storage.cmap.ytox_map);
-  ConvertPlaneAndClamp(Rect(image.channel[1].plane), image.channel[1].plane, cr,
-                       &dec_state->shared_storage.cmap.ytob_map);
+  JXL_RETURN_IF_ERROR(
+      ConvertPlaneAndClamp(Rect(image.channel[0].plane), image.channel[0].plane,
+                           cr, &dec_state->shared_storage.cmap.ytox_map));
+  JXL_RETURN_IF_ERROR(
+      ConvertPlaneAndClamp(Rect(image.channel[1].plane), image.channel[1].plane,
+                           cr, &dec_state->shared_storage.cmap.ytob_map));
   size_t num = 0;
-  bool is444 = dec_state->shared->frame_header.chroma_subsampling.Is444();
+  bool is444 = frame_header.chroma_subsampling.Is444();
   auto& ac_strategy = dec_state->shared_storage.ac_strategy;
   size_t xlim = std::min(ac_strategy.xsize(), r.x0() + r.xsize());
   size_t ylim = std::min(ac_strategy.ysize(), r.y0() + r.ysize());
@@ -493,25 +518,25 @@ Status ModularFrameDecoder::DecodeAcMetadata(size_t group_id, BitReader* reader,
         return JXL_FAILURE("Invalid AC strategy, y overflow");
       }
       JXL_RETURN_IF_ERROR(
-          ac_strategy.SetNoBoundsCheck(x, y, AcStrategy::Type(row_in_1[num])));
+          ac_strategy.SetNoBoundsCheck(x, y, AcStrategyType(row_in_1[num])));
       row_qf[ix] = 1 + std::max<int32_t>(0, std::min(Quantizer::kQuantMax - 1,
                                                      row_in_2[num]));
       num++;
     }
   }
   dec_state->used_acs |= local_used_acs;
-  if (dec_state->shared->frame_header.loop_filter.epf_iters > 0) {
-    ComputeSigma(r, dec_state);
+  if (frame_header.loop_filter.epf_iters > 0) {
+    JXL_RETURN_IF_ERROR(ComputeSigma(frame_header.loop_filter, r, dec_state));
   }
   return true;
 }
 
 Status ModularFrameDecoder::ModularImageToDecodedRect(
-    Image& gi, PassesDecoderState* dec_state, jxl::ThreadPool* pool,
-    RenderPipelineInput& render_pipeline_input, Rect modular_rect) {
-  const auto& frame_header = dec_state->shared->frame_header;
+    const FrameHeader& frame_header, Image& gi, PassesDecoderState* dec_state,
+    jxl::ThreadPool* pool, RenderPipelineInput& render_pipeline_input,
+    Rect modular_rect) const {
   const auto* metadata = frame_header.nonserialized_metadata;
-  JXL_CHECK(gi.transform.empty());
+  JXL_ENSURE(gi.transform.empty());
 
   auto get_row = [&](size_t c, size_t y) {
     const auto& buffer = render_pipeline_input.GetBuffer(c);
@@ -537,13 +562,13 @@ Status ModularFrameDecoder::ModularImageToDecodedRect(
       } else if (rgb_from_gray) {
         c_in = 0;
       }
-      JXL_ASSERT(c_in < gi.channel.size());
+      JXL_ENSURE(c_in < gi.channel.size());
       Channel& ch_in = gi.channel[c_in];
       // TODO(eustas): could we detect it on earlier stage?
       if (ch_in.w == 0 || ch_in.h == 0) {
         return JXL_FAILURE("Empty image");
       }
-      JXL_CHECK(ch_in.hshift <= 3 && ch_in.vshift <= 3);
+      JXL_ENSURE(ch_in.hshift <= 3 && ch_in.vshift <= 3);
       Rect r = render_pipeline_input.GetBuffer(c).second;
       Rect mr(modular_rect.x0() >> ch_in.hshift,
               modular_rect.y0() >> ch_in.vshift,
@@ -560,72 +585,76 @@ Status ModularFrameDecoder::ModularImageToDecodedRect(
                            mr.xsize(), mr.ysize(), r.xsize(), r.ysize());
       }
       if (frame_header.color_transform == ColorTransform::kXYB && c == 2) {
-        JXL_ASSERT(!fp);
-        JXL_RETURN_IF_ERROR(RunOnPool(
-            pool, 0, ysize_shifted, ThreadPool::NoInit,
-            [&](const uint32_t task, size_t /* thread */) {
-              const size_t y = task;
-              const pixel_type* const JXL_RESTRICT row_in =
-                  mr.Row(&ch_in.plane, y);
-              const pixel_type* const JXL_RESTRICT row_in_Y =
-                  mr.Row(&gi.channel[0].plane, y);
-              float* const JXL_RESTRICT row_out = get_row(c, y);
-              HWY_DYNAMIC_DISPATCH(MultiplySum)
-              (xsize_shifted, row_in, row_in_Y, factor, row_out);
-            },
-            "ModularIntToFloat"));
+        JXL_ENSURE(!fp);
+        const auto process_row = [&](const uint32_t task,
+                                     size_t /* thread */) -> Status {
+          const size_t y = task;
+          const pixel_type* const JXL_RESTRICT row_in = mr.Row(&ch_in.plane, y);
+          const pixel_type* const JXL_RESTRICT row_in_Y =
+              mr.Row(&gi.channel[0].plane, y);
+          float* const JXL_RESTRICT row_out = get_row(c, y);
+          HWY_DYNAMIC_DISPATCH(MultiplySum)
+          (xsize_shifted, row_in, row_in_Y, factor, row_out);
+          return true;
+        };
+        JXL_RETURN_IF_ERROR(RunOnPool(pool, 0, ysize_shifted,
+                                      ThreadPool::NoInit, process_row,
+                                      "ModularIntToFloat"));
       } else if (fp) {
         int bits = metadata->m.bit_depth.bits_per_sample;
         int exp_bits = metadata->m.bit_depth.exponent_bits_per_sample;
-        JXL_RETURN_IF_ERROR(RunOnPool(
-            pool, 0, ysize_shifted, ThreadPool::NoInit,
-            [&](const uint32_t task, size_t /* thread */) {
-              const size_t y = task;
-              const pixel_type* const JXL_RESTRICT row_in =
-                  mr.Row(&ch_in.plane, y);
-              if (rgb_from_gray) {
-                for (size_t cc = 0; cc < 3; cc++) {
-                  float* const JXL_RESTRICT row_out = get_row(cc, y);
-                  int_to_float(row_in, row_out, xsize_shifted, bits, exp_bits);
-                }
-              } else {
-                float* const JXL_RESTRICT row_out = get_row(c, y);
-                int_to_float(row_in, row_out, xsize_shifted, bits, exp_bits);
-              }
-            },
-            "ModularIntToFloat_losslessfloat"));
+        const auto process_row = [&](const uint32_t task,
+                                     size_t /* thread */) -> Status {
+          const size_t y = task;
+          const pixel_type* const JXL_RESTRICT row_in = mr.Row(&ch_in.plane, y);
+          if (rgb_from_gray) {
+            for (size_t cc = 0; cc < 3; cc++) {
+              float* const JXL_RESTRICT row_out = get_row(cc, y);
+              JXL_RETURN_IF_ERROR(
+                  int_to_float(row_in, row_out, xsize_shifted, bits, exp_bits));
+            }
+          } else {
+            float* const JXL_RESTRICT row_out = get_row(c, y);
+            JXL_RETURN_IF_ERROR(
+                int_to_float(row_in, row_out, xsize_shifted, bits, exp_bits));
+          }
+          return true;
+        };
+        JXL_RETURN_IF_ERROR(RunOnPool(pool, 0, ysize_shifted,
+                                      ThreadPool::NoInit, process_row,
+                                      "ModularIntToFloat_losslessfloat"));
       } else {
-        JXL_RETURN_IF_ERROR(RunOnPool(
-            pool, 0, ysize_shifted, ThreadPool::NoInit,
-            [&](const uint32_t task, size_t /* thread */) {
-              const size_t y = task;
-              const pixel_type* const JXL_RESTRICT row_in =
-                  mr.Row(&ch_in.plane, y);
-              if (rgb_from_gray) {
-                if (full_image.bitdepth < 23) {
-                  HWY_DYNAMIC_DISPATCH(RgbFromSingle)
-                  (xsize_shifted, row_in, factor, get_row(0, y), get_row(1, y),
-                   get_row(2, y));
-                } else {
-                  SingleFromSingleAccurate(xsize_shifted, row_in, factor,
-                                           get_row(0, y));
-                  SingleFromSingleAccurate(xsize_shifted, row_in, factor,
-                                           get_row(1, y));
-                  SingleFromSingleAccurate(xsize_shifted, row_in, factor,
-                                           get_row(2, y));
-                }
-              } else {
-                float* const JXL_RESTRICT row_out = get_row(c, y);
-                if (full_image.bitdepth < 23) {
-                  HWY_DYNAMIC_DISPATCH(SingleFromSingle)
-                  (xsize_shifted, row_in, factor, row_out);
-                } else {
-                  SingleFromSingleAccurate(xsize_shifted, row_in, factor,
-                                           row_out);
-                }
-              }
-            },
-            "ModularIntToFloat"));
+        const auto process_row = [&](const uint32_t task,
+                                     size_t /* thread */) -> Status {
+          const size_t y = task;
+          const pixel_type* const JXL_RESTRICT row_in = mr.Row(&ch_in.plane, y);
+          if (rgb_from_gray) {
+            if (full_image.bitdepth < 23) {
+              HWY_DYNAMIC_DISPATCH(RgbFromSingle)
+              (xsize_shifted, row_in, factor, get_row(0, y), get_row(1, y),
+               get_row(2, y));
+            } else {
+              SingleFromSingleAccurate(xsize_shifted, row_in, factor,
+                                       get_row(0, y));
+              SingleFromSingleAccurate(xsize_shifted, row_in, factor,
+                                       get_row(1, y));
+              SingleFromSingleAccurate(xsize_shifted, row_in, factor,
+                                       get_row(2, y));
+            }
+          } else {
+            float* const JXL_RESTRICT row_out = get_row(c, y);
+            if (full_image.bitdepth < 23) {
+              HWY_DYNAMIC_DISPATCH(SingleFromSingle)
+              (xsize_shifted, row_in, factor, row_out);
+            } else {
+              SingleFromSingleAccurate(xsize_shifted, row_in, factor, row_out);
+            }
+          }
+          return true;
+        };
+        JXL_RETURN_IF_ERROR(RunOnPool(pool, 0, ysize_shifted,
+                                      ThreadPool::NoInit, process_row,
+                                      "ModularIntToFloat"));
       }
       if (rgb_from_gray) {
         break;
@@ -641,11 +670,12 @@ Status ModularFrameDecoder::ModularImageToDecodedRect(
     int bits = eci.bit_depth.bits_per_sample;
     int exp_bits = eci.bit_depth.exponent_bits_per_sample;
     bool fp = eci.bit_depth.floating_point_sample;
-    JXL_ASSERT(fp || bits < 32);
+    JXL_ENSURE(fp || bits < 32);
     const double factor = fp ? 0 : (1.0 / ((1u << bits) - 1));
-    JXL_ASSERT(c < gi.channel.size());
+    JXL_ENSURE(c < gi.channel.size());
     Channel& ch_in = gi.channel[c];
-    Rect r = render_pipeline_input.GetBuffer(3 + ec).second;
+    const auto& buffer = render_pipeline_input.GetBuffer(3 + ec);
+    Rect r = buffer.second;
     Rect mr(modular_rect.x0() >> ch_in.hshift,
             modular_rect.y0() >> ch_in.vshift,
             DivCeil(modular_rect.xsize(), 1 << ch_in.hshift),
@@ -659,11 +689,11 @@ Status ModularFrameDecoder::ModularImageToDecodedRect(
                          mr.xsize(), mr.ysize(), r.xsize(), r.ysize());
     }
     for (size_t y = 0; y < r.ysize(); ++y) {
-      float* const JXL_RESTRICT row_out =
-          r.Row(render_pipeline_input.GetBuffer(3 + ec).first, y);
+      float* const JXL_RESTRICT row_out = r.Row(buffer.first, y);
       const pixel_type* const JXL_RESTRICT row_in = mr.Row(&ch_in.plane, y);
       if (fp) {
-        int_to_float(row_in, row_out, r.xsize(), bits, exp_bits);
+        JXL_RETURN_IF_ERROR(
+            int_to_float(row_in, row_out, r.xsize(), bits, exp_bits));
       } else {
         if (full_image.bitdepth < 23) {
           HWY_DYNAMIC_DISPATCH(SingleFromSingle)
@@ -677,11 +707,18 @@ Status ModularFrameDecoder::ModularImageToDecodedRect(
   return true;
 }
 
-Status ModularFrameDecoder::FinalizeDecoding(PassesDecoderState* dec_state,
+Status ModularFrameDecoder::FinalizeDecoding(const FrameHeader& frame_header,
+                                             PassesDecoderState* dec_state,
                                              jxl::ThreadPool* pool,
                                              bool inplace) {
   if (!use_full_image) return true;
-  Image gi = (inplace ? std::move(full_image) : full_image.clone());
+  JxlMemoryManager* memory_manager = dec_state->memory_manager();
+  Image gi{memory_manager};
+  if (inplace) {
+    gi = std::move(full_image);
+  } else {
+    JXL_ASSIGN_OR_RETURN(gi, Image::Clone(full_image));
+  }
   size_t xsize = gi.w;
   size_t ysize = gi.h;
 
@@ -693,44 +730,41 @@ Status ModularFrameDecoder::FinalizeDecoding(PassesDecoderState* dec_state,
 
   // Undo the global transforms
   gi.undo_transforms(global_header.wp_header, pool);
-  JXL_DASSERT(global_transform.empty());
+  JXL_ENSURE(global_transform.empty());
   if (gi.error) return JXL_FAILURE("Undoing transforms failed");
 
   for (size_t i = 0; i < dec_state->shared->frame_dim.num_groups; i++) {
     dec_state->render_pipeline->ClearDone(i);
   }
-  std::atomic<bool> has_error{false};
-  JXL_RETURN_IF_ERROR(RunOnPool(
-      pool, 0, dec_state->shared->frame_dim.num_groups,
-      [&](size_t num_threads) {
-        const auto& frame_header = dec_state->shared->frame_header;
-        bool use_group_ids = (frame_header.encoding == FrameEncoding::kVarDCT ||
-                              (frame_header.flags & FrameHeader::kNoise));
-        return dec_state->render_pipeline->PrepareForThreads(num_threads,
-                                                             use_group_ids);
-      },
-      [&](const uint32_t group, size_t thread_id) {
-        RenderPipelineInput input =
-            dec_state->render_pipeline->GetInputBuffers(group, thread_id);
-        if (!ModularImageToDecodedRect(gi, dec_state, nullptr, input,
-                                       dec_state->shared->GroupRect(group))) {
-          has_error = true;
-          return;
-        }
-        input.Done();
-      },
-      "ModularToRect"));
-  if (has_error) {
-    return JXL_FAILURE("Error producing input to render pipeline");
-  }
+
+  const auto init = [&](size_t num_threads) -> Status {
+    bool use_group_ids = (frame_header.encoding == FrameEncoding::kVarDCT ||
+                          (frame_header.flags & FrameHeader::kNoise));
+    JXL_RETURN_IF_ERROR(dec_state->render_pipeline->PrepareForThreads(
+        num_threads, use_group_ids));
+    return true;
+  };
+  const auto process_group = [&](const uint32_t group,
+                                 size_t thread_id) -> Status {
+    RenderPipelineInput input =
+        dec_state->render_pipeline->GetInputBuffers(group, thread_id);
+    JXL_RETURN_IF_ERROR(ModularImageToDecodedRect(
+        frame_header, gi, dec_state, nullptr, input,
+        dec_state->shared->frame_dim.GroupRect(group)));
+    JXL_RETURN_IF_ERROR(input.Done());
+    return true;
+  };
+  JXL_RETURN_IF_ERROR(RunOnPool(pool, 0,
+                                dec_state->shared->frame_dim.num_groups, init,
+                                process_group, "ModularToRect"));
   return true;
 }
 
 static constexpr const float kAlmostZero = 1e-8f;
 
 Status ModularFrameDecoder::DecodeQuantTable(
-    size_t required_size_x, size_t required_size_y, BitReader* br,
-    QuantEncoding* encoding, size_t idx,
+    JxlMemoryManager* memory_manager, size_t required_size_x,
+    size_t required_size_y, BitReader* br, QuantEncoding* encoding, size_t idx,
     ModularFrameDecoder* modular_frame_decoder) {
   JXL_RETURN_IF_ERROR(F16Coder::Read(br, &encoding->qraw.qtable_den));
   if (encoding->qraw.qtable_den < kAlmostZero) {
@@ -738,12 +772,14 @@ Status ModularFrameDecoder::DecodeQuantTable(
     // be negative.
     return JXL_FAILURE("Invalid qtable_den: value too small");
   }
-  Image image(required_size_x, required_size_y, 8, 3);
+  JXL_ASSIGN_OR_RETURN(
+      Image image,
+      Image::Create(memory_manager, required_size_x, required_size_y, 8, 3));
   ModularOptions options;
   if (modular_frame_decoder) {
+    JXL_ASSIGN_OR_RETURN(ModularStreamId qt, ModularStreamId::QuantTable(idx));
     JXL_RETURN_IF_ERROR(ModularGenericDecompress(
-        br, image, /*header=*/nullptr,
-        ModularStreamId::QuantTable(idx).ID(modular_frame_decoder->frame_dim),
+        br, image, /*header=*/nullptr, qt.ID(modular_frame_decoder->frame_dim),
         &options, /*undo_transforms=*/true, &modular_frame_decoder->tree,
         &modular_frame_decoder->code, &modular_frame_decoder->context_map));
   } else {
@@ -752,15 +788,19 @@ Status ModularFrameDecoder::DecodeQuantTable(
                                                  /*undo_transforms=*/true));
   }
   if (!encoding->qraw.qtable) {
-    encoding->qraw.qtable = new std::vector<int>();
+    encoding->qraw.qtable =
+        new std::vector<int>(required_size_x * required_size_y * 3);
+  } else {
+    JXL_ENSURE(encoding->qraw.qtable->size() ==
+               required_size_x * required_size_y * 3);
   }
-  encoding->qraw.qtable->resize(required_size_x * required_size_y * 3);
+  int* qtable = encoding->qraw.qtable->data();
   for (size_t c = 0; c < 3; c++) {
     for (size_t y = 0; y < required_size_y; y++) {
       int32_t* JXL_RESTRICT row = image.channel[c].Row(y);
       for (size_t x = 0; x < required_size_x; x++) {
-        (*encoding->qraw.qtable)[c * required_size_x * required_size_y +
-                                 y * required_size_x + x] = row[x];
+        qtable[c * required_size_x * required_size_y + y * required_size_x +
+               x] = row[x];
         if (row[x] <= 0) {
           return JXL_FAILURE("Invalid raw quantization table");
         }
