@@ -5,7 +5,7 @@
 
 #include "lib/jxl/dec_xyb.h"
 
-#include <string.h>
+#include <cstring>
 
 #undef HWY_TARGET_INCLUDE
 #define HWY_TARGET_INCLUDE "lib/jxl/dec_xyb.cc"
@@ -14,17 +14,16 @@
 
 #include "lib/jxl/base/compiler_specific.h"
 #include "lib/jxl/base/matrix_ops.h"
+#include "lib/jxl/base/rect.h"
+#include "lib/jxl/base/sanitizers.h"
 #include "lib/jxl/base/status.h"
 #include "lib/jxl/cms/jxl_cms_internal.h"
 #include "lib/jxl/cms/opsin_params.h"
 #include "lib/jxl/color_encoding_internal.h"
-#include "lib/jxl/dec_group_border.h"
 #include "lib/jxl/dec_xyb-inl.h"
-#include "lib/jxl/fields.h"
 #include "lib/jxl/image.h"
 #include "lib/jxl/opsin_params.h"
 #include "lib/jxl/quantizer.h"
-#include "lib/jxl/sanitizers.h"
 HWY_BEFORE_NAMESPACE();
 namespace jxl {
 namespace HWY_NAMESPACE {
@@ -32,80 +31,85 @@ namespace HWY_NAMESPACE {
 // These templates are not found via ADL.
 using hwy::HWY_NAMESPACE::MulAdd;
 
-void OpsinToLinearInplace(Image3F* JXL_RESTRICT inout, ThreadPool* pool,
-                          const OpsinParams& opsin_params) {
+Status OpsinToLinearInplace(Image3F* JXL_RESTRICT inout, ThreadPool* pool,
+                            const OpsinParams& opsin_params) {
   JXL_CHECK_IMAGE_INITIALIZED(*inout, Rect(*inout));
 
   const size_t xsize = inout->xsize();  // not padded
-  JXL_CHECK(RunOnPool(
-      pool, 0, inout->ysize(), ThreadPool::NoInit,
-      [&](const uint32_t task, size_t /* thread */) {
-        const size_t y = task;
+  const auto process_row = [&](const uint32_t task,
+                               size_t /* thread */) -> Status {
+    const size_t y = task;
 
-        // Faster than adding via ByteOffset at end of loop.
-        float* JXL_RESTRICT row0 = inout->PlaneRow(0, y);
-        float* JXL_RESTRICT row1 = inout->PlaneRow(1, y);
-        float* JXL_RESTRICT row2 = inout->PlaneRow(2, y);
+    // Faster than adding via ByteOffset at end of loop.
+    float* JXL_RESTRICT row0 = inout->PlaneRow(0, y);
+    float* JXL_RESTRICT row1 = inout->PlaneRow(1, y);
+    float* JXL_RESTRICT row2 = inout->PlaneRow(2, y);
 
-        const HWY_FULL(float) d;
+    const HWY_FULL(float) d;
 
-        for (size_t x = 0; x < xsize; x += Lanes(d)) {
-          const auto in_opsin_x = Load(d, row0 + x);
-          const auto in_opsin_y = Load(d, row1 + x);
-          const auto in_opsin_b = Load(d, row2 + x);
-          auto linear_r = Undefined(d);
-          auto linear_g = Undefined(d);
-          auto linear_b = Undefined(d);
-          XybToRgb(d, in_opsin_x, in_opsin_y, in_opsin_b, opsin_params,
-                   &linear_r, &linear_g, &linear_b);
+    for (size_t x = 0; x < xsize; x += Lanes(d)) {
+      const auto in_opsin_x = Load(d, row0 + x);
+      const auto in_opsin_y = Load(d, row1 + x);
+      const auto in_opsin_b = Load(d, row2 + x);
+      auto linear_r = Undefined(d);
+      auto linear_g = Undefined(d);
+      auto linear_b = Undefined(d);
+      XybToRgb(d, in_opsin_x, in_opsin_y, in_opsin_b, opsin_params, &linear_r,
+               &linear_g, &linear_b);
 
-          Store(linear_r, d, row0 + x);
-          Store(linear_g, d, row1 + x);
-          Store(linear_b, d, row2 + x);
-        }
-      },
-      "OpsinToLinear"));
+      Store(linear_r, d, row0 + x);
+      Store(linear_g, d, row1 + x);
+      Store(linear_b, d, row2 + x);
+    }
+    return true;
+  };
+  JXL_RETURN_IF_ERROR(RunOnPool(pool, 0, inout->ysize(), ThreadPool::NoInit,
+                                process_row, "OpsinToLinear"));
+  return true;
 }
 
 // Same, but not in-place.
-void OpsinToLinear(const Image3F& opsin, const Rect& rect, ThreadPool* pool,
-                   Image3F* JXL_RESTRICT linear,
-                   const OpsinParams& opsin_params) {
-  JXL_ASSERT(SameSize(rect, *linear));
+Status OpsinToLinear(const Image3F& opsin, const Rect& rect, ThreadPool* pool,
+                     Image3F* JXL_RESTRICT linear,
+                     const OpsinParams& opsin_params) {
+  JXL_ENSURE(SameSize(rect, *linear));
   JXL_CHECK_IMAGE_INITIALIZED(opsin, rect);
 
-  JXL_CHECK(RunOnPool(
-      pool, 0, static_cast<int>(rect.ysize()), ThreadPool::NoInit,
-      [&](const uint32_t task, size_t /*thread*/) {
-        const size_t y = static_cast<size_t>(task);
+  const auto process_row = [&](const uint32_t task,
+                               size_t /*thread*/) -> Status {
+    const size_t y = static_cast<size_t>(task);
 
-        // Faster than adding via ByteOffset at end of loop.
-        const float* JXL_RESTRICT row_opsin_0 = rect.ConstPlaneRow(opsin, 0, y);
-        const float* JXL_RESTRICT row_opsin_1 = rect.ConstPlaneRow(opsin, 1, y);
-        const float* JXL_RESTRICT row_opsin_2 = rect.ConstPlaneRow(opsin, 2, y);
-        float* JXL_RESTRICT row_linear_0 = linear->PlaneRow(0, y);
-        float* JXL_RESTRICT row_linear_1 = linear->PlaneRow(1, y);
-        float* JXL_RESTRICT row_linear_2 = linear->PlaneRow(2, y);
+    // Faster than adding via ByteOffset at end of loop.
+    const float* JXL_RESTRICT row_opsin_0 = rect.ConstPlaneRow(opsin, 0, y);
+    const float* JXL_RESTRICT row_opsin_1 = rect.ConstPlaneRow(opsin, 1, y);
+    const float* JXL_RESTRICT row_opsin_2 = rect.ConstPlaneRow(opsin, 2, y);
+    float* JXL_RESTRICT row_linear_0 = linear->PlaneRow(0, y);
+    float* JXL_RESTRICT row_linear_1 = linear->PlaneRow(1, y);
+    float* JXL_RESTRICT row_linear_2 = linear->PlaneRow(2, y);
 
-        const HWY_FULL(float) d;
+    const HWY_FULL(float) d;
 
-        for (size_t x = 0; x < rect.xsize(); x += Lanes(d)) {
-          const auto in_opsin_x = Load(d, row_opsin_0 + x);
-          const auto in_opsin_y = Load(d, row_opsin_1 + x);
-          const auto in_opsin_b = Load(d, row_opsin_2 + x);
-          auto linear_r = Undefined(d);
-          auto linear_g = Undefined(d);
-          auto linear_b = Undefined(d);
-          XybToRgb(d, in_opsin_x, in_opsin_y, in_opsin_b, opsin_params,
-                   &linear_r, &linear_g, &linear_b);
+    for (size_t x = 0; x < rect.xsize(); x += Lanes(d)) {
+      const auto in_opsin_x = Load(d, row_opsin_0 + x);
+      const auto in_opsin_y = Load(d, row_opsin_1 + x);
+      const auto in_opsin_b = Load(d, row_opsin_2 + x);
+      auto linear_r = Undefined(d);
+      auto linear_g = Undefined(d);
+      auto linear_b = Undefined(d);
+      XybToRgb(d, in_opsin_x, in_opsin_y, in_opsin_b, opsin_params, &linear_r,
+               &linear_g, &linear_b);
 
-          Store(linear_r, d, row_linear_0 + x);
-          Store(linear_g, d, row_linear_1 + x);
-          Store(linear_b, d, row_linear_2 + x);
-        }
-      },
-      "OpsinToLinear(Rect)"));
+      Store(linear_r, d, row_linear_0 + x);
+      Store(linear_g, d, row_linear_1 + x);
+      Store(linear_b, d, row_linear_2 + x);
+    }
+    return true;
+  };
+  JXL_RETURN_IF_ERROR(RunOnPool(pool, 0, static_cast<int>(rect.ysize()),
+                                ThreadPool::NoInit, process_row,
+                                "OpsinToLinear(Rect)"));
   JXL_CHECK_IMAGE_INITIALIZED(*linear, rect);
+  return true;
 }
 
 // Transform YCbCr to RGB.
@@ -158,30 +162,30 @@ HWY_AFTER_NAMESPACE();
 namespace jxl {
 
 HWY_EXPORT(OpsinToLinearInplace);
-void OpsinToLinearInplace(Image3F* JXL_RESTRICT inout, ThreadPool* pool,
-                          const OpsinParams& opsin_params) {
+Status OpsinToLinearInplace(Image3F* JXL_RESTRICT inout, ThreadPool* pool,
+                            const OpsinParams& opsin_params) {
   return HWY_DYNAMIC_DISPATCH(OpsinToLinearInplace)(inout, pool, opsin_params);
 }
 
 HWY_EXPORT(OpsinToLinear);
-void OpsinToLinear(const Image3F& opsin, const Rect& rect, ThreadPool* pool,
-                   Image3F* JXL_RESTRICT linear,
-                   const OpsinParams& opsin_params) {
+Status OpsinToLinear(const Image3F& opsin, const Rect& rect, ThreadPool* pool,
+                     Image3F* JXL_RESTRICT linear,
+                     const OpsinParams& opsin_params) {
   return HWY_DYNAMIC_DISPATCH(OpsinToLinear)(opsin, rect, pool, linear,
                                              opsin_params);
 }
 
 HWY_EXPORT(YcbcrToRgb);
 void YcbcrToRgb(const Image3F& ycbcr, Image3F* rgb, const Rect& rect) {
-  return HWY_DYNAMIC_DISPATCH(YcbcrToRgb)(ycbcr, rgb, rect);
+  HWY_DYNAMIC_DISPATCH(YcbcrToRgb)(ycbcr, rgb, rect);
 }
 
 HWY_EXPORT(HasFastXYBTosRGB8);
 bool HasFastXYBTosRGB8() { return HWY_DYNAMIC_DISPATCH(HasFastXYBTosRGB8)(); }
 
 HWY_EXPORT(FastXYBTosRGB8);
-void FastXYBTosRGB8(const float* input[4], uint8_t* output, bool is_rgba,
-                    size_t xsize) {
+Status FastXYBTosRGB8(const float* input[4], uint8_t* output, bool is_rgba,
+                      size_t xsize) {
   return HWY_DYNAMIC_DISPATCH(FastXYBTosRGB8)(input, output, is_rgba, xsize);
 }
 
@@ -218,7 +222,7 @@ Status OutputEncodingInfo::SetFromMetadata(const CodecMetadata& metadata) {
   orig_intensity_target = metadata.m.IntensityTarget();
   desired_intensity_target = orig_intensity_target;
   const auto& im = metadata.transform_data.opsin_inverse_matrix;
-  memcpy(orig_inverse_matrix, im.inverse_matrix, sizeof(orig_inverse_matrix));
+  orig_inverse_matrix = im.inverse_matrix;
   default_transform = im.all_default;
   xyb_encoded = metadata.m.xyb_encoded;
   std::copy(std::begin(im.opsin_biases), std::end(im.opsin_biases),
@@ -258,38 +262,39 @@ Status OutputEncodingInfo::SetColorEncoding(const ColorEncoding& c_desired) {
 
   // Compute the opsin inverse matrix and luminances based on primaries and
   // white point.
-  float inverse_matrix[9];
+  Matrix3x3 inverse_matrix;
   bool inverse_matrix_is_default = default_transform;
-  memcpy(inverse_matrix, orig_inverse_matrix, sizeof(inverse_matrix));
-  constexpr float kSRGBLuminances[3] = {0.2126, 0.7152, 0.0722};
-  memcpy(luminances, kSRGBLuminances, sizeof(luminances));
+  inverse_matrix = orig_inverse_matrix;
+  constexpr Vector3 kSRGBLuminances{0.2126, 0.7152, 0.0722};
+  luminances = kSRGBLuminances;
   if ((c_desired.GetPrimariesType() != Primaries::kSRGB ||
        c_desired.GetWhitePointType() != WhitePoint::kD65) &&
       !c_desired.IsGray()) {
-    float srgb_to_xyzd50[9];
+    Matrix3x3 srgb_to_xyzd50;
     const auto& srgb = ColorEncoding::SRGB(/*is_gray=*/false);
-    PrimariesCIExy p = srgb.GetPrimaries();
+    PrimariesCIExy p;
+    JXL_RETURN_IF_ERROR(srgb.GetPrimaries(p));
     CIExy w = srgb.GetWhitePoint();
-    JXL_CHECK(PrimariesToXYZD50(p.r.x, p.r.y, p.g.x, p.g.y, p.b.x, p.b.y, w.x,
-                                w.y, srgb_to_xyzd50));
-    float original_to_xyz[3][3];
-    p = c_desired.GetPrimaries();
+    JXL_RETURN_IF_ERROR(PrimariesToXYZD50(p.r.x, p.r.y, p.g.x, p.g.y, p.b.x,
+                                          p.b.y, w.x, w.y, srgb_to_xyzd50));
+    Matrix3x3 original_to_xyz;
+    JXL_RETURN_IF_ERROR(c_desired.GetPrimaries(p));
     w = c_desired.GetWhitePoint();
     if (!PrimariesToXYZ(p.r.x, p.r.y, p.g.x, p.g.y, p.b.x, p.b.y, w.x, w.y,
-                        &original_to_xyz[0][0])) {
+                        original_to_xyz)) {
       return JXL_FAILURE("PrimariesToXYZ failed");
     }
-    memcpy(luminances, original_to_xyz[1], sizeof luminances);
+    luminances = original_to_xyz[1];
     if (xyb_encoded) {
-      float adapt_to_d50[9];
+      Matrix3x3 adapt_to_d50;
       if (!AdaptToXYZD50(c_desired.GetWhitePoint().x,
                          c_desired.GetWhitePoint().y, adapt_to_d50)) {
         return JXL_FAILURE("AdaptToXYZD50 failed");
       }
-      float xyzd50_to_original[9];
-      Mul3x3Matrix(adapt_to_d50, &original_to_xyz[0][0], xyzd50_to_original);
+      Matrix3x3 xyzd50_to_original;
+      Mul3x3Matrix(adapt_to_d50, original_to_xyz, xyzd50_to_original);
       JXL_RETURN_IF_ERROR(Inv3x3Matrix(xyzd50_to_original));
-      float srgb_to_original[9];
+      Matrix3x3 srgb_to_original;
       Mul3x3Matrix(xyzd50_to_original, srgb_to_xyzd50, srgb_to_original);
       Mul3x3Matrix(srgb_to_original, orig_inverse_matrix, inverse_matrix);
       inverse_matrix_is_default = false;
@@ -297,12 +302,8 @@ Status OutputEncodingInfo::SetColorEncoding(const ColorEncoding& c_desired) {
   }
 
   if (c_desired.IsGray()) {
-    float tmp_inv_matrix[9];
-    memcpy(tmp_inv_matrix, inverse_matrix, sizeof(inverse_matrix));
-    float srgb_to_luma[9];
-    memcpy(&srgb_to_luma[0], luminances, sizeof(luminances));
-    memcpy(&srgb_to_luma[3], luminances, sizeof(luminances));
-    memcpy(&srgb_to_luma[6], luminances, sizeof(luminances));
+    Matrix3x3 tmp_inv_matrix = inverse_matrix;
+    Matrix3x3 srgb_to_luma{luminances, luminances, luminances};
     Mul3x3Matrix(srgb_to_luma, tmp_inv_matrix, inverse_matrix);
   }
 
