@@ -9,18 +9,19 @@
 #include <cmath>
 #include <limits>
 #include <map>
-#include <memory>
 #include <numeric>
 #include <queue>
 #include <tuple>
 
+#include "lib/jxl/base/status.h"
+
 #undef HWY_TARGET_INCLUDE
 #define HWY_TARGET_INCLUDE "lib/jxl/enc_cluster.cc"
 #include <hwy/foreach_target.h>
 #include <hwy/highway.h>
 
-#include "lib/jxl/ac_context.h"
 #include "lib/jxl/base/fast_math-inl.h"
+#include "lib/jxl/enc_ans.h"
 HWY_BEFORE_NAMESPACE();
 namespace jxl {
 namespace HWY_NAMESPACE {
@@ -81,11 +82,40 @@ float HistogramDistance(const Histogram& a, const Histogram& b) {
   return total_distance - a.entropy_ - b.entropy_;
 }
 
+constexpr const float kInfinity = std::numeric_limits<float>::infinity();
+
+float HistogramKLDivergence(const Histogram& actual, const Histogram& coding) {
+  if (actual.total_count_ == 0) return 0;
+  if (coding.total_count_ == 0) return kInfinity;
+
+  const HWY_CAPPED(float, Histogram::kRounding) df;
+  const HWY_CAPPED(int32_t, Histogram::kRounding) di;
+
+  const auto coding_inv = Set(df, 1.0f / coding.total_count_);
+  auto cost_lanes = Zero(df);
+
+  for (size_t i = 0; i < actual.data_.size(); i += Lanes(di)) {
+    const auto counts = LoadU(di, &actual.data_[i]);
+    const auto coding_counts =
+        coding.data_.size() > i ? LoadU(di, &coding.data_[i]) : Zero(di);
+    const auto coding_probs = Mul(ConvertTo(df, coding_counts), coding_inv);
+    const auto neg_coding_cost = BitCast(
+        df,
+        IfThenZeroElse(Eq(counts, Zero(di)),
+                       IfThenElse(Eq(coding_counts, Zero(di)),
+                                  BitCast(di, Set(df, -kInfinity)),
+                                  BitCast(di, FastLog2f(df, coding_probs)))));
+    cost_lanes = NegMulAdd(ConvertTo(df, counts), neg_coding_cost, cost_lanes);
+  }
+  const float total_cost = GetLane(SumOfLanes(df, cost_lanes));
+  return total_cost - actual.entropy_;
+}
+
 // First step of a k-means clustering with a fancy distance metric.
-void FastClusterHistograms(const std::vector<Histogram>& in,
-                           size_t max_histograms, std::vector<Histogram>* out,
-                           std::vector<uint32_t>* histogram_symbols) {
-  out->clear();
+Status FastClusterHistograms(const std::vector<Histogram>& in,
+                             size_t max_histograms, std::vector<Histogram>* out,
+                             std::vector<uint32_t>* histogram_symbols) {
+  const size_t prev_histograms = out->size();
   out->reserve(max_histograms);
   histogram_symbols->clear();
   histogram_symbols->resize(in.size(), max_histograms);
@@ -104,6 +134,22 @@ void FastClusterHistograms(const std::vector<Histogram>& in,
     }
   }
 
+  if (prev_histograms > 0) {
+    for (size_t j = 0; j < prev_histograms; ++j) {
+      HistogramEntropy((*out)[j]);
+    }
+    for (size_t i = 0; i < in.size(); i++) {
+      if (dists[i] == 0.0f) continue;
+      for (size_t j = 0; j < prev_histograms; ++j) {
+        dists[i] = std::min(HistogramKLDivergence(in[i], (*out)[j]), dists[i]);
+      }
+    }
+    auto max_dist = std::max_element(dists.begin(), dists.end());
+    if (*max_dist > 0.0f) {
+      largest_idx = max_dist - dists.begin();
+    }
+  }
+
   constexpr float kMinDistanceForDistinct = 48.0f;
   while (out->size() < max_histograms) {
     (*histogram_symbols)[largest_idx] = out->size();
@@ -121,18 +167,23 @@ void FastClusterHistograms(const std::vector<Histogram>& in,
   for (size_t i = 0; i < in.size(); i++) {
     if ((*histogram_symbols)[i] != max_histograms) continue;
     size_t best = 0;
-    float best_dist = HistogramDistance(in[i], (*out)[best]);
-    for (size_t j = 1; j < out->size(); j++) {
-      float dist = HistogramDistance(in[i], (*out)[j]);
+    float best_dist = std::numeric_limits<float>::max();
+    for (size_t j = 0; j < out->size(); j++) {
+      float dist = j < prev_histograms ? HistogramKLDivergence(in[i], (*out)[j])
+                                       : HistogramDistance(in[i], (*out)[j]);
       if (dist < best_dist) {
         best = j;
         best_dist = dist;
       }
     }
-    (*out)[best].AddHistogram(in[i]);
-    HistogramEntropy((*out)[best]);
+    JXL_ENSURE(best_dist < std::numeric_limits<float>::max());
+    if (best >= prev_histograms) {
+      (*out)[best].AddHistogram(in[i]);
+      HistogramEntropy((*out)[best]);
+    }
     (*histogram_symbols)[i] = best;
   }
+  return true;
 }
 
 // NOLINTNEXTLINE(google-readability-namespace-comments)
@@ -145,6 +196,10 @@ namespace jxl {
 HWY_EXPORT(FastClusterHistograms);  // Local function
 HWY_EXPORT(HistogramEntropy);       // Local function
 
+StatusOr<float> Histogram::PopulationCost() const {
+  return ANSPopulationCost(data_.data(), data_.size());
+}
+
 float Histogram::ShannonEntropy() const {
   HWY_DYNAMIC_DISPATCH(HistogramEntropy)(*this);
   return entropy_;
@@ -156,11 +211,14 @@ namespace {
 
 // Reorder histograms in *out so that the new symbols in *symbols come in
 // increasing order.
-void HistogramReindex(std::vector<Histogram>* out,
+void HistogramReindex(std::vector<Histogram>* out, size_t prev_histograms,
                       std::vector<uint32_t>* symbols) {
   std::vector<Histogram> tmp(*out);
   std::map<int, int> new_index;
-  int next_index = 0;
+  for (size_t i = 0; i < prev_histograms; ++i) {
+    new_index[i] = i;
+  }
+  int next_index = prev_histograms;
   for (uint32_t symbol : *symbols) {
     if (new_index.find(symbol) == new_index.end()) {
       new_index[symbol] = next_index;
@@ -179,23 +237,26 @@ void HistogramReindex(std::vector<Histogram>* out,
 // Clusters similar histograms in 'in' together, the selected histograms are
 // placed in 'out', and for each index in 'in', *histogram_symbols will
 // indicate which of the 'out' histograms is the best approximation.
-void ClusterHistograms(const HistogramParams params,
-                       const std::vector<Histogram>& in, size_t max_histograms,
-                       std::vector<Histogram>* out,
-                       std::vector<uint32_t>* histogram_symbols) {
+Status ClusterHistograms(const HistogramParams& params,
+                         const std::vector<Histogram>& in,
+                         size_t max_histograms, std::vector<Histogram>* out,
+                         std::vector<uint32_t>* histogram_symbols) {
+  size_t prev_histograms = out->size();
   max_histograms = std::min(max_histograms, params.max_histograms);
   max_histograms = std::min(max_histograms, in.size());
   if (params.clustering == HistogramParams::ClusteringType::kFastest) {
     max_histograms = std::min(max_histograms, static_cast<size_t>(4));
   }
 
-  HWY_DYNAMIC_DISPATCH(FastClusterHistograms)
-  (in, max_histograms, out, histogram_symbols);
+  JXL_RETURN_IF_ERROR(HWY_DYNAMIC_DISPATCH(FastClusterHistograms)(
+      in, prev_histograms + max_histograms, out, histogram_symbols));
 
-  if (params.clustering == HistogramParams::ClusteringType::kBest) {
-    for (size_t i = 0; i < out->size(); i++) {
-      (*out)[i].entropy_ =
-          ANSPopulationCost((*out)[i].data_.data(), (*out)[i].data_.size());
+  if (prev_histograms == 0 &&
+      params.clustering == HistogramParams::ClusteringType::kBest) {
+    for (auto& histo : *out) {
+      JXL_ASSIGN_OR_RETURN(
+          histo.entropy_,
+          ANSPopulationCost(histo.data_.data(), histo.data_.size()));
     }
     uint32_t next_version = 2;
     std::vector<uint32_t> version(out->size(), 1);
@@ -226,8 +287,9 @@ void ClusterHistograms(const HistogramParams params,
         Histogram histo;
         histo.AddHistogram((*out)[i]);
         histo.AddHistogram((*out)[j]);
-        float cost = ANSPopulationCost(histo.data_.data(), histo.data_.size()) -
-                     (*out)[i].entropy_ - (*out)[j].entropy_;
+        JXL_ASSIGN_OR_RETURN(float cost, ANSPopulationCost(histo.data_.data(),
+                                                           histo.data_.size()));
+        cost -= (*out)[i].entropy_ + (*out)[j].entropy_;
         // Avoid enqueueing pairs that are not advantageous to merge.
         if (cost >= 0) continue;
         pairs_to_merge.push(
@@ -247,11 +309,13 @@ void ClusterHistograms(const HistogramParams params,
         continue;
       }
       (*out)[first].AddHistogram((*out)[second]);
-      (*out)[first].entropy_ = ANSPopulationCost((*out)[first].data_.data(),
-                                                 (*out)[first].data_.size());
-      for (size_t i = 0; i < renumbering.size(); i++) {
-        if (renumbering[i] == second) {
-          renumbering[i] = first;
+      JXL_ASSIGN_OR_RETURN(float cost,
+                           ANSPopulationCost((*out)[first].data_.data(),
+                                             (*out)[first].data_.size()));
+      (*out)[first].entropy_ = cost;
+      for (uint32_t& item : renumbering) {
+        if (item == second) {
+          item = first;
         }
       }
       version[second] = 0;
@@ -262,8 +326,9 @@ void ClusterHistograms(const HistogramParams params,
         Histogram histo;
         histo.AddHistogram((*out)[first]);
         histo.AddHistogram((*out)[j]);
-        float cost = ANSPopulationCost(histo.data_.data(), histo.data_.size()) -
-                     (*out)[first].entropy_ - (*out)[j].entropy_;
+        JXL_ASSIGN_OR_RETURN(float cost, ANSPopulationCost(histo.data_.data(),
+                                                           histo.data_.size()));
+        cost -= (*out)[first].entropy_ + (*out)[j].entropy_;
         // Avoid enqueueing pairs that are not advantageous to merge.
         if (cost >= 0) continue;
         pairs_to_merge.push(
@@ -279,14 +344,14 @@ void ClusterHistograms(const HistogramParams params,
       reverse_renumbering[i] = num_alive - 1;
     }
     out->resize(num_alive);
-    for (size_t i = 0; i < histogram_symbols->size(); i++) {
-      (*histogram_symbols)[i] =
-          reverse_renumbering[renumbering[(*histogram_symbols)[i]]];
+    for (uint32_t& item : *histogram_symbols) {
+      item = reverse_renumbering[renumbering[item]];
     }
   }
 
   // Convert the context map to a canonical form.
-  HistogramReindex(out, histogram_symbols);
+  HistogramReindex(out, prev_histograms, histogram_symbols);
+  return true;
 }
 
 }  // namespace jxl
