@@ -5,11 +5,12 @@
 
 #include "lib/jxl/enc_detect_dots.h"
 
-#include <stdint.h>
+#include <jxl/memory_manager.h>
 
 #include <algorithm>
 #include <array>
 #include <cmath>
+#include <cstdint>
 #include <cstdio>
 #include <utility>
 #include <vector>
@@ -19,28 +20,22 @@
 #include <hwy/foreach_target.h>
 #include <hwy/highway.h>
 
+#include "lib/jxl/base/common.h"
 #include "lib/jxl/base/compiler_specific.h"
 #include "lib/jxl/base/data_parallel.h"
 #include "lib/jxl/base/printf_macros.h"
-#include "lib/jxl/base/profiler.h"
+#include "lib/jxl/base/rect.h"
 #include "lib/jxl/base/status.h"
-#include "lib/jxl/codec_in_out.h"
-#include "lib/jxl/common.h"
 #include "lib/jxl/convolve.h"
+#include "lib/jxl/enc_linalg.h"
 #include "lib/jxl/image.h"
 #include "lib/jxl/image_ops.h"
-#include "lib/jxl/linalg.h"
-#include "lib/jxl/optimize.h"
 
 // Set JXL_DEBUG_DOT_DETECT to 1 to enable debugging.
 #ifndef JXL_DEBUG_DOT_DETECT
 #define JXL_DEBUG_DOT_DETECT 0
 #endif
 
-#if JXL_DEBUG_DOT_DETECT
-#include "lib/jxl/aux_out.h"
-#endif
-
 HWY_BEFORE_NAMESPACE();
 namespace jxl {
 namespace HWY_NAMESPACE {
@@ -50,39 +45,42 @@ using hwy::HWY_NAMESPACE::Add;
 using hwy::HWY_NAMESPACE::Mul;
 using hwy::HWY_NAMESPACE::Sub;
 
-ImageF SumOfSquareDifferences(const Image3F& forig, const Image3F& smooth,
-                              ThreadPool* pool) {
+StatusOr<ImageF> SumOfSquareDifferences(const Image3F& forig,
+                                        const Image3F& smooth,
+                                        ThreadPool* pool) {
   const HWY_FULL(float) d;
   const auto color_coef0 = Set(d, 0.0f);
   const auto color_coef1 = Set(d, 10.0f);
   const auto color_coef2 = Set(d, 0.0f);
+  JxlMemoryManager* memory_manager = forig.memory_manager();
 
-  ImageF sum_of_squares(forig.xsize(), forig.ysize());
-  JXL_CHECK(RunOnPool(
-      pool, 0, forig.ysize(), ThreadPool::NoInit,
-      [&](const uint32_t task, size_t thread) {
-        const size_t y = static_cast<size_t>(task);
-        const float* JXL_RESTRICT orig_row0 = forig.Plane(0).ConstRow(y);
-        const float* JXL_RESTRICT orig_row1 = forig.Plane(1).ConstRow(y);
-        const float* JXL_RESTRICT orig_row2 = forig.Plane(2).ConstRow(y);
-        const float* JXL_RESTRICT smooth_row0 = smooth.Plane(0).ConstRow(y);
-        const float* JXL_RESTRICT smooth_row1 = smooth.Plane(1).ConstRow(y);
-        const float* JXL_RESTRICT smooth_row2 = smooth.Plane(2).ConstRow(y);
-        float* JXL_RESTRICT sos_row = sum_of_squares.Row(y);
+  JXL_ASSIGN_OR_RETURN(
+      ImageF sum_of_squares,
+      ImageF::Create(memory_manager, forig.xsize(), forig.ysize()));
+  const auto process_row = [&](const uint32_t task, size_t thread) -> Status {
+    const size_t y = static_cast<size_t>(task);
+    const float* JXL_RESTRICT orig_row0 = forig.Plane(0).ConstRow(y);
+    const float* JXL_RESTRICT orig_row1 = forig.Plane(1).ConstRow(y);
+    const float* JXL_RESTRICT orig_row2 = forig.Plane(2).ConstRow(y);
+    const float* JXL_RESTRICT smooth_row0 = smooth.Plane(0).ConstRow(y);
+    const float* JXL_RESTRICT smooth_row1 = smooth.Plane(1).ConstRow(y);
+    const float* JXL_RESTRICT smooth_row2 = smooth.Plane(2).ConstRow(y);
+    float* JXL_RESTRICT sos_row = sum_of_squares.Row(y);
 
-        for (size_t x = 0; x < forig.xsize(); x += Lanes(d)) {
-          auto v0 = Sub(Load(d, orig_row0 + x), Load(d, smooth_row0 + x));
-          auto v1 = Sub(Load(d, orig_row1 + x), Load(d, smooth_row1 + x));
-          auto v2 = Sub(Load(d, orig_row2 + x), Load(d, smooth_row2 + x));
-          v0 = Mul(Mul(v0, v0), color_coef0);
-          v1 = Mul(Mul(v1, v1), color_coef1);
-          v2 = Mul(Mul(v2, v2), color_coef2);
-          const auto sos =
-              Add(v0, Add(v1, v2));  // weighted sum of square diffs
-          Store(sos, d, sos_row + x);
-        }
-      },
-      "ComputeEnergyImage"));
+    for (size_t x = 0; x < forig.xsize(); x += Lanes(d)) {
+      auto v0 = Sub(Load(d, orig_row0 + x), Load(d, smooth_row0 + x));
+      auto v1 = Sub(Load(d, orig_row1 + x), Load(d, smooth_row1 + x));
+      auto v2 = Sub(Load(d, orig_row2 + x), Load(d, smooth_row2 + x));
+      v0 = Mul(Mul(v0, v0), color_coef0);
+      v1 = Mul(Mul(v1, v1), color_coef1);
+      v2 = Mul(Mul(v2, v2), color_coef2);
+      const auto sos = Add(v0, Add(v1, v2));  // weighted sum of square diffs
+      Store(sos, d, sos_row + x);
+    }
+    return true;
+  };
+  JXL_RETURN_IF_ERROR(RunOnPool(pool, 0, forig.ysize(), ThreadPool::NoInit,
+                                process_row, "ComputeEnergyImage"));
   return sum_of_squares;
 }
 
@@ -148,13 +146,15 @@ const WeightsSeparable5& WeightsSeparable5Gaussian3() {
   return weights;
 }
 
-ImageF ComputeEnergyImage(const Image3F& orig, Image3F* smooth,
-                          ThreadPool* pool) {
-  PROFILER_FUNC;
-
+StatusOr<ImageF> ComputeEnergyImage(const Image3F& orig, Image3F* smooth,
+                                    ThreadPool* pool) {
+  JxlMemoryManager* memory_manager = orig.memory_manager();
   // Prepare guidance images for dot selection.
-  Image3F forig(orig.xsize(), orig.ysize());
-  *smooth = Image3F(orig.xsize(), orig.ysize());
+  JXL_ASSIGN_OR_RETURN(
+      Image3F forig,
+      Image3F::Create(memory_manager, orig.xsize(), orig.ysize()));
+  JXL_ASSIGN_OR_RETURN(
+      *smooth, Image3F::Create(memory_manager, orig.xsize(), orig.ysize()));
   Rect rect(orig);
 
   const auto& weights1 = WeightsSeparable5Gaussian0_65();
@@ -162,18 +162,14 @@ ImageF ComputeEnergyImage(const Image3F& orig, Image3F* smooth,
 
   for (size_t c = 0; c < 3; ++c) {
     // Use forig as temporary storage to reduce memory and keep it warmer.
-    Separable5(orig.Plane(c), rect, weights3, pool, &forig.Plane(c));
-    Separable5(forig.Plane(c), rect, weights3, pool, &smooth->Plane(c));
-    Separable5(orig.Plane(c), rect, weights1, pool, &forig.Plane(c));
+    JXL_RETURN_IF_ERROR(
+        Separable5(orig.Plane(c), rect, weights3, pool, &forig.Plane(c)));
+    JXL_RETURN_IF_ERROR(
+        Separable5(forig.Plane(c), rect, weights3, pool, &smooth->Plane(c)));
+    JXL_RETURN_IF_ERROR(
+        Separable5(orig.Plane(c), rect, weights1, pool, &forig.Plane(c)));
   }
 
-#if JXL_DEBUG_DOT_DETECT
-  AuxOut aux;
-  aux.debug_prefix = "/tmp/sebastian/";
-  aux.DumpImage("filtered", forig);
-  aux.DumpImage("sm", *smooth);
-#endif
-
   return HWY_DYNAMIC_DISPATCH(SumOfSquareDifferences)(forig, *smooth, pool);
 }
 
@@ -191,9 +187,8 @@ const size_t kMaxCCSize = 1000;
 
 // Extracts a connected component from a Binary image where seed is part
 // of the component
-bool ExtractComponent(ImageF* img, std::vector<Pixel>* pixels,
+bool ExtractComponent(const Rect& rect, ImageF* img, std::vector<Pixel>* pixels,
                       const Pixel& seed, double threshold) {
-  PROFILER_FUNC;
   static const std::vector<Pixel> neighbors{{1, -1}, {1, 0},   {1, 1},  {0, -1},
                                             {0, 1},  {-1, -1}, {-1, 1}, {1, 0}};
   std::vector<Pixel> q{seed};
@@ -204,9 +199,9 @@ bool ExtractComponent(ImageF* img, std::vector<Pixel>* pixels,
     if (pixels->size() > kMaxCCSize) return false;
     for (const Pixel& delta : neighbors) {
       Pixel child = current + delta;
-      if (child.x >= 0 && static_cast<size_t>(child.x) < img->xsize() &&
-          child.y >= 0 && static_cast<size_t>(child.y) < img->ysize()) {
-        float* value = &img->Row(child.y)[child.x];
+      if (child.x >= 0 && static_cast<size_t>(child.x) < rect.xsize() &&
+          child.y >= 0 && static_cast<size_t>(child.y) < rect.ysize()) {
+        float* value = &rect.Row(img, child.y)[child.x];
         if (*value > threshold) {
           *value = 0.0;
           q.push_back(child);
@@ -237,8 +232,7 @@ struct ConnectedComponent {
   float score;
   Pixel mode;
 
-  void CompStats(const ImageF& energy, int extra) {
-    PROFILER_FUNC;
+  void CompStats(const ImageF& energy, const Rect& rect, int extra) {
     maxEnergy = 0.0;
     meanEnergy = 0.0;
     varEnergy = 0.0;
@@ -251,12 +245,12 @@ struct ConnectedComponent {
     for (int sy = -extra; sy < (static_cast<int>(bounds.ysize()) + extra);
          sy++) {
       int y = sy + static_cast<int>(bounds.y0());
-      if (y < 0 || static_cast<size_t>(y) >= energy.ysize()) continue;
-      const float* JXL_RESTRICT erow = energy.ConstRow(y);
+      if (y < 0 || static_cast<size_t>(y) >= rect.ysize()) continue;
+      const float* JXL_RESTRICT erow = rect.ConstRow(energy, y);
       for (int sx = -extra; sx < (static_cast<int>(bounds.xsize()) + extra);
            sx++) {
         int x = sx + static_cast<int>(bounds.x0());
-        if (x < 0 || static_cast<size_t>(x) >= energy.xsize()) continue;
+        if (x < 0 || static_cast<size_t>(x) >= rect.xsize()) continue;
         if (erow[x] > maxEnergy) {
           maxEnergy = erow[x];
           mode.x = x;
@@ -282,9 +276,11 @@ struct ConnectedComponent {
 };
 
 Rect BoundingRectangle(const std::vector<Pixel>& pixels) {
-  PROFILER_FUNC;
-  JXL_ASSERT(!pixels.empty());
-  int low_x, high_x, low_y, high_y;
+  JXL_DASSERT(!pixels.empty());
+  int low_x;
+  int high_x;
+  int low_y;
+  int high_y;
   low_x = high_x = pixels[0].x;
   low_y = high_y = pixels[0].y;
   for (const Pixel& p : pixels) {
@@ -296,22 +292,26 @@ Rect BoundingRectangle(const std::vector<Pixel>& pixels) {
   return Rect(low_x, low_y, high_x - low_x + 1, high_y - low_y + 1);
 }
 
-std::vector<ConnectedComponent> FindCC(const ImageF& energy, double t_low,
-                                       double t_high, uint32_t maxWindow,
-                                       double minScore) {
-  PROFILER_FUNC;
+StatusOr<std::vector<ConnectedComponent>> FindCC(const ImageF& energy,
+                                                 const Rect& rect, double t_low,
+                                                 double t_high,
+                                                 uint32_t maxWindow,
+                                                 double minScore) {
   const int kExtraRect = 4;
-  ImageF img = CopyImage(energy);
+  JxlMemoryManager* memory_manager = energy.memory_manager();
+  JXL_ASSIGN_OR_RETURN(
+      ImageF img,
+      ImageF::Create(memory_manager, energy.xsize(), energy.ysize()));
+  JXL_RETURN_IF_ERROR(CopyImageTo(energy, &img));
   std::vector<ConnectedComponent> ans;
-  for (size_t y = 0; y < img.ysize(); y++) {
-    float* JXL_RESTRICT row = img.Row(y);
-    for (size_t x = 0; x < img.xsize(); x++) {
+  for (size_t y = 0; y < rect.ysize(); y++) {
+    float* JXL_RESTRICT row = rect.Row(&img, y);
+    for (size_t x = 0; x < rect.xsize(); x++) {
       if (row[x] > t_high) {
         std::vector<Pixel> pixels;
         row[x] = 0.0;
-        bool success = ExtractComponent(
-            &img, &pixels, Pixel{static_cast<int>(x), static_cast<int>(y)},
-            t_low);
+        Pixel seed = Pixel{static_cast<int>(x), static_cast<int>(y)};
+        bool success = ExtractComponent(rect, &img, &pixels, seed, t_low);
         if (!success) continue;
 #if JXL_DEBUG_DOT_DETECT
         for (size_t i = 0; i < pixels.size(); i++) {
@@ -322,7 +322,7 @@ std::vector<ConnectedComponent> FindCC(const ImageF& energy, double t_low,
         Rect bounds = BoundingRectangle(pixels);
         if (bounds.xsize() < maxWindow && bounds.ysize() < maxWindow) {
           ConnectedComponent cc{bounds, std::move(pixels)};
-          cc.CompStats(energy, kExtraRect);
+          cc.CompStats(energy, rect, kExtraRect);
           if (cc.score < minScore) continue;
           JXL_DEBUG(JXL_DEBUG_DOT_DETECT,
                     "cc mode: (%d,%d), max: %f, bgMean: %f bgVar: "
@@ -338,16 +338,17 @@ std::vector<ConnectedComponent> FindCC(const ImageF& energy, double t_low,
   return ans;
 }
 
-// TODO (sggonzalez): Adapt this function for the different color spaces or
+// TODO(sggonzalez): Adapt this function for the different color spaces or
 // remove it if the color space with the best performance does not need it
 void ComputeDotLosses(GaussianEllipse* ellipse, const ConnectedComponent& cc,
-                      const Image3F& img, const Image3F& background) {
-  PROFILER_FUNC;
+                      const Rect& rect, const Image3F& img,
+                      const Image3F& background) {
   const int rectBounds = 2;
   const double kIntensityR = 0.0;   // 0.015;
   const double kSigmaR = 0.0;       // 0.01;
   const double kZeroEpsilon = 0.1;  // Tolerance to consider a value negative
-  double ct = cos(ellipse->angle), st = sin(ellipse->angle);
+  double ct = cos(ellipse->angle);
+  double st = sin(ellipse->angle);
   const std::array<double, 3> channelGains{{1.0, 1.0, 1.0}};
   int N = 0;
   ellipse->l1_loss = 0.0;
@@ -361,15 +362,15 @@ void ComputeDotLosses(GaussianEllipse* ellipse, const ConnectedComponent& cc,
     for (int sy = -rectBounds;
          sy < (static_cast<int>(cc.bounds.ysize()) + rectBounds); sy++) {
       int y = sy + cc.bounds.y0();
-      if (y < 0 || static_cast<size_t>(y) >= img.ysize()) continue;
-      const float* JXL_RESTRICT row = img.ConstPlaneRow(c, y);
+      if (y < 0 || static_cast<size_t>(y) >= rect.ysize()) continue;
+      const float* JXL_RESTRICT row = rect.ConstPlaneRow(img, c, y);
       // bgrow is only used if kOptimizeBackground is false.
       // NOLINTNEXTLINE(clang-analyzer-deadcode.DeadStores)
-      const float* JXL_RESTRICT bgrow = background.ConstPlaneRow(c, y);
+      const float* JXL_RESTRICT bgrow = rect.ConstPlaneRow(background, c, y);
       for (int sx = -rectBounds;
            sx < (static_cast<int>(cc.bounds.xsize()) + rectBounds); sx++) {
         int x = sx + cc.bounds.x0();
-        if (x < 0 || static_cast<size_t>(x) >= img.xsize()) continue;
+        if (x < 0 || static_cast<size_t>(x) >= rect.xsize()) continue;
         double target = row[x];
         double dotDelta = DotGaussianModel(
             x - ellipse->x, y - ellipse->y, ct, st, ellipse->sigma_x,
@@ -404,10 +405,9 @@ void ComputeDotLosses(GaussianEllipse* ellipse, const ConnectedComponent& cc,
   ellipse->ridge_loss = ellipse->l2_loss + ridgeTerm;
 }
 
-GaussianEllipse FitGaussianFast(const ConnectedComponent& cc,
-                                const ImageF& energy, const Image3F& img,
-                                const Image3F& background) {
-  PROFILER_FUNC;
+StatusOr<GaussianEllipse> FitGaussianFast(const ConnectedComponent& cc,
+                                          const Rect& rect, const Image3F& img,
+                                          const Image3F& background) {
   constexpr bool leastSqIntensity = true;
   constexpr double kEpsilon = 1e-6;
   GaussianEllipse ans;
@@ -425,18 +425,18 @@ GaussianEllipse FitGaussianFast(const ConnectedComponent& cc,
             "%" PRIuS " %" PRIuS " %" PRIuS " %" PRIuS "\n", cc.bounds.x0(),
             cc.bounds.y0(), cc.bounds.xsize(), cc.bounds.ysize());
   for (int c = 0; c < 3; c++) {
-    color[c] = img.ConstPlaneRow(c, cc.mode.y)[cc.mode.x] -
-               background.ConstPlaneRow(c, cc.mode.y)[cc.mode.x];
+    color[c] = rect.ConstPlaneRow(img, c, cc.mode.y)[cc.mode.x] -
+               rect.ConstPlaneRow(background, c, cc.mode.y)[cc.mode.x];
   }
   double sign = (color[1] > 0) ? 1 : -1;
   for (int sy = -kRectBounds; sy <= kRectBounds; sy++) {
     int y = sy + cc.mode.y;
-    if (y < 0 || static_cast<size_t>(y) >= energy.ysize()) continue;
-    const float* JXL_RESTRICT row = img.ConstPlaneRow(1, y);
-    const float* JXL_RESTRICT bgrow = background.ConstPlaneRow(1, y);
+    if (y < 0 || static_cast<size_t>(y) >= rect.ysize()) continue;
+    const float* JXL_RESTRICT row = rect.ConstPlaneRow(img, 1, y);
+    const float* JXL_RESTRICT bgrow = rect.ConstPlaneRow(background, 1, y);
     for (int sx = -kRectBounds; sx <= kRectBounds; sx++) {
       int x = sx + cc.mode.x;
-      if (x < 0 || static_cast<size_t>(x) >= energy.xsize()) continue;
+      if (x < 0 || static_cast<size_t>(x) >= rect.xsize()) continue;
       double w = std::max(kEpsilon, sign * (row[x] - bgrow[x]));
       sum += w;
 
@@ -446,12 +446,12 @@ GaussianEllipse FitGaussianFast(const ConnectedComponent& cc,
       m2[1] += w * x * y;
       m2[2] += w * y * y;
       for (int c = 0; c < 3; c++) {
-        bgColor[c] += background.ConstPlaneRow(c, y)[x];
+        bgColor[c] += rect.ConstPlaneRow(background, c, y)[x];
       }
       N++;
     }
   }
-  JXL_CHECK(N > 0);
+  JXL_ENSURE(N > 0);
 
   for (int i = 0; i < 3; i++) {
     m1[i] /= sum;
@@ -470,14 +470,16 @@ GaussianEllipse FitGaussianFast(const ConnectedComponent& cc,
     ans.intensity[j] = kScaleMult[j] * color[j];
   }
 
-  ImageD Sigma(2, 2), D(1, 2), U(2, 2);
-  Sigma.Row(0)[0] = m2[0] - m1[0] * m1[0];
-  Sigma.Row(1)[1] = m2[2] - m1[1] * m1[1];
-  Sigma.Row(0)[1] = Sigma.Row(1)[0] = m2[1] - m1[0] * m1[1];
-  ConvertToDiagonal(Sigma, &D, &U);
-  const double* JXL_RESTRICT d = D.ConstRow(0);
-  const double* JXL_RESTRICT u = U.ConstRow(1);
-  int p1 = 0, p2 = 1;
+  Matrix2x2 Sigma;
+  Vector2 d;
+  Matrix2x2 U;
+  Sigma[0][0] = m2[0] - m1[0] * m1[0];
+  Sigma[1][1] = m2[2] - m1[1] * m1[1];
+  Sigma[0][1] = Sigma[1][0] = m2[1] - m1[0] * m1[1];
+  ConvertToDiagonal(Sigma, d, U);
+  Vector2& u = U[1];
+  int p1 = 0;
+  int p2 = 1;
   if (d[0] < d[1]) std::swap(p1, p2);
   ans.sigma_x = kSigmaMult * d[p1];
   ans.sigma_y = kSigmaMult * d[p2];
@@ -486,7 +488,8 @@ GaussianEllipse FitGaussianFast(const ConnectedComponent& cc,
   ans.bgColor = bgColor;
   if (leastSqIntensity) {
     GaussianEllipse* ellipse = &ans;
-    double ct = cos(ans.angle), st = sin(ans.angle);
+    double ct = cos(ans.angle);
+    double st = sin(ans.angle);
     // Estimate intensity with least squares (fixed background)
     for (int c = 0; c < 3; c++) {
       double gg = 0.0;
@@ -494,11 +497,11 @@ GaussianEllipse FitGaussianFast(const ConnectedComponent& cc,
       int yc = static_cast<int>(cc.mode.y);
       int xc = static_cast<int>(cc.mode.x);
       for (int y = yc - kRectBounds; y <= yc + kRectBounds; y++) {
-        if (y < 0 || static_cast<size_t>(y) >= img.ysize()) continue;
-        const float* JXL_RESTRICT row = img.ConstPlaneRow(c, y);
-        const float* JXL_RESTRICT bgrow = background.ConstPlaneRow(c, y);
+        if (y < 0 || static_cast<size_t>(y) >= rect.ysize()) continue;
+        const float* JXL_RESTRICT row = rect.ConstPlaneRow(img, c, y);
+        const float* JXL_RESTRICT bgrow = rect.ConstPlaneRow(background, c, y);
         for (int x = xc - kRectBounds; x <= xc + kRectBounds; x++) {
-          if (x < 0 || static_cast<size_t>(x) >= img.xsize()) continue;
+          if (x < 0 || static_cast<size_t>(x) >= rect.xsize()) continue;
           double target = row[x] - bgrow[x];
           double gaussian =
               DotGaussianModel(x - ellipse->x, y - ellipse->y, ct, st,
@@ -510,13 +513,15 @@ GaussianEllipse FitGaussianFast(const ConnectedComponent& cc,
       ans.intensity[c] = gd / (gg + 1e-6);  // Regularized least squares
     }
   }
-  ComputeDotLosses(&ans, cc, img, background);
+  ComputeDotLosses(&ans, cc, rect, img, background);
   return ans;
 }
 
-GaussianEllipse FitGaussian(const ConnectedComponent& cc, const ImageF& energy,
-                            const Image3F& img, const Image3F& background) {
-  auto ellipse = FitGaussianFast(cc, energy, img, background);
+StatusOr<GaussianEllipse> FitGaussian(const ConnectedComponent& cc,
+                                      const Rect& rect, const Image3F& img,
+                                      const Image3F& background) {
+  JXL_ASSIGN_OR_RETURN(GaussianEllipse ellipse,
+                       FitGaussianFast(cc, rect, img, background));
   if (ellipse.sigma_x < ellipse.sigma_y) {
     std::swap(ellipse.sigma_x, ellipse.sigma_y);
     ellipse.angle += kPi / 2.0;
@@ -525,8 +530,8 @@ GaussianEllipse FitGaussian(const ConnectedComponent& cc, const ImageF& energy,
   if (fabs(ellipse.angle - kPi) < 1e-6 || fabs(ellipse.angle) < 1e-6) {
     ellipse.angle = 0.0;
   }
-  JXL_CHECK(ellipse.angle >= 0 && ellipse.angle <= kPi &&
-            ellipse.sigma_x >= ellipse.sigma_y);
+  JXL_ENSURE(ellipse.angle >= 0 && ellipse.angle <= kPi &&
+             ellipse.sigma_x >= ellipse.sigma_y);
   JXL_DEBUG(JXL_DEBUG_DOT_DETECT,
             "Ellipse mu=(%lf,%lf) sigma=(%lf,%lf) angle=%lf "
             "intensity=(%lf,%lf,%lf) bg=(%lf,%lf,%lf) l2_loss=%lf "
@@ -542,21 +547,18 @@ GaussianEllipse FitGaussian(const ConnectedComponent& cc, const ImageF& energy,
 
 }  // namespace
 
-std::vector<PatchInfo> DetectGaussianEllipses(
-    const Image3F& opsin, const GaussianDetectParams& params,
+StatusOr<std::vector<PatchInfo>> DetectGaussianEllipses(
+    const Image3F& opsin, const Rect& rect, const GaussianDetectParams& params,
     const EllipseQuantParams& qParams, ThreadPool* pool) {
-  PROFILER_FUNC;
+  JxlMemoryManager* memory_manager = opsin.memory_manager();
   std::vector<PatchInfo> dots;
-  Image3F smooth(opsin.xsize(), opsin.ysize());
-  ImageF energy = ComputeEnergyImage(opsin, &smooth, pool);
-#if JXL_DEBUG_DOT_DETECT
-  AuxOut aux;
-  aux.debug_prefix = "/tmp/sebastian/";
-  aux.DumpXybImage("smooth", smooth);
-  aux.DumpPlaneNormalized("energy", energy);
-#endif  // JXL_DEBUG_DOT_DETECT
-  std::vector<ConnectedComponent> components = FindCC(
-      energy, params.t_low, params.t_high, params.maxWinSize, params.minScore);
+  JXL_ASSIGN_OR_RETURN(
+      Image3F smooth,
+      Image3F::Create(memory_manager, opsin.xsize(), opsin.ysize()));
+  JXL_ASSIGN_OR_RETURN(ImageF energy, ComputeEnergyImage(opsin, &smooth, pool));
+  JXL_ASSIGN_OR_RETURN(std::vector<ConnectedComponent> components,
+                       FindCC(energy, rect, params.t_low, params.t_high,
+                              params.maxWinSize, params.minScore));
   size_t numCC =
       std::min(params.maxCC, (components.size() * params.percCC) / 100);
   if (components.size() > numCC) {
@@ -568,11 +570,12 @@ std::vector<PatchInfo> DetectGaussianEllipses(
     components.erase(components.begin() + numCC, components.end());
   }
   for (const auto& cc : components) {
-    GaussianEllipse ellipse = FitGaussian(cc, energy, opsin, smooth);
+    JXL_ASSIGN_OR_RETURN(GaussianEllipse ellipse,
+                         FitGaussian(cc, rect, opsin, smooth));
     if (ellipse.x < 0.0 ||
-        std::ceil(ellipse.x) >= static_cast<double>(opsin.xsize()) ||
+        std::ceil(ellipse.x) >= static_cast<double>(rect.xsize()) ||
         ellipse.y < 0.0 ||
-        std::ceil(ellipse.y) >= static_cast<double>(opsin.ysize())) {
+        std::ceil(ellipse.y) >= static_cast<double>(rect.ysize())) {
       continue;
     }
     if (ellipse.neg_pixels > params.maxNegPixels) continue;
@@ -600,26 +603,13 @@ std::vector<PatchInfo> DetectGaussianEllipses(
         for (size_t x = 0; x < patch.xsize; x++) {
           for (size_t c = 0; c < 3; c++) {
             patch.fpixels[c][y * patch.xsize + x] =
-                opsin.ConstPlaneRow(c, y0 + y)[x0 + x] -
-                smooth.ConstPlaneRow(c, y0 + y)[x0 + x];
+                rect.ConstPlaneRow(opsin, c, y0 + y)[x0 + x] -
+                rect.ConstPlaneRow(smooth, c, y0 + y)[x0 + x];
           }
         }
       }
     }
   }
-#if JXL_DEBUG_DOT_DETECT
-  JXL_DEBUG(JXL_DEBUG_DOT_DETECT, "Candidates: %" PRIuS ", Dots: %" PRIuS "\n",
-            components.size(), dots.size());
-  ApplyGaussianEllipses(&smooth, dots, 1.0);
-  aux.DumpXybImage("draw", smooth);
-  ApplyGaussianEllipses(&smooth, dots, -1.0);
-
-  auto qdots = QuantizeGaussianEllipses(dots, qParams);
-  auto deq = DequantizeGaussianEllipses(qdots, qParams);
-  ApplyGaussianEllipses(&smooth, deq, 1.0);
-  aux.DumpXybImage("qdraw", smooth);
-  ApplyGaussianEllipses(&smooth, deq, -1.0);
-#endif  // JXL_DEBUG_DOT_DETECT
   return dots;
 }
 
