@@ -9,27 +9,21 @@
 // Dots are stored in a dictionary to avoid storing similar dots multiple
 // times.
 
-#include <stddef.h>
-
 #include <vector>
 
-#include "lib/jxl/aux_out.h"
-#include "lib/jxl/aux_out_fwd.h"
+#include "lib/jxl/base/data_parallel.h"
+#include "lib/jxl/base/rect.h"
 #include "lib/jxl/base/status.h"
 #include "lib/jxl/chroma_from_luma.h"
-#include "lib/jxl/dec_bit_reader.h"
-#include "lib/jxl/dec_patch_dictionary.h"
-#include "lib/jxl/enc_bit_writer.h"
 #include "lib/jxl/enc_params.h"
 #include "lib/jxl/enc_patch_dictionary.h"
 #include "lib/jxl/image.h"
 
 namespace jxl {
 
-std::vector<PatchInfo> FindDotDictionary(const CompressParams& cparams,
-                                         const Image3F& opsin,
-                                         const ColorCorrelationMap& cmap,
-                                         ThreadPool* pool);
+StatusOr<std::vector<PatchInfo>> FindDotDictionary(
+    const CompressParams& cparams, const Image3F& opsin, const Rect& rect,
+    const ColorCorrelation& color_correlation, ThreadPool* pool);
 
 }  // namespace jxl
 
