@@ -6,8 +6,9 @@
 #ifndef LIB_JXL_ENC_GROUP_H_
 #define LIB_JXL_ENC_GROUP_H_
 
-#include <stddef.h>
+#include <cstddef>
 
+#include "lib/jxl/base/rect.h"
 #include "lib/jxl/base/status.h"
 #include "lib/jxl/enc_bit_writer.h"
 #include "lib/jxl/image.h"
@@ -18,8 +19,8 @@ struct AuxOut;
 struct PassesEncoderState;
 
 // Fills DC
-void ComputeCoefficients(size_t group_idx, PassesEncoderState* enc_state,
-                         const Image3F& opsin, const Rect& rect, Image3F* dc);
+Status ComputeCoefficients(size_t group_idx, PassesEncoderState* enc_state,
+                           const Image3F& opsin, const Rect& rect, Image3F* dc);
 
 Status EncodeGroupTokenizedCoefficients(size_t group_idx, size_t pass_idx,
                                         size_t histogram_idx,
