@@ -8,9 +8,8 @@
 
 // Noise parameter estimation.
 
-#include <stddef.h>
+#include <cstdint>
 
-#include "lib/jxl/aux_out_fwd.h"
 #include "lib/jxl/base/status.h"
 #include "lib/jxl/enc_bit_writer.h"
 #include "lib/jxl/image.h"
@@ -18,6 +17,9 @@
 
 namespace jxl {
 
+struct AuxOut;
+enum class LayerType : uint8_t;
+
 // Get parameters of the noise for NoiseParams model
 // Returns whether a valid noise model (with HasAny()) is set.
 Status GetNoiseParameter(const Image3F& opsin, NoiseParams* noise_params,
@@ -25,8 +27,8 @@ Status GetNoiseParameter(const Image3F& opsin, NoiseParams* noise_params,
 
 // Does not write anything if `noise_params` are empty. Otherwise, caller must
 // set FrameHeader.flags.kNoise.
-void EncodeNoise(const NoiseParams& noise_params, BitWriter* writer,
-                 size_t layer, AuxOut* aux_out);
+Status EncodeNoise(const NoiseParams& noise_params, BitWriter* writer,
+                   LayerType layer, AuxOut* aux_out);
 
 }  // namespace jxl
 
