@@ -6,31 +6,22 @@
 #ifndef LIB_JXL_ENC_SPLINES_H_
 #define LIB_JXL_ENC_SPLINES_H_
 
-#include <stddef.h>
-#include <stdint.h>
+#include <cstdint>
 
-#include <utility>
-#include <vector>
-
-#include "lib/jxl/ans_params.h"
-#include "lib/jxl/aux_out.h"
-#include "lib/jxl/aux_out_fwd.h"
 #include "lib/jxl/base/status.h"
-#include "lib/jxl/chroma_from_luma.h"
-#include "lib/jxl/dec_ans.h"
-#include "lib/jxl/dec_bit_reader.h"
-#include "lib/jxl/enc_ans.h"
+#include "lib/jxl/enc_ans_params.h"
 #include "lib/jxl/enc_bit_writer.h"
-#include "lib/jxl/entropy_coder.h"
 #include "lib/jxl/image.h"
 #include "lib/jxl/splines.h"
 
 namespace jxl {
 
+struct AuxOut;
+enum class LayerType : uint8_t;
+
 // Only call if splines.HasAny().
-void EncodeSplines(const Splines& splines, BitWriter* writer,
-                   const size_t layer, const HistogramParams& histogram_params,
-                   AuxOut* aux_out);
+Status EncodeSplines(const Splines& splines, BitWriter* writer, LayerType layer,
+                     const HistogramParams& histogram_params, AuxOut* aux_out);
 
 Splines FindSplines(const Image3F& opsin);
 
