@@ -10,16 +10,19 @@
 #define LIB_JXL_ENC_TRANSFORMS_INL_H_
 #endif
 
-#include <stddef.h>
-
+#include <cstddef>
+#include <cstdint>
 #include <hwy/highway.h>
 
 #include "lib/jxl/ac_strategy.h"
-#include "lib/jxl/coeff_order_fwd.h"
 #include "lib/jxl/dct-inl.h"
 #include "lib/jxl/dct_scales.h"
+
 HWY_BEFORE_NAMESPACE();
 namespace jxl {
+
+enum class AcStrategyType : uint32_t;
+
 namespace HWY_NAMESPACE {
 namespace {
 
@@ -49,7 +52,7 @@ HWY_INLINE void ReinterpretingIDCT(const float* input,
   }
 
   // ROWS, COLS <= 8, so we can put scratch space on the stack.
-  HWY_ALIGN float scratch_space[ROWS * COLS];
+  HWY_ALIGN float scratch_space[ROWS * COLS * 3];
   ComputeScaledIDCT<ROWS, COLS>()(block, DCTTo(output, output_stride),
                                   scratch_space);
 }
@@ -399,10 +402,10 @@ template <size_t afv_kind>
 void AFVTransformFromPixels(const float* JXL_RESTRICT pixels,
                             size_t pixels_stride,
                             float* JXL_RESTRICT coefficients) {
-  HWY_ALIGN float scratch_space[4 * 8 * 2];
+  HWY_ALIGN float scratch_space[4 * 8 * 5];
   size_t afv_x = afv_kind & 1;
   size_t afv_y = afv_kind / 2;
-  HWY_ALIGN float block[4 * 8];
+  HWY_ALIGN float block[4 * 8] = {};
   for (size_t iy = 0; iy < 4; iy++) {
     for (size_t ix = 0; ix < 4; ix++) {
       block[(afv_y == 1 ? 3 - iy : iy) * 4 + (afv_x == 1 ? 3 - ix : ix)] =
@@ -445,15 +448,14 @@ void AFVTransformFromPixels(const float* JXL_RESTRICT pixels,
   coefficients[8] = (block00 + block01 - 2 * block10) * 0.25f;
 }
 
-HWY_MAYBE_UNUSED void TransformFromPixels(const AcStrategy::Type strategy,
+HWY_MAYBE_UNUSED void TransformFromPixels(const AcStrategyType strategy,
                                           const float* JXL_RESTRICT pixels,
                                           size_t pixels_stride,
                                           float* JXL_RESTRICT coefficients,
                                           float* JXL_RESTRICT scratch_space) {
-  using Type = AcStrategy::Type;
+  using Type = AcStrategyType;
   switch (strategy) {
     case Type::IDENTITY: {
-      PROFILER_ZONE("DCT Identity");
       for (size_t y = 0; y < 2; y++) {
         for (size_t x = 0; x < 2; x++) {
           float block_dc = 0;
@@ -486,7 +488,6 @@ HWY_MAYBE_UNUSED void TransformFromPixels(const AcStrategy::Type strategy,
       break;
     }
     case Type::DCT8X4: {
-      PROFILER_ZONE("DCT 8x4");
       for (size_t x = 0; x < 2; x++) {
         HWY_ALIGN float block[4 * 8];
         ComputeScaledDCT<8, 4>()(DCTFrom(pixels + x * 4, pixels_stride), block,
@@ -505,7 +506,6 @@ HWY_MAYBE_UNUSED void TransformFromPixels(const AcStrategy::Type strategy,
       break;
     }
     case Type::DCT4X8: {
-      PROFILER_ZONE("DCT 4x8");
       for (size_t y = 0; y < 2; y++) {
         HWY_ALIGN float block[4 * 8];
         ComputeScaledDCT<4, 8>()(
@@ -524,7 +524,6 @@ HWY_MAYBE_UNUSED void TransformFromPixels(const AcStrategy::Type strategy,
       break;
     }
     case Type::DCT4X4: {
-      PROFILER_ZONE("DCT 4");
       for (size_t y = 0; y < 2; y++) {
         for (size_t x = 0; x < 2; x++) {
           HWY_ALIGN float block[4 * 4];
@@ -549,149 +548,124 @@ HWY_MAYBE_UNUSED void TransformFromPixels(const AcStrategy::Type strategy,
       break;
     }
     case Type::DCT2X2: {
-      PROFILER_ZONE("DCT 2");
       DCT2TopBlock<8>(pixels, pixels_stride, coefficients);
       DCT2TopBlock<4>(coefficients, kBlockDim, coefficients);
       DCT2TopBlock<2>(coefficients, kBlockDim, coefficients);
       break;
     }
     case Type::DCT16X16: {
-      PROFILER_ZONE("DCT 16");
       ComputeScaledDCT<16, 16>()(DCTFrom(pixels, pixels_stride), coefficients,
                                  scratch_space);
       break;
     }
     case Type::DCT16X8: {
-      PROFILER_ZONE("DCT 16x8");
       ComputeScaledDCT<16, 8>()(DCTFrom(pixels, pixels_stride), coefficients,
                                 scratch_space);
       break;
     }
     case Type::DCT8X16: {
-      PROFILER_ZONE("DCT 8x16");
       ComputeScaledDCT<8, 16>()(DCTFrom(pixels, pixels_stride), coefficients,
                                 scratch_space);
       break;
     }
     case Type::DCT32X8: {
-      PROFILER_ZONE("DCT 32x8");
       ComputeScaledDCT<32, 8>()(DCTFrom(pixels, pixels_stride), coefficients,
                                 scratch_space);
       break;
     }
     case Type::DCT8X32: {
-      PROFILER_ZONE("DCT 8x32");
       ComputeScaledDCT<8, 32>()(DCTFrom(pixels, pixels_stride), coefficients,
                                 scratch_space);
       break;
     }
     case Type::DCT32X16: {
-      PROFILER_ZONE("DCT 32x16");
       ComputeScaledDCT<32, 16>()(DCTFrom(pixels, pixels_stride), coefficients,
                                  scratch_space);
       break;
     }
     case Type::DCT16X32: {
-      PROFILER_ZONE("DCT 16x32");
       ComputeScaledDCT<16, 32>()(DCTFrom(pixels, pixels_stride), coefficients,
                                  scratch_space);
       break;
     }
     case Type::DCT32X32: {
-      PROFILER_ZONE("DCT 32");
       ComputeScaledDCT<32, 32>()(DCTFrom(pixels, pixels_stride), coefficients,
                                  scratch_space);
       break;
     }
     case Type::DCT: {
-      PROFILER_ZONE("DCT 8");
       ComputeScaledDCT<8, 8>()(DCTFrom(pixels, pixels_stride), coefficients,
                                scratch_space);
       break;
     }
     case Type::AFV0: {
-      PROFILER_ZONE("AFV0");
       AFVTransformFromPixels<0>(pixels, pixels_stride, coefficients);
       break;
     }
     case Type::AFV1: {
-      PROFILER_ZONE("AFV1");
       AFVTransformFromPixels<1>(pixels, pixels_stride, coefficients);
       break;
     }
     case Type::AFV2: {
-      PROFILER_ZONE("AFV2");
       AFVTransformFromPixels<2>(pixels, pixels_stride, coefficients);
       break;
     }
     case Type::AFV3: {
-      PROFILER_ZONE("AFV3");
       AFVTransformFromPixels<3>(pixels, pixels_stride, coefficients);
       break;
     }
     case Type::DCT64X64: {
-      PROFILER_ZONE("DCT 64x64");
       ComputeScaledDCT<64, 64>()(DCTFrom(pixels, pixels_stride), coefficients,
                                  scratch_space);
       break;
     }
     case Type::DCT64X32: {
-      PROFILER_ZONE("DCT 64x32");
       ComputeScaledDCT<64, 32>()(DCTFrom(pixels, pixels_stride), coefficients,
                                  scratch_space);
       break;
     }
     case Type::DCT32X64: {
-      PROFILER_ZONE("DCT 32x64");
       ComputeScaledDCT<32, 64>()(DCTFrom(pixels, pixels_stride), coefficients,
                                  scratch_space);
       break;
     }
     case Type::DCT128X128: {
-      PROFILER_ZONE("DCT 128x128");
       ComputeScaledDCT<128, 128>()(DCTFrom(pixels, pixels_stride), coefficients,
                                    scratch_space);
       break;
     }
     case Type::DCT128X64: {
-      PROFILER_ZONE("DCT 128x64");
       ComputeScaledDCT<128, 64>()(DCTFrom(pixels, pixels_stride), coefficients,
                                   scratch_space);
       break;
     }
     case Type::DCT64X128: {
-      PROFILER_ZONE("DCT 64x128");
       ComputeScaledDCT<64, 128>()(DCTFrom(pixels, pixels_stride), coefficients,
                                   scratch_space);
       break;
     }
     case Type::DCT256X256: {
-      PROFILER_ZONE("DCT 256x256");
       ComputeScaledDCT<256, 256>()(DCTFrom(pixels, pixels_stride), coefficients,
                                    scratch_space);
       break;
     }
     case Type::DCT256X128: {
-      PROFILER_ZONE("DCT 256x128");
       ComputeScaledDCT<256, 128>()(DCTFrom(pixels, pixels_stride), coefficients,
                                    scratch_space);
       break;
     }
     case Type::DCT128X256: {
-      PROFILER_ZONE("DCT 128x256");
       ComputeScaledDCT<128, 256>()(DCTFrom(pixels, pixels_stride), coefficients,
                                    scratch_space);
       break;
     }
-    case Type::kNumValidStrategies:
-      JXL_ABORT("Invalid strategy");
   }
 }
 
-HWY_MAYBE_UNUSED void DCFromLowestFrequencies(const AcStrategy::Type strategy,
+HWY_MAYBE_UNUSED void DCFromLowestFrequencies(const AcStrategyType strategy,
                                               const float* block, float* dc,
                                               size_t dc_stride) {
-  using Type = AcStrategy::Type;
+  using Type = AcStrategyType;
   switch (strategy) {
     case Type::DCT16X8: {
       ReinterpretingIDCT</*DCT_ROWS=*/2 * kBlockDim, /*DCT_COLS=*/kBlockDim,
@@ -813,8 +787,6 @@ HWY_MAYBE_UNUSED void DCFromLowestFrequencies(const AcStrategy::Type strategy,
     case Type::IDENTITY:
       dc[0] = block[0];
       break;
-    case Type::kNumValidStrategies:
-      JXL_ABORT("Invalid strategy");
   }
 }
 
