@@ -17,24 +17,23 @@ namespace jxl {
 
 #if HWY_ONCE
 HWY_EXPORT(TransformFromPixels);
-void TransformFromPixels(const AcStrategy::Type strategy,
+void TransformFromPixels(const AcStrategyType strategy,
                          const float* JXL_RESTRICT pixels, size_t pixels_stride,
                          float* JXL_RESTRICT coefficients,
                          float* scratch_space) {
-  return HWY_DYNAMIC_DISPATCH(TransformFromPixels)(
-      strategy, pixels, pixels_stride, coefficients, scratch_space);
+  HWY_DYNAMIC_DISPATCH(TransformFromPixels)
+  (strategy, pixels, pixels_stride, coefficients, scratch_space);
 }
 
 HWY_EXPORT(DCFromLowestFrequencies);
-void DCFromLowestFrequencies(AcStrategy::Type strategy, const float* block,
+void DCFromLowestFrequencies(AcStrategyType strategy, const float* block,
                              float* dc, size_t dc_stride) {
-  return HWY_DYNAMIC_DISPATCH(DCFromLowestFrequencies)(strategy, block, dc,
-                                                       dc_stride);
+  HWY_DYNAMIC_DISPATCH(DCFromLowestFrequencies)(strategy, block, dc, dc_stride);
 }
 
 HWY_EXPORT(AFVDCT4x4);
 void AFVDCT4x4(const float* JXL_RESTRICT pixels, float* JXL_RESTRICT coeffs) {
-  return HWY_DYNAMIC_DISPATCH(AFVDCT4x4)(pixels, coeffs);
+  HWY_DYNAMIC_DISPATCH(AFVDCT4x4)(pixels, coeffs);
 }
 #endif  // HWY_ONCE
 
