@@ -7,24 +7,33 @@
 #ifndef LIB_JXL_ENCODE_INTERNAL_H_
 #define LIB_JXL_ENCODE_INTERNAL_H_
 
+#include <jxl/cms_interface.h>
+#include <jxl/codestream_header.h>
 #include <jxl/encode.h>
 #include <jxl/memory_manager.h>
-#include <jxl/parallel_runner.h>
 #include <jxl/types.h>
-#include <sys/types.h>
 
+#include <algorithm>
+#include <array>
 #include <cstddef>
 #include <cstdint>
+#include <cstring>
+#include <functional>
 #include <map>
 #include <memory>
+#include <string>
+#include <utility>
 #include <vector>
 
+#include "lib/jxl/base/c_callback_support.h"
 #include "lib/jxl/base/common.h"
+#include "lib/jxl/base/compiler_specific.h"
 #include "lib/jxl/base/data_parallel.h"
 #include "lib/jxl/base/status.h"
-#include "lib/jxl/enc_aux_out.h"
 #include "lib/jxl/enc_fast_lossless.h"
-#include "lib/jxl/enc_frame.h"
+#include "lib/jxl/enc_params.h"
+#include "lib/jxl/image_metadata.h"
+#include "lib/jxl/jpeg/jpeg_data.h"
 #include "lib/jxl/memory_manager_internal.h"
 #include "lib/jxl/padded_bytes.h"
 
@@ -141,9 +150,230 @@ constexpr std::array<unsigned char, 32> kContainerHeader = {
 constexpr std::array<unsigned char, 8> kLevelBoxHeader = {0,   0,   0,   0x9,
                                                           'j', 'x', 'l', 'l'};
 
+static JXL_INLINE size_t BitsPerChannel(JxlDataType data_type) {
+  switch (data_type) {
+    case JXL_TYPE_UINT8:
+      return 8;
+    case JXL_TYPE_UINT16:
+      return 16;
+    case JXL_TYPE_FLOAT:
+      return 32;
+    case JXL_TYPE_FLOAT16:
+      return 16;
+    default:
+      return 0;  // signals unhandled JxlDataType
+  }
+}
+
+static JXL_INLINE size_t BytesPerPixel(JxlPixelFormat format) {
+  return format.num_channels * BitsPerChannel(format.data_type) / 8;
+}
+
+using ScopedInputBuffer =
+    std::unique_ptr<const void, std::function<void(const void*)>>;
+
+static JXL_INLINE ScopedInputBuffer
+GetColorBuffer(JxlChunkedFrameInputSource& input, size_t xpos, size_t ypos,
+               size_t xsize, size_t ysize, size_t* row_offset) {
+  return ScopedInputBuffer(
+      input.get_color_channel_data_at(input.opaque, xpos, ypos, xsize, ysize,
+                                      row_offset),
+      [&input](const void* p) { input.release_buffer(input.opaque, p); });
+}
+
+static JXL_INLINE ScopedInputBuffer GetExtraChannelBuffer(
+    JxlChunkedFrameInputSource& input, size_t ec_index, size_t xpos,
+    size_t ypos, size_t xsize, size_t ysize, size_t* row_offset) {
+  return ScopedInputBuffer(
+      input.get_extra_channel_data_at(input.opaque, ec_index, xpos, ypos, xsize,
+                                      ysize, row_offset),
+      [&input](const void* p) { input.release_buffer(input.opaque, p); });
+}
+
+// Common adapter for an existing frame input source or a whole-image input
+// buffer or a parsed JPEG file.
+class JxlEncoderChunkedFrameAdapter {
+ public:
+  JxlEncoderChunkedFrameAdapter(size_t xs, size_t ys, size_t num_extra_channels)
+      : xsize(xs), ysize(ys), channels_(1 + num_extra_channels) {}
+
+  void SetInputSource(JxlChunkedFrameInputSource input_source) {
+    input_source_ = input_source;
+    has_input_source_ = true;
+  }
+
+  bool SetFromBuffer(size_t channel, const uint8_t* buffer, size_t size,
+                     JxlPixelFormat format) {
+    if (channel >= channels_.size()) return false;
+    if (!channels_[channel].SetFromBuffer(buffer, size, format, xsize, ysize)) {
+      return false;
+    }
+    if (channel > 0) channels_[channel].CopyBuffer();
+    return true;
+  }
+
+  void SetJPEGData(std::unique_ptr<jpeg::JPEGData> jpeg_data) {
+    jpeg_data_ = std::move(jpeg_data);
+  }
+
+  // NB: after TakeJPEGData it will return false!
+  bool IsJPEG() const { return jpeg_data_ != nullptr; }
+
+  std::unique_ptr<jpeg::JPEGData> TakeJPEGData() {
+    return std::move(jpeg_data_);
+  }
+
+  JxlChunkedFrameInputSource GetInputSource() {
+    if (has_input_source_) {
+      return input_source_;
+    }
+    return JxlChunkedFrameInputSource{
+        this,
+        METHOD_TO_C_CALLBACK(
+            &JxlEncoderChunkedFrameAdapter::GetColorChannelsPixelFormat),
+        METHOD_TO_C_CALLBACK(
+            &JxlEncoderChunkedFrameAdapter::GetColorChannelDataAt),
+        METHOD_TO_C_CALLBACK(
+            &JxlEncoderChunkedFrameAdapter::GetExtraChannelPixelFormat),
+        METHOD_TO_C_CALLBACK(
+            &JxlEncoderChunkedFrameAdapter::GetExtraChannelDataAt),
+        METHOD_TO_C_CALLBACK(
+            &JxlEncoderChunkedFrameAdapter::ReleaseCurrentData)};
+  }
+
+  bool CopyBuffers() {
+    if (has_input_source_) {
+      JxlPixelFormat format{4, JXL_TYPE_UINT8, JXL_NATIVE_ENDIAN, 0};
+      input_source_.get_color_channels_pixel_format(input_source_.opaque,
+                                                    &format);
+      size_t row_offset;
+      {
+        auto buffer =
+            GetColorBuffer(input_source_, 0, 0, xsize, ysize, &row_offset);
+        if (!buffer) return false;
+        channels_[0].CopyFromBuffer(buffer.get(), format, xsize, ysize,
+                                    row_offset);
+      }
+      for (size_t ec = 0; ec + 1 < channels_.size(); ++ec) {
+        input_source_.get_extra_channel_pixel_format(input_source_.opaque, ec,
+                                                     &format);
+        auto buffer = GetExtraChannelBuffer(input_source_, ec, 0, 0, xsize,
+                                            ysize, &row_offset);
+        if (!buffer) continue;
+        channels_[1 + ec].CopyFromBuffer(buffer.get(), format, xsize, ysize,
+                                         row_offset);
+      }
+      has_input_source_ = false;
+    } else {
+      channels_[0].CopyBuffer();
+    }
+    return true;
+  }
+
+  bool StreamingInput() const { return has_input_source_; }
+
+  const size_t xsize;
+  const size_t ysize;
+
+ private:
+  void GetColorChannelsPixelFormat(JxlPixelFormat* pixel_format) {
+    *pixel_format = channels_[0].format_;
+  }
+
+  const void* GetColorChannelDataAt(size_t xpos, size_t ypos, size_t xsize,
+                                    size_t ysize, size_t* row_offset) {
+    return channels_[0].GetDataAt(xpos, ypos, xsize, ysize, row_offset);
+  }
+
+  void GetExtraChannelPixelFormat(size_t ec_index,
+                                  JxlPixelFormat* pixel_format) {
+    JXL_DASSERT(1 + ec_index < channels_.size());
+    *pixel_format = channels_[1 + ec_index].format_;
+  }
+
+  const void* GetExtraChannelDataAt(size_t ec_index, size_t xpos, size_t ypos,
+                                    size_t xsize, size_t ysize,
+                                    size_t* row_offset) {
+    JXL_DASSERT(1 + ec_index < channels_.size());
+    return channels_[1 + ec_index].GetDataAt(xpos, ypos, xsize, ysize,
+                                             row_offset);
+  }
+
+  void ReleaseCurrentData(const void* buffer) {
+    // No dynamic memory is allocated in GetColorChannelDataAt or
+    // GetExtraChannelDataAt. Therefore, no cleanup is required here.
+  }
+
+  JxlChunkedFrameInputSource input_source_ = {};
+  bool has_input_source_ = false;
+  std::unique_ptr<jpeg::JPEGData> jpeg_data_;
+  struct Channel {
+    const uint8_t* buffer_ = nullptr;
+    size_t buffer_size_;
+    JxlPixelFormat format_;
+    size_t xsize_;
+    size_t ysize_;
+    size_t bytes_per_pixel_;
+    size_t stride_;
+    std::vector<uint8_t> copy_;
+
+    void SetFormatAndDimensions(JxlPixelFormat format, size_t xsize,
+                                size_t ysize) {
+      format_ = format;
+      xsize_ = xsize;
+      ysize_ = ysize;
+      bytes_per_pixel_ = BytesPerPixel(format_);
+      const size_t last_row_size = xsize_ * bytes_per_pixel_;
+      const size_t align = format_.align;
+      stride_ = (align > 1 ? jxl::DivCeil(last_row_size, align) * align
+                           : last_row_size);
+    }
+
+    bool SetFromBuffer(const uint8_t* buffer, size_t size,
+                       JxlPixelFormat format, size_t xsize, size_t ysize) {
+      SetFormatAndDimensions(format, xsize, ysize);
+      buffer_ = buffer;
+      buffer_size_ = size;
+      const size_t min_buffer_size =
+          stride_ * (ysize_ - 1) + xsize_ * bytes_per_pixel_;
+      return min_buffer_size <= size;
+    }
+
+    void CopyFromBuffer(const void* buffer, JxlPixelFormat format, size_t xsize,
+                        size_t ysize, size_t row_offset) {
+      SetFormatAndDimensions(format, xsize, ysize);
+      buffer_ = nullptr;
+      copy_.resize(ysize * stride_);
+      for (size_t y = 0; y < ysize; ++y) {
+        memcpy(copy_.data() + y * stride_,
+               reinterpret_cast<const uint8_t*>(buffer) + y * row_offset,
+               stride_);
+      }
+    }
+
+    void CopyBuffer() {
+      if (buffer_) {
+        copy_ = std::vector<uint8_t>(buffer_, buffer_ + buffer_size_);
+        buffer_ = nullptr;
+      }
+    }
+
+    const void* GetDataAt(size_t xpos, size_t ypos, size_t xsize, size_t ysize,
+                          size_t* row_offset) const {
+      const uint8_t* buffer = copy_.empty() ? buffer_ : copy_.data();
+      JXL_DASSERT(ypos + ysize <= ysize_);
+      JXL_DASSERT(xpos + xsize <= xsize_);
+      JXL_DASSERT(buffer);
+      *row_offset = stride_;
+      return buffer + ypos * stride_ + xpos * bytes_per_pixel_;
+    }
+  };
+  std::vector<Channel> channels_;
+};
+
 struct JxlEncoderQueuedFrame {
   JxlEncoderFrameSettingsValues option_values;
-  ImageBundle frame;
+  JxlEncoderChunkedFrameAdapter frame_data;
   std::vector<uint8_t> ec_initialized;
 };
 
@@ -198,9 +428,12 @@ class JxlEncoderOutputProcessorWrapper {
   friend class JxlOutputProcessorBuffer;
 
  public:
-  JxlEncoderOutputProcessorWrapper() = default;
-  explicit JxlEncoderOutputProcessorWrapper(JxlEncoderOutputProcessor processor)
-      : external_output_processor_(
+  explicit JxlEncoderOutputProcessorWrapper(JxlMemoryManager* memory_manager)
+      : memory_manager_(memory_manager) {}
+  JxlEncoderOutputProcessorWrapper(JxlMemoryManager* memory_manager,
+                                   JxlEncoderOutputProcessor processor)
+      : memory_manager_(memory_manager),
+        external_output_processor_(
             jxl::make_unique<JxlEncoderOutputProcessor>(processor)) {}
 
   bool HasAvailOut() const { return avail_out_ != nullptr; }
@@ -211,13 +444,13 @@ class JxlEncoderOutputProcessorWrapper {
   jxl::StatusOr<JxlOutputProcessorBuffer> GetBuffer(size_t min_size,
                                                     size_t requested_size = 0);
 
-  void Seek(size_t pos);
+  jxl::Status Seek(size_t pos);
 
-  void SetFinalizedPosition();
+  jxl::Status SetFinalizedPosition();
 
   size_t CurrentPosition() const { return position_; }
 
-  bool SetAvailOut(uint8_t** next_out, size_t* avail_out);
+  jxl::Status SetAvailOut(uint8_t** next_out, size_t* avail_out);
 
   bool WasStopRequested() const { return stop_requested_; }
   bool OutputProcessorSet() const {
@@ -227,15 +460,22 @@ class JxlEncoderOutputProcessorWrapper {
     return output_position_ < finalized_position_;
   }
 
+  jxl::Status CopyOutput(std::vector<uint8_t>& output, uint8_t* next_out,
+                         size_t& avail_out);
+
  private:
-  void ReleaseBuffer(size_t bytes_used);
+  jxl::Status ReleaseBuffer(size_t bytes_used);
 
   // Tries to write all the bytes up to the finalized position.
-  void FlushOutput();
+  jxl::Status FlushOutput();
 
   bool AppendBufferToExternalProcessor(void* data, size_t count);
 
   struct InternalBuffer {
+    explicit InternalBuffer(JxlMemoryManager* memory_manager)
+        : owned_data(memory_manager) {
+      JXL_DASSERT(memory_manager != nullptr);
+    }
     // Bytes in the range `[output_position_ - start_of_the_buffer,
     // written_bytes)` need to be flushed out.
     size_t written_bytes = 0;
@@ -260,6 +500,7 @@ class JxlEncoderOutputProcessorWrapper {
   bool stop_requested_ = false;
   bool has_buffer_ = false;
 
+  JxlMemoryManager* memory_manager_;
   std::unique_ptr<JxlEncoderOutputProcessor> external_output_processor_;
 };
 
@@ -274,7 +515,11 @@ class JxlOutputProcessorBuffer {
         size_(size),
         bytes_used_(bytes_used),
         wrapper_(wrapper) {}
-  ~JxlOutputProcessorBuffer() { release(); }
+  ~JxlOutputProcessorBuffer() {
+    jxl::Status result = release();
+    (void)result;
+    JXL_DASSERT(result);
+  }
 
   JxlOutputProcessorBuffer(const JxlOutputProcessorBuffer&) = delete;
   JxlOutputProcessorBuffer(JxlOutputProcessorBuffer&& other) noexcept
@@ -284,30 +529,36 @@ class JxlOutputProcessorBuffer {
     other.size_ = 0;
   }
 
-  void advance(size_t count) {
-    JXL_ASSERT(count <= size_);
+  jxl::Status advance(size_t count) {
+    JXL_ENSURE(count <= size_);
     data_ += count;
     size_ -= count;
     bytes_used_ += count;
+    return true;
   }
 
-  void release() {
+  jxl::Status release() {
+    jxl::Status result = jxl::OkStatus();
     if (this->data_) {
-      wrapper_->ReleaseBuffer(bytes_used_);
+      result = wrapper_->ReleaseBuffer(bytes_used_);
     }
     data_ = nullptr;
     size_ = 0;
+    return result;
   }
 
-  void append(const void* data, size_t count) {
+  jxl::Status append(const void* data, size_t count) {
     memcpy(data_, data, count);
-    advance(count);
+    JXL_RETURN_IF_ERROR(advance(count));
+    return true;
   }
 
   template <typename T>
-  void append(const T& data) {
+  jxl::Status append(const T& data) {
     static_assert(sizeof(*std::begin(data)) == 1, "Cannot append non-bytes");
-    append(&*std::begin(data), std::end(data) - std::begin(data));
+    JXL_RETURN_IF_ERROR(
+        append(&*std::begin(data), std::end(data) - std::begin(data)));
+    return true;
   }
 
   JxlOutputProcessorBuffer& operator=(const JxlOutputProcessorBuffer&) = delete;
@@ -326,15 +577,28 @@ class JxlOutputProcessorBuffer {
   JxlEncoderOutputProcessorWrapper* wrapper_;
 };
 
+template <typename T>
+jxl::Status AppendData(JxlEncoderOutputProcessorWrapper& output_processor,
+                       const T& data) {
+  size_t size = std::end(data) - std::begin(data);
+  size_t written = 0;
+  while (written < size) {
+    JXL_ASSIGN_OR_RETURN(auto buffer,
+                         output_processor.GetBuffer(1, size - written));
+    size_t n = std::min(size - written, buffer.size());
+    JXL_RETURN_IF_ERROR(buffer.append(data.data() + written, n));
+    written += n;
+  }
+  return jxl::OkStatus();
+}
+
 // Internal use only struct, can only be initialized correctly by
 // JxlEncoderCreate.
 struct JxlEncoderStruct {
-  JxlEncoderError error = JxlEncoderError::JXL_ENC_ERR_OK;
+  JxlEncoderStruct() : output_processor(&memory_manager) {}
   JxlMemoryManager memory_manager;
   jxl::MemoryManagerUniquePtr<jxl::ThreadPool> thread_pool{
       nullptr, jxl::MemoryManagerDeleteHelper(&memory_manager)};
-  JxlCmsInterface cms;
-  bool cms_set;
   std::vector<jxl::MemoryManagerUniquePtr<JxlEncoderFrameSettings>>
       encoder_options;
 
@@ -349,10 +613,12 @@ struct JxlEncoderStruct {
   // so we need to keep track of the total of flushed or queue
   // codestream bytes. These bytes may be in a single jxlc box
   // or across multiple jxlp boxes.
-  size_t codestream_bytes_written_beginning_of_frame;
   size_t codestream_bytes_written_end_of_frame;
   jxl::JxlEncoderFrameIndexBox frame_index_box;
 
+  JxlCmsInterface cms;
+  bool cms_set;
+
   // Force using the container even if not needed
   bool use_container;
   // User declared they will add metadata boxes
@@ -361,23 +627,26 @@ struct JxlEncoderStruct {
   // TODO(lode): move level into jxl::CompressParams since some C++
   // implementation decisions should be based on it: level 10 allows more
   // features to be used.
-  int32_t codestream_level;
   bool store_jpeg_metadata;
+  int32_t codestream_level;
   jxl::CodecMetadata metadata;
   std::vector<uint8_t> jpeg_metadata;
 
-  // Wrote any output at all, so wrote the data before the first user added
-  // frame or box, such as signature, basic info, ICC profile or jpeg
-  // reconstruction box.
-  bool wrote_bytes;
   jxl::CompressParams last_used_cparams;
   JxlBasicInfo basic_info;
 
+  JxlEncoderError error = JxlEncoderError::JXL_ENC_ERR_OK;
+
   // Encoder wrote a jxlp (partial codestream) box, so any next codestream
   // parts must also be written in jxlp boxes, a single jxlc box cannot be
   // used. The counter is used for the 4-byte jxlp box index header.
   size_t jxlp_counter;
 
+  // Wrote any output at all, so wrote the data before the first user added
+  // frame or box, such as signature, basic info, ICC profile or jpeg
+  // reconstruction box.
+  bool wrote_bytes;
+
   bool frames_closed;
   bool boxes_closed;
   bool basic_info_set;
@@ -413,7 +682,7 @@ struct JxlEncoderFrameSettingsStruct {
 };
 
 struct JxlEncoderStatsStruct {
-  jxl::AuxOut aux_out;
+  std::unique_ptr<jxl::AuxOut> aux_out;
 };
 
 #endif  // LIB_JXL_ENCODE_INTERNAL_H_
