@@ -5,11 +5,13 @@
 
 #include "lib/jxl/modular/transform/squeeze.h"
 
-#include <stdlib.h>
+#include <jxl/memory_manager.h>
 
+#include <cstdlib>
+
+#include "lib/jxl/base/common.h"
 #include "lib/jxl/base/data_parallel.h"
 #include "lib/jxl/base/printf_macros.h"
-#include "lib/jxl/common.h"
 #include "lib/jxl/modular/modular_image.h"
 #include "lib/jxl/modular/transform/transform.h"
 #undef HWY_TARGET_INCLUDE
@@ -58,6 +60,7 @@ JXL_INLINE void FastUnsqueeze(const pixel_type *JXL_RESTRICT p_residual,
     auto next_avg = Load(d, p_navg + x);
     auto top = Load(d, p_pout + x);
     // Equivalent to SmoothTendency(top,avg,next_avg), but without branches
+    // typo:off
     auto Ba = Sub(top, avg);
     auto an = Sub(avg, next_avg);
     auto nonmono = Xor(Ba, an);
@@ -79,6 +82,7 @@ JXL_INLINE void FastUnsqueeze(const pixel_type *JXL_RESTRICT p_residual,
     auto absBa2 = Add(ShiftLeft<1>(absBa), And(absdiff, Set(d, 1)));
     absdiff = IfThenElse(Gt(absdiff, absBa2),
                          Add(ShiftLeft<1>(absBa), Set(d, 1)), absdiff);
+    // typo:on
     auto absan2 = ShiftLeft<1>(absan);
     absdiff = IfThenElse(Gt(Add(absdiff, And(absdiff, Set(d, 1))), absan2),
                          absan2, absdiff);
@@ -98,13 +102,14 @@ JXL_INLINE void FastUnsqueeze(const pixel_type *JXL_RESTRICT p_residual,
 #endif
 
 Status InvHSqueeze(Image &input, uint32_t c, uint32_t rc, ThreadPool *pool) {
-  JXL_ASSERT(c < input.channel.size());
-  JXL_ASSERT(rc < input.channel.size());
+  JXL_ENSURE(c < input.channel.size());
+  JXL_ENSURE(rc < input.channel.size());
   Channel &chin = input.channel[c];
   const Channel &chin_residual = input.channel[rc];
   // These must be valid since we ran MetaApply already.
-  JXL_ASSERT(chin.w == DivCeil(chin.w + chin_residual.w, 2));
-  JXL_ASSERT(chin.h == chin_residual.h);
+  JXL_ENSURE(chin.w == DivCeil(chin.w + chin_residual.w, 2));
+  JXL_ENSURE(chin.h == chin_residual.h);
+  JxlMemoryManager *memory_manager = input.memory_manager();
 
   if (chin_residual.w == 0) {
     // Short-circuit: output channel has same dimensions as input.
@@ -113,7 +118,9 @@ Status InvHSqueeze(Image &input, uint32_t c, uint32_t rc, ThreadPool *pool) {
   }
 
   // Note: chin.w >= chin_residual.w and at most 1 different.
-  Channel chout(chin.w + chin_residual.w, chin.h, chin.hshift - 1, chin.vshift);
+  JXL_ASSIGN_OR_RETURN(Channel chout,
+                       Channel::Create(memory_manager, chin.w + chin_residual.w,
+                                       chin.h, chin.hshift - 1, chin.vshift));
   JXL_DEBUG_V(4,
               "Undoing horizontal squeeze of channel %i using residuals in "
               "channel %i (going from width %" PRIuS " to %" PRIuS ")",
@@ -148,7 +155,8 @@ Status InvHSqueeze(Image &input, uint32_t c, uint32_t rc, ThreadPool *pool) {
   // 8 rows at a time and treat it as a vertical unsqueeze of a
   // transposed 8x8 block (or 9x8 for one input).
   static constexpr const size_t kRowsPerThread = 8;
-  const auto unsqueeze_span = [&](const uint32_t task, size_t /* thread */) {
+  const auto unsqueeze_span = [&](const uint32_t task,
+                                  size_t /* thread */) -> Status {
     const size_t y0 = task * kRowsPerThread;
     const size_t rows = std::min(kRowsPerThread, chin.h - y0);
     size_t x = 0;
@@ -198,6 +206,7 @@ Status InvHSqueeze(Image &input, uint32_t c, uint32_t rc, ThreadPool *pool) {
     for (size_t y = 0; y < rows; y++) {
       unsqueeze_row(y0 + y, x);
     }
+    return true;
   };
   JXL_RETURN_IF_ERROR(RunOnPool(pool, 0, DivCeil(chin.h, kRowsPerThread),
                                 ThreadPool::NoInit, unsqueeze_span,
@@ -207,13 +216,14 @@ Status InvHSqueeze(Image &input, uint32_t c, uint32_t rc, ThreadPool *pool) {
 }
 
 Status InvVSqueeze(Image &input, uint32_t c, uint32_t rc, ThreadPool *pool) {
-  JXL_ASSERT(c < input.channel.size());
-  JXL_ASSERT(rc < input.channel.size());
+  JXL_ENSURE(c < input.channel.size());
+  JXL_ENSURE(rc < input.channel.size());
   const Channel &chin = input.channel[c];
   const Channel &chin_residual = input.channel[rc];
   // These must be valid since we ran MetaApply already.
-  JXL_ASSERT(chin.h == DivCeil(chin.h + chin_residual.h, 2));
-  JXL_ASSERT(chin.w == chin_residual.w);
+  JXL_ENSURE(chin.h == DivCeil(chin.h + chin_residual.h, 2));
+  JXL_ENSURE(chin.w == chin_residual.w);
+  JxlMemoryManager *memory_manager = input.memory_manager();
 
   if (chin_residual.h == 0) {
     // Short-circuit: output channel has same dimensions as input.
@@ -222,7 +232,10 @@ Status InvVSqueeze(Image &input, uint32_t c, uint32_t rc, ThreadPool *pool) {
   }
 
   // Note: chin.h >= chin_residual.h and at most 1 different.
-  Channel chout(chin.w, chin.h + chin_residual.h, chin.hshift, chin.vshift - 1);
+  JXL_ASSIGN_OR_RETURN(
+      Channel chout,
+      Channel::Create(memory_manager, chin.w, chin.h + chin_residual.h,
+                      chin.hshift, chin.vshift - 1));
   JXL_DEBUG_V(
       4,
       "Undoing vertical squeeze of channel %i using residuals in channel "
@@ -236,9 +249,11 @@ Status InvVSqueeze(Image &input, uint32_t c, uint32_t rc, ThreadPool *pool) {
   }
 
   static constexpr const int kColsPerThread = 64;
-  const auto unsqueeze_slice = [&](const uint32_t task, size_t /* thread */) {
+  const auto unsqueeze_slice = [&](const uint32_t task,
+                                   size_t /* thread */) -> Status {
     const size_t x0 = task * kColsPerThread;
-    const size_t x1 = std::min((size_t)(task + 1) * kColsPerThread, chin.w);
+    const size_t x1 =
+        std::min(static_cast<size_t>(task + 1) * kColsPerThread, chin.w);
     const size_t w = x1 - x0;
     // We only iterate up to std::min(chin_residual.h, chin.h) which is
     // always chin_residual.h.
@@ -272,6 +287,7 @@ Status InvVSqueeze(Image &input, uint32_t c, uint32_t rc, ThreadPool *pool) {
         p_nout[x] = out - diff;
       }
     }
+    return true;
   };
   JXL_RETURN_IF_ERROR(RunOnPool(pool, 0, DivCeil(chin.w, kColsPerThread),
                                 ThreadPool::NoInit, unsqueeze_slice,
@@ -289,7 +305,7 @@ Status InvVSqueeze(Image &input, uint32_t c, uint32_t rc, ThreadPool *pool) {
   return true;
 }
 
-Status InvSqueeze(Image &input, std::vector<SqueezeParams> parameters,
+Status InvSqueeze(Image &input, const std::vector<SqueezeParams> &parameters,
                   ThreadPool *pool) {
   for (int i = parameters.size() - 1; i >= 0; i--) {
     JXL_RETURN_IF_ERROR(
@@ -306,7 +322,7 @@ Status InvSqueeze(Image &input, std::vector<SqueezeParams> parameters,
     }
     if (beginc < input.nb_meta_channels) {
       // This is checked in MetaSqueeze.
-      JXL_ASSERT(input.nb_meta_channels > parameters[i].num_c);
+      JXL_ENSURE(input.nb_meta_channels > parameters[i].num_c);
       input.nb_meta_channels -= parameters[i].num_c;
     }
 
@@ -314,7 +330,7 @@ Status InvSqueeze(Image &input, std::vector<SqueezeParams> parameters,
       uint32_t rc = offset + c - beginc;
       // MetaApply should imply that `rc` is within range, otherwise there's a
       // programming bug.
-      JXL_ASSERT(rc < input.channel.size());
+      JXL_ENSURE(rc < input.channel.size());
       if ((input.channel[c].w < input.channel[rc].w) ||
           (input.channel[c].h < input.channel[rc].h)) {
         return JXL_FAILURE("Corrupted squeeze transform");
@@ -340,7 +356,7 @@ HWY_AFTER_NAMESPACE();
 namespace jxl {
 
 HWY_EXPORT(InvSqueeze);
-Status InvSqueeze(Image &input, std::vector<SqueezeParams> parameters,
+Status InvSqueeze(Image &input, const std::vector<SqueezeParams> &parameters,
                   ThreadPool *pool) {
   return HWY_DYNAMIC_DISPATCH(InvSqueeze)(input, parameters, pool);
 }
@@ -380,21 +396,21 @@ void DefaultSqueezeParameters(std::vector<SqueezeParams> *parameters,
   params.in_place = true;
 
   if (!wide) {
-    if (h > JXL_MAX_FIRST_PREVIEW_SIZE) {
+    if (h > kMaxFirstPreviewSize) {
       params.horizontal = false;
       parameters->push_back(params);
       h = (h + 1) / 2;
       JXL_DEBUG_V(7, "Vertical (%" PRIuS "x%" PRIuS "), ", w, h);
     }
   }
-  while (w > JXL_MAX_FIRST_PREVIEW_SIZE || h > JXL_MAX_FIRST_PREVIEW_SIZE) {
-    if (w > JXL_MAX_FIRST_PREVIEW_SIZE) {
+  while (w > kMaxFirstPreviewSize || h > kMaxFirstPreviewSize) {
+    if (w > kMaxFirstPreviewSize) {
       params.horizontal = true;
       parameters->push_back(params);
       w = (w + 1) / 2;
       JXL_DEBUG_V(7, "Horizontal (%" PRIuS "x%" PRIuS "), ", w, h);
     }
-    if (h > JXL_MAX_FIRST_PREVIEW_SIZE) {
+    if (h > kMaxFirstPreviewSize) {
       params.horizontal = false;
       parameters->push_back(params);
       h = (h + 1) / 2;
@@ -415,17 +431,18 @@ Status CheckMetaSqueezeParams(const SqueezeParams &parameter,
 }
 
 Status MetaSqueeze(Image &image, std::vector<SqueezeParams> *parameters) {
+  JxlMemoryManager *memory_manager = image.memory_manager();
   if (parameters->empty()) {
     DefaultSqueezeParameters(parameters, image);
   }
 
-  for (size_t i = 0; i < parameters->size(); i++) {
+  for (auto &parameter : *parameters) {
     JXL_RETURN_IF_ERROR(
-        CheckMetaSqueezeParams((*parameters)[i], image.channel.size()));
-    bool horizontal = (*parameters)[i].horizontal;
-    bool in_place = (*parameters)[i].in_place;
-    uint32_t beginc = (*parameters)[i].begin_c;
-    uint32_t endc = (*parameters)[i].begin_c + (*parameters)[i].num_c - 1;
+        CheckMetaSqueezeParams(parameter, image.channel.size()));
+    bool horizontal = parameter.horizontal;
+    bool in_place = parameter.in_place;
+    uint32_t beginc = parameter.begin_c;
+    uint32_t endc = parameter.begin_c + parameter.num_c - 1;
 
     uint32_t offset;
     if (beginc < image.nb_meta_channels) {
@@ -436,7 +453,7 @@ Status MetaSqueeze(Image &image, std::vector<SqueezeParams> *parameters) {
         return JXL_FAILURE(
             "Invalid squeeze: meta channels require in-place residuals");
       }
-      image.nb_meta_channels += (*parameters)[i].num_c;
+      image.nb_meta_channels += parameter.num_c;
     }
     if (in_place) {
       offset = endc + 1;
@@ -449,22 +466,24 @@ Status MetaSqueeze(Image &image, std::vector<SqueezeParams> *parameters) {
       }
       size_t w = image.channel[c].w;
       size_t h = image.channel[c].h;
+      if (w == 0 || h == 0) return JXL_FAILURE("Squeezing empty channel");
       if (horizontal) {
         image.channel[c].w = (w + 1) / 2;
-        image.channel[c].hshift++;
+        if (image.channel[c].hshift >= 0) image.channel[c].hshift++;
         w = w - (w + 1) / 2;
       } else {
         image.channel[c].h = (h + 1) / 2;
-        image.channel[c].vshift++;
+        if (image.channel[c].vshift >= 0) image.channel[c].vshift++;
         h = h - (h + 1) / 2;
       }
-      image.channel[c].shrink();
-      Channel dummy(w, h);
-      dummy.hshift = image.channel[c].hshift;
-      dummy.vshift = image.channel[c].vshift;
+      JXL_RETURN_IF_ERROR(image.channel[c].shrink());
+      JXL_ASSIGN_OR_RETURN(Channel placeholder,
+                           Channel::Create(memory_manager, w, h));
+      placeholder.hshift = image.channel[c].hshift;
+      placeholder.vshift = image.channel[c].vshift;
 
       image.channel.insert(image.channel.begin() + offset + (c - beginc),
-                           std::move(dummy));
+                           std::move(placeholder));
       JXL_DEBUG_V(8, "MetaSqueeze applied, current image: %s",
                   image.DebugString().c_str());
     }
