@@ -3,21 +3,20 @@
 // Use of this source code is governed by a BSD-style
 // license that can be found in the LICENSE file.
 
-#include <jxl/cms.h>
+#include <jxl/encode.h>
+#include <jxl/types.h>
 
+#include <cstddef>
 #include <cstdint>
+#include <ios>
+#include <ostream>
 #include <string>
 #include <vector>
 
-#include "lib/extras/codec.h"
-#include "lib/jxl/base/data_parallel.h"
-#include "lib/jxl/base/span.h"
-#include "lib/jxl/codec_in_out.h"
-#include "lib/jxl/enc_butteraugli_comparator.h"
-#include "lib/jxl/enc_cache.h"
-#include "lib/jxl/enc_params.h"
-#include "lib/jxl/image.h"
-#include "lib/jxl/image_test_utils.h"
+#include "lib/extras/dec/jxl.h"
+#include "lib/extras/enc/jxl.h"
+#include "lib/jxl/common.h"
+#include "lib/jxl/test_image.h"
 #include "lib/jxl/test_utils.h"
 #include "lib/jxl/testing.h"
 
@@ -82,37 +81,37 @@ JXL_GTEST_INSTANTIATE_TEST_SUITE_P(
                                         /*shrink8=*/true}));
 
 TEST_P(SpeedTierTest, Roundtrip) {
+  const SpeedTierTestParams& params = GetParam();
   const std::vector<uint8_t> orig = jxl::test::ReadTestData(
       "external/wesaturate/500px/u76c0g_bliznaca_srgb8.png");
-  CodecInOut io;
-  test::ThreadPoolForTests pool(8);
-  ASSERT_TRUE(SetFromBytes(Bytes(orig), &io, &pool));
-
-  const SpeedTierTestParams& params = GetParam();
-
+  test::TestImage t;
+  ASSERT_TRUE(t.DecodeFromBytes(orig));
+  t.ClearMetadata();
   if (params.speed_tier == SpeedTier::kGlacier) {
     // just a few pixels will already take enough time at this setting
-    io.ShrinkTo(8, 8);
+    ASSERT_TRUE(t.SetDimensions(8, 8));
   } else if (params.shrink8) {
-    io.ShrinkTo(io.xsize() / 8, io.ysize() / 8);
+    ASSERT_TRUE(t.SetDimensions(t.ppf().xsize() / 8, t.ppf().ysize() / 8));
   }
 
-  CompressParams cparams;
-  cparams.speed_tier = params.speed_tier;
-  cparams.SetCms(*JxlGetDefaultCms());
-
-  CodecInOut io2, io3;
-  JXL_EXPECT_OK(test::Roundtrip(&io, cparams, {}, &io2, _));
-  EXPECT_LE(ButteraugliDistance(io.frames, io2.frames, ButteraugliParams(),
-                                *JxlGetDefaultCms(),
-                                /*distmap=*/nullptr, /*pool=*/nullptr),
-            1.6);
+  extras::JXLCompressParams cparams;
+  cparams.distance = 1.0f;
+  cparams.allow_expert_options = true;
+  cparams.AddOption(JXL_ENC_FRAME_SETTING_EFFORT,
+                    10 - static_cast<int>(params.speed_tier));
+  extras::JXLDecompressParams dparams;
+  dparams.accepted_formats = {{3, JXL_TYPE_UINT8, JXL_LITTLE_ENDIAN, 0}};
 
+  {
+    extras::PackedPixelFile ppf_out;
+    test::Roundtrip(t.ppf(), cparams, dparams, nullptr, &ppf_out);
+    EXPECT_LE(test::ButteraugliDistance(t.ppf(), ppf_out), 2.0);
+  }
   if (params.shrink8) {
-    cparams.SetLossless();
-    JXL_EXPECT_OK(test::Roundtrip(&io, cparams, {}, &io3, _));
-
-    JXL_EXPECT_OK(SamePixels(*io.Main().color(), *io3.Main().color(), _));
+    cparams.distance = 0.0f;
+    extras::PackedPixelFile ppf_out;
+    test::Roundtrip(t.ppf(), cparams, dparams, nullptr, &ppf_out);
+    EXPECT_EQ(0.0f, test::ComputeDistance2(t.ppf(), ppf_out));
   }
 }
 }  // namespace
