@@ -4,7 +4,6 @@
 // license that can be found in the LICENSE file.
 
 #include <stdint.h>
-#include <stdio.h>
 
 #include <algorithm>
 #include <vector>
@@ -13,10 +12,11 @@
 #define HWY_TARGET_INCLUDE "lib/jxl/xorshift128plus_test.cc"
 #include <hwy/foreach_target.h>
 #include <hwy/highway.h>
-#include <hwy/tests/test_util-inl.h>
+#include <hwy/tests/hwy_gtest.h>
 
 #include "lib/jxl/base/data_parallel.h"
-#include "lib/jxl/base/thread_pool_internal.h"
+#include "lib/jxl/test_utils.h"
+#include "lib/jxl/testing.h"
 #include "lib/jxl/xorshift128plus-inl.h"
 
 HWY_BEFORE_NAMESPACE();
@@ -286,75 +286,75 @@ void TestSeedChanges() {
 }
 
 void TestFloat() {
-  ThreadPoolInternal pool(8);
+  test::ThreadPoolForTests pool(8);
 
 #ifdef JXL_DISABLE_SLOW_TESTS
   const uint32_t kMaxSeed = 256;
 #else   // JXL_DISABLE_SLOW_TESTS
   const uint32_t kMaxSeed = 4096;
 #endif  // JXL_DISABLE_SLOW_TESTS
-  EXPECT_TRUE(RunOnPool(
-      &pool, 0, kMaxSeed, ThreadPool::NoInit,
-      [](const uint32_t seed, size_t /*thread*/) {
-        HWY_ALIGN Xorshift128Plus rng(seed);
+  const auto test_seed = [](const uint32_t seed, size_t /*thread*/) -> Status {
+    HWY_ALIGN Xorshift128Plus rng(seed);
 
-        const HWY_FULL(uint32_t) du;
-        const HWY_FULL(float) df;
-        HWY_ALIGN uint64_t batch[Xorshift128Plus::N];
-        HWY_ALIGN float lanes[MaxLanes(df)];
-        double sum = 0.0;
-        size_t count = 0;
-        const size_t kReps = 2000;
-        for (size_t reps = 0; reps < kReps; ++reps) {
-          rng.Fill(batch);
-          for (size_t i = 0; i < Xorshift128Plus::N * 2; i += Lanes(df)) {
-            const auto bits =
-                Load(du, reinterpret_cast<const uint32_t*>(batch) + i);
-            // 1.0 + 23 random mantissa bits = [1, 2)
-            const auto rand12 =
-                BitCast(df, Or(ShiftRight<9>(bits), Set(du, 0x3F800000)));
-            const auto rand01 = Sub(rand12, Set(df, 1.0f));
-            Store(rand01, df, lanes);
-            for (float lane : lanes) {
-              sum += lane;
-              count += 1;
-              EXPECT_LE(lane, 1.0f);
-              EXPECT_GE(lane, 0.0f);
-            }
-          }
+    const HWY_FULL(uint32_t) du;
+    const HWY_FULL(float) df;
+    HWY_ALIGN uint64_t batch[Xorshift128Plus::N];
+    HWY_ALIGN float lanes[MaxLanes(df)];
+    double sum = 0.0;
+    size_t count = 0;
+    const size_t kReps = 2000;
+    for (size_t reps = 0; reps < kReps; ++reps) {
+      rng.Fill(batch);
+      for (size_t i = 0; i < Xorshift128Plus::N * 2; i += Lanes(df)) {
+        const auto bits =
+            Load(du, reinterpret_cast<const uint32_t*>(batch) + i);
+        // 1.0 + 23 random mantissa bits = [1, 2)
+        const auto rand12 =
+            BitCast(df, Or(ShiftRight<9>(bits), Set(du, 0x3F800000)));
+        const auto rand01 = Sub(rand12, Set(df, 1.0f));
+        Store(rand01, df, lanes);
+        for (float lane : lanes) {
+          sum += lane;
+          count += 1;
+          EXPECT_LE(lane, 1.0f);
+          EXPECT_GE(lane, 0.0f);
         }
+      }
+    }
 
-        // Verify average (uniform distribution)
-        EXPECT_NEAR(0.5, sum / count, 0.00702);
-      },
-      "TestXorShift"));
+    // Verify average (uniform distribution)
+    EXPECT_NEAR(0.5, sum / count, 0.00702);
+    return true;
+  };
+  EXPECT_TRUE(RunOnPool(pool.get(), 0, kMaxSeed, ThreadPool::NoInit, test_seed,
+                        "TestXorShift"));
 }
 
 // Not more than one 64-bit zero
 void TestNotZero() {
-  ThreadPoolInternal pool(8);
+  test::ThreadPoolForTests pool(8);
 
 #ifdef JXL_DISABLE_SLOW_TESTS
   const uint32_t kMaxSeed = 500;
 #else   // JXL_DISABLE_SLOW_TESTS
   const uint32_t kMaxSeed = 2000;
 #endif  // JXL_DISABLE_SLOW_TESTS
-  EXPECT_TRUE(RunOnPool(
-      &pool, 0, kMaxSeed, ThreadPool::NoInit,
-      [](const uint32_t task, size_t /*thread*/) {
-        HWY_ALIGN uint64_t lanes[Xorshift128Plus::N];
+  const auto test_seed = [](const uint32_t task, size_t /*thread*/) -> Status {
+    HWY_ALIGN uint64_t lanes[Xorshift128Plus::N];
 
-        HWY_ALIGN Xorshift128Plus rng(task);
-        size_t num_zero = 0;
-        for (size_t vectors = 0; vectors < 10000; ++vectors) {
-          rng.Fill(lanes);
-          for (uint64_t lane : lanes) {
-            num_zero += static_cast<size_t>(lane == 0);
-          }
-        }
-        EXPECT_LE(num_zero, 1u);
-      },
-      "TestNotZero"));
+    HWY_ALIGN Xorshift128Plus rng(task);
+    size_t num_zero = 0;
+    for (size_t vectors = 0; vectors < 10000; ++vectors) {
+      rng.Fill(lanes);
+      for (uint64_t lane : lanes) {
+        num_zero += static_cast<size_t>(lane == 0);
+      }
+    }
+    EXPECT_LE(num_zero, 1u);
+    return true;
+  };
+  EXPECT_TRUE(RunOnPool(pool.get(), 0, kMaxSeed, ThreadPool::NoInit, test_seed,
+                        "TestNotZero"));
 }
 
 // NOLINTNEXTLINE(google-readability-namespace-comments)
