@@ -6,24 +6,17 @@
 #ifndef TOOLS_BENCHMARK_BENCHMARK_CODEC_H_
 #define TOOLS_BENCHMARK_BENCHMARK_CODEC_H_
 
-#include <stdint.h>
+#include <jxl/memory_manager.h>
 
-#include <deque>
+#include <cstdint>
 #include <string>
 #include <vector>
 
 #include "lib/extras/packed_image.h"
-#include "lib/extras/packed_image_convert.h"
-#include "lib/jxl/base/data_parallel.h"
 #include "lib/jxl/base/span.h"
-#include "lib/jxl/base/status.h"
-#include "lib/jxl/butteraugli/butteraugli.h"
 #include "lib/jxl/codec_in_out.h"
-#include "lib/jxl/image.h"
-#include "tools/args.h"
 #include "tools/benchmark/benchmark_args.h"
 #include "tools/benchmark/benchmark_stats.h"
-#include "tools/cmdline.h"
 #include "tools/speed_stats.h"
 #include "tools/thread_pool_internal.h"
 
@@ -32,6 +25,7 @@ namespace tools {
 
 using ::jxl::CodecInOut;
 using ::jxl::Span;
+using ::jxl::Status;
 using ::jxl::extras::PackedPixelFile;
 
 // Thread-compatible.
@@ -48,7 +42,7 @@ class ImageCodec {
   void set_description(const std::string& desc) { description_ = desc; }
   const std::string& description() const { return description_; }
 
-  virtual void ParseParameters(const std::string& parameters);
+  virtual Status ParseParameters(const std::string& parameters);
 
   virtual Status ParseParam(const std::string& param);
 
@@ -58,8 +52,8 @@ class ImageCodec {
                           jpegxl::tools::SpeedStats* speed_stats) = 0;
 
   virtual Status Decompress(const std::string& filename,
-                            const Span<const uint8_t> compressed,
-                            ThreadPool* pool, PackedPixelFile* ppf,
+                            Span<const uint8_t> compressed, ThreadPool* pool,
+                            PackedPixelFile* ppf,
                             jpegxl::tools::SpeedStats* speed_stats) = 0;
 
   virtual void GetMoreStats(BenchmarkStats* stats) {}
@@ -91,7 +85,8 @@ using ImageCodecPtr = std::unique_ptr<ImageCodec>;
 // Creates an image codec by name, e.g. "jxl" to get a new instance of the
 // jxl codec. Optionally, behind a colon, parameters can be specified,
 // then ParseParameters of the codec gets called with the part behind the colon.
-ImageCodecPtr CreateImageCodec(const std::string& description);
+ImageCodecPtr CreateImageCodec(const std::string& description,
+                               JxlMemoryManager* memory_manager);
 
 }  // namespace tools
 }  // namespace jpegxl
