@@ -6,21 +6,19 @@
 #ifndef TOOLS_BENCHMARK_BENCHMARK_CODEC_PNG_H_
 #define TOOLS_BENCHMARK_BENCHMARK_CODEC_PNG_H_
 
-#if JPEGXL_ENABLE_APNG
-
 #include <string>
 
 #include "lib/jxl/base/status.h"
 #include "tools/benchmark/benchmark_args.h"
 #include "tools/benchmark/benchmark_codec.h"
 
-namespace jxl {
+namespace jpegxl {
+namespace tools {
 ImageCodec* CreateNewPNGCodec(const BenchmarkArgs& args);
 
 // Registers the png-specific command line options.
 Status AddCommandLineOptionsPNGCodec(BenchmarkArgs* args);
-}  // namespace jxl
-
-#endif
+}  // namespace tools
+}  // namespace jpegxl
 
 #endif  // TOOLS_BENCHMARK_BENCHMARK_CODEC_PNG_H_
