@@ -5,13 +5,17 @@
 
 #include "tools/benchmark/benchmark_stats.h"
 
-#include <stdarg.h>
-#include <stddef.h>
-#include <stdio.h>
-#include <string.h>
+#include <jxl/stats.h>
 
 #include <algorithm>
 #include <cmath>
+#include <cstdarg>
+#include <cstddef>
+#include <cstdint>
+#include <cstdio>
+#include <cstring>
+#include <string>
+#include <vector>
 
 #include "lib/jxl/base/printf_macros.h"
 #include "lib/jxl/base/status.h"
@@ -109,7 +113,7 @@ struct ColumnDescriptor {
   bool more;  // Whether to print only if more_columns is enabled
 };
 
-static ColumnDescriptor ExtraMetricDescriptor() {
+ColumnDescriptor ExtraMetricDescriptor() {
   ColumnDescriptor d{{"DO NOT USE"}, 12, 4, TYPE_POSITIVE_FLOAT, false};
   return d;
 }
@@ -146,12 +150,12 @@ std::vector<ColumnDescriptor> GetColumnDescriptors(size_t num_extra_metrics) {
 }
 
 // Computes throughput [megapixels/s] as reported in the report table
-static double ComputeSpeed(size_t pixels, double time_s) {
+double ComputeSpeed(size_t pixels, double time_s) {
   if (time_s == 0.0) return 0;
   return pixels * 1E-6 / time_s;
 }
 
-static std::string FormatFloat(const ColumnDescriptor& label, double value) {
+std::string FormatFloat(const ColumnDescriptor& label, double value) {
   std::string result =
       StringPrintf("%*.*f", label.width - 1, label.precision, value);
 
@@ -192,6 +196,9 @@ void BenchmarkStats::Assimilate(const BenchmarkStats& victim) {
   psnr += victim.psnr;
   distances.insert(distances.end(), victim.distances.begin(),
                    victim.distances.end());
+  pnorms.insert(pnorms.end(), victim.pnorms.begin(), victim.pnorms.end());
+  ssimulacra2s.insert(ssimulacra2s.end(), victim.ssimulacra2s.begin(),
+                      victim.ssimulacra2s.end());
   total_errors += victim.total_errors;
   jxl_stats.Assimilate(victim.jxl_stats);
   if (extra_metrics.size() < victim.extra_metrics.size()) {
@@ -202,23 +209,49 @@ void BenchmarkStats::Assimilate(const BenchmarkStats& victim) {
   }
 }
 
-void BenchmarkStats::PrintMoreStats() const {
+::jxl::Status BenchmarkStats::PrintMoreStats() const {
   if (Args()->print_more_stats) {
     jxl_stats.Print();
   }
-  if (Args()->print_distance_percentiles) {
+  if (Args()->print_distance_percentiles && distances.size() > 1) {
+    const auto& descriptors = GetColumnDescriptors(0);
+    int spaces = 0;
+    for (int i = 0; i < 4; i++) spaces += descriptors[i].width;
+    JXL_ENSURE(distances.size() == pnorms.size());
+    JXL_ENSURE(distances.size() == ssimulacra2s.size());
     std::vector<float> sorted = distances;
     std::sort(sorted.begin(), sorted.end());
+    std::vector<float> sorted2 = pnorms;
+    std::sort(sorted2.begin(), sorted2.end());
+    std::vector<float> sorted3 = ssimulacra2s;
+    std::sort(sorted3.begin(), sorted3.end());
+    int p5idx = 0.05 * distances.size();
+    int p10idx = 0.1 * distances.size();
     int p50idx = 0.5 * distances.size();
     int p90idx = 0.9 * distances.size();
-    printf("50th/90th percentile distance: %.8f  %.8f\n", sorted[p50idx],
-           sorted[p90idx]);
+    int p95idx = 0.95 * distances.size();
+    printf("   ");
+    for (int i = 2; i < spaces; i++) printf("_");
+    printf("| median:       %12.8f %12.8f        %12.8f\n", sorted[p50idx],
+           sorted3[p50idx], sorted2[p50idx]);
+    if (distances.size() >= 10) {
+      printf("  /");
+      for (int i = 2; i < spaces; i++) printf(" ");
+      printf("| p10 (worst):  %12.8f %12.8f        %12.8f\n", sorted[p90idx],
+             sorted3[p10idx], sorted2[p90idx]);
+    }
+    if (distances.size() >= 20) {
+      printf(" / ");
+      for (int i = 2; i < spaces; i++) printf(" ");
+      printf("| p5 (worst):   %12.8f %12.8f        %12.8f\n", sorted[p95idx],
+             sorted3[p5idx], sorted2[p95idx]);
+    }
   }
+  return true;
 }
 
 std::vector<ColumnValue> BenchmarkStats::ComputeColumns(
-    const std::string& codec_desc, size_t corpus_size) const {
-  JXL_CHECK(total_input_files == corpus_size);
+    const std::string& codec_desc) const {
   const double comp_bpp = total_compressed_size * 8.0 / total_input_pixels;
   const double adj_comp_bpp =
       total_adj_compressed_size * 8.0 / total_input_pixels;
@@ -232,7 +265,8 @@ std::vector<ColumnValue> BenchmarkStats::ComputeColumns(
   const double ssimulacra2_avg = ssimulacra2 / total_input_pixels;
   const double bpp_p_norm = p_norm_avg * comp_bpp;
 
-  const double max_distance_avg = sqrt(max_distance / total_input_pixels);
+  const double max_distance_avg =
+      sqrt(max_distance / static_cast<double>(total_input_pixels));
 
   std::vector<ColumnValue> values(
       GetColumnDescriptors(extra_metrics.size()).size());
@@ -281,20 +315,20 @@ static std::string PrintFormattedEntries(
     }
     // All except the first one are right-aligned, the first one is the name,
     // others are numbers with digits matching from the right.
-    if (i == 0) out += value.c_str();
+    if (i == 0) out += value;
     out += std::string(numspaces, ' ');
-    if (i != 0) out += value.c_str();
+    if (i != 0) out += value;
   }
   return out + "\n";
 }
 
-std::string BenchmarkStats::PrintLine(const std::string& codec_desc,
-                                      size_t corpus_size) const {
-  std::vector<ColumnValue> values = ComputeColumns(codec_desc, corpus_size);
+std::string BenchmarkStats::PrintLine(const std::string& codec_desc) const {
+  std::vector<ColumnValue> values = ComputeColumns(codec_desc);
   return PrintFormattedEntries(extra_metrics.size(), values);
 }
 
-std::string PrintHeader(const std::vector<std::string>& extra_metrics_names) {
+::jxl::StatusOr<std::string> PrintHeader(
+    const std::vector<std::string>& extra_metrics_names) {
   std::string out;
   // Extra metrics are handled separately.
   const auto& descriptors = GetColumnDescriptors(0);
@@ -303,14 +337,14 @@ std::string PrintHeader(const std::vector<std::string>& extra_metrics_names) {
     const std::string& label = descriptors[i].label;
     int numspaces = descriptors[i].width - label.size();
     // All except the first one are right-aligned.
-    if (i == 0) out += label.c_str();
+    if (i == 0) out += label;
     out += std::string(numspaces, ' ');
-    if (i != 0) out += label.c_str();
+    if (i != 0) out += label;
   }
   for (const std::string& em : extra_metrics_names) {
     int numspaces = ExtraMetricDescriptor().width - em.size();
-    JXL_CHECK(numspaces >= 1);
-    out += std::string(numspaces, ' ');
+    JXL_ENSURE(numspaces >= 1);
+    out += std::string(std::max(numspaces, 1), ' ');
     out += em;
   }
   out += '\n';
@@ -323,14 +357,14 @@ std::string PrintHeader(const std::vector<std::string>& extra_metrics_names) {
   return out + "\n";
 }
 
-std::string PrintAggregate(
+::jxl::StatusOr<std::string> PrintAggregate(
     size_t num_extra_metrics,
     const std::vector<std::vector<ColumnValue>>& aggregate) {
   const auto& descriptors = GetColumnDescriptors(num_extra_metrics);
 
-  for (size_t i = 0; i < aggregate.size(); i++) {
+  for (const auto& column : aggregate) {
     // Check when statistics has wrong amount of column entries
-    JXL_CHECK(aggregate[i].size() == descriptors.size());
+    JXL_ENSURE(column.size() == descriptors.size());
   }
 
   std::vector<ColumnValue> result(descriptors.size());
@@ -346,8 +380,8 @@ std::string PrintAggregate(
     }
     if (descriptors[i].type == TYPE_COUNT) {
       size_t sum = 0;
-      for (size_t j = 0; j < aggregate.size(); j++) {
-        sum += aggregate[j][i].i;
+      for (const auto& column : aggregate) {
+        sum += column[i].i;
       }
       result[i].i = sum;
       continue;
@@ -357,9 +391,8 @@ std::string PrintAggregate(
 
     double logsum = 0;
     size_t numvalid = 0;
-    for (size_t j = 0; j < aggregate.size(); j++) {
-      double value =
-          (type == TYPE_SIZE) ? aggregate[j][i].i : aggregate[j][i].f;
+    for (const auto& column : aggregate) {
+      double value = (type == TYPE_SIZE) ? column[i].i : column[i].f;
       if (value > 0) {
         numvalid++;
         logsum += std::log2(value);
@@ -367,12 +400,20 @@ std::string PrintAggregate(
     }
     double geomean = numvalid ? std::exp2(logsum / numvalid) : 0.0;
 
+    // ssimulacra2 can get negative, so use arithmetic mean instead
+    if (i == 7) {
+      geomean = 0;
+      for (const auto& column : aggregate) {
+        geomean += column[i].f;
+      }
+      geomean /= aggregate.size();
+    }
     if (type == TYPE_SIZE || type == TYPE_COUNT) {
-      result[i].i = static_cast<size_t>(geomean + 0.5);
+      result[i].i = static_cast<size_t>(std::llround(geomean));
     } else if (type == TYPE_POSITIVE_FLOAT) {
       result[i].f = geomean;
     } else {
-      JXL_ABORT("unknown entry type");
+      JXL_DEBUG_ABORT("Unreachable");
     }
   }
 
