@@ -4,6 +4,9 @@
 # Use of this source code is governed by a BSD-style
 # license that can be found in the LICENSE file.
 
+self=$(realpath "$0")
+mydir=$(dirname "${self}")
+
 ./compute_octave_metric.sh "$@" \
-  --path "$(dirname "$0")"/../../../third_party/hdr_metrics/ \
-  "$(dirname "$0")"/compute-pumetrics.m 'ssim'
+  --path "${mydir}"/../../../third_party/hdr_metrics/ \
+  "${mydir}"/compute-pumetrics.m 'ssim'
