@@ -4,23 +4,43 @@
 // license that can be found in the LICENSE file.
 
 #include <jxl/decode.h>
-#include <stdint.h>
 
-namespace jpegxl {
-namespace tools {
+#include <cstddef>
+#include <cstdint>
+#include <vector>
 
-int TestOneInput(const uint8_t* data, size_t size) {
+#include "lib/jxl/base/compiler_specific.h"
+#include "lib/jxl/fuzztest.h"
+#include "tools/tracking_memory_manager.h"
+
+namespace {
+
+using ::jpegxl::tools::kGiB;
+using ::jpegxl::tools::TrackingMemoryManager;
+
+void Check(bool ok) {
+  if (!ok) {
+    JXL_CRASH();
+  }
+}
+
+int DoTestOneInput(const uint8_t* data, size_t size) {
   JxlDecoderStatus status;
-  JxlDecoder* dec = JxlDecoderCreate(nullptr);
+  TrackingMemoryManager memory_manager{/* cap */ 1 * kGiB,
+                                       /* total_cap */ 5 * kGiB};
+  JxlDecoder* dec = JxlDecoderCreate(memory_manager.get());
+  const auto finish = [&]() -> int {
+    JxlDecoderDestroy(dec);
+    Check(memory_manager.Reset());
+    return 0;
+  };
+
   JxlDecoderSubscribeEvents(dec, JXL_DEC_BASIC_INFO | JXL_DEC_COLOR_ENCODING);
   JxlDecoderSetInput(dec, data, size);
 
   status = JxlDecoderProcessInput(dec);
 
-  if (status != JXL_DEC_BASIC_INFO) {
-    JxlDecoderDestroy(dec);
-    return 0;
-  }
+  if (status != JXL_DEC_BASIC_INFO) return finish();
 
   JxlBasicInfo info;
   status = JxlDecoderGetBasicInfo(dec, &info);
@@ -36,10 +56,7 @@ int TestOneInput(const uint8_t* data, size_t size) {
   }
   status = JxlDecoderProcessInput(dec);
 
-  if (status != JXL_DEC_COLOR_ENCODING) {
-    JxlDecoderDestroy(dec);
-    return 0;
-  }
+  if (status != JXL_DEC_COLOR_ENCODING) return finish();
 
   JxlDecoderGetColorAsEncodedProfile(dec, JXL_COLOR_PROFILE_TARGET_ORIGINAL,
                                      nullptr);
@@ -47,13 +64,17 @@ int TestOneInput(const uint8_t* data, size_t size) {
   JxlDecoderGetICCProfileSize(dec, JXL_COLOR_PROFILE_TARGET_ORIGINAL,
                               &dec_profile_size);
 
-  JxlDecoderDestroy(dec);
-  return 0;
+  return finish();
 }
 
-}  // namespace tools
-}  // namespace jpegxl
+}  // namespace
 
 extern "C" int LLVMFuzzerTestOneInput(const uint8_t* data, size_t size) {
-  return jpegxl::tools::TestOneInput(data, size);
+  return DoTestOneInput(data, size);
 }
+
+void TestOneInput(const std::vector<uint8_t>& data) {
+  DoTestOneInput(data.data(), data.size());
+}
+
+FUZZ_TEST(DecodeBasiInfoFuzzTest, TestOneInput);
