@@ -4,16 +4,19 @@
 // license that can be found in the LICENSE file.
 
 #include <jxl/cms.h>
+#include <jxl/memory_manager.h>
 #include <jxl/types.h>
-#include <stdio.h>
-#include <string.h>
 
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 #include <fstream>
 #include <iostream>
 #include <istream>
 #include <string>
 #include <unordered_map>
 
+#include "lib/jxl/base/status.h"
 #include "lib/jxl/codec_in_out.h"
 #include "lib/jxl/enc_cache.h"
 #include "lib/jxl/enc_fields.h"
@@ -24,6 +27,7 @@
 #include "lib/jxl/splines.h"
 #include "lib/jxl/test_utils.h"  // TODO(eustas): cut this dependency
 #include "tools/file_io.h"
+#include "tools/no_memory_manager.h"
 
 namespace jpegxl {
 namespace tools {
@@ -32,7 +36,6 @@ using ::jxl::BitWriter;
 using ::jxl::BlendMode;
 using ::jxl::CodecInOut;
 using ::jxl::CodecMetadata;
-using ::jxl::ColorCorrelationMap;
 using ::jxl::ColorEncoding;
 using ::jxl::ColorTransform;
 using ::jxl::CompressParams;
@@ -47,6 +50,7 @@ using ::jxl::PropertyDecisionNode;
 using ::jxl::QuantizedSpline;
 using ::jxl::Spline;
 using ::jxl::Splines;
+using ::jxl::StatusOr;
 using ::jxl::Tree;
 
 namespace {
@@ -55,15 +59,17 @@ struct SplineData {
   std::vector<Spline> splines;
 };
 
-Splines SplinesFromSplineData(const SplineData& spline_data) {
+StatusOr<Splines> SplinesFromSplineData(const SplineData& spline_data) {
   std::vector<QuantizedSpline> quantized_splines;
   std::vector<Spline::Point> starting_points;
   quantized_splines.reserve(spline_data.splines.size());
   starting_points.reserve(spline_data.splines.size());
   for (const Spline& spline : spline_data.splines) {
-    JXL_CHECK(!spline.control_points.empty());
-    quantized_splines.emplace_back(spline, spline_data.quantization_adjustment,
-                                   0.0, 1.0);
+    JXL_ASSIGN_OR_RETURN(
+        QuantizedSpline qspline,
+        QuantizedSpline::Create(spline, spline_data.quantization_adjustment,
+                                0.0, 1.0));
+    quantized_splines.emplace_back(std::move(qspline));
     starting_points.push_back(spline.control_points.front());
   }
   return Splines(spline_data.quantization_adjustment,
@@ -239,16 +245,21 @@ bool ParseNode(F& tok, Tree& tree, SplineData& spline_data,
     }
   } else if (t == "Alpha") {
     io.metadata.m.SetAlphaBits(io.metadata.m.bit_depth.bits_per_sample);
-    JXL_ASSIGN_OR_RETURN(ImageF alpha, ImageF::Create(W, H));
-    io.frames[0].SetAlpha(std::move(alpha));
+    JXL_ASSIGN_OR_RETURN(
+        ImageF alpha, ImageF::Create(jpegxl::tools::NoMemoryManager(), W, H));
+    if (!io.frames[0].SetAlpha(std::move(alpha))) {
+      fprintf(stderr, "Internal: SetAlpha failed\n");
+      return false;
+    }
   } else if (t == "Bitdepth") {
     t = tok();
     size_t num = 0;
-    io.metadata.m.bit_depth.bits_per_sample = std::stoul(t, &num);
-    if (num != t.size()) {
+    uint32_t bits_per_sample = std::stoul(t, &num);
+    if (num != t.size() || bits_per_sample < 1 || bits_per_sample > 32) {
       fprintf(stderr, "Invalid Bitdepth: %s\n", t.c_str());
       return false;
     }
+    io.metadata.m.bit_depth.bits_per_sample = bits_per_sample;
   } else if (t == "FloatExpBits") {
     t = tok();
     size_t num = 0;
@@ -425,6 +436,17 @@ bool ParseNode(F& tok, Tree& tree, SplineData& spline_data,
         return false;
       }
     }
+  } else if (t == "PQ") {
+    io.metadata.m.color_encoding.Tf().transfer_function =
+        jxl::TransferFunction::kPQ;
+    io.metadata.m.tone_mapping.intensity_target = 10000;
+  } else if (t == "HLG") {
+    io.metadata.m.color_encoding.Tf().transfer_function =
+        jxl::TransferFunction::kHLG;
+    io.metadata.m.tone_mapping.intensity_target = 1000;
+  } else if (t == "Rec2100") {
+    JXL_RETURN_IF_ERROR(
+        io.metadata.m.color_encoding.SetPrimariesType(jxl::Primaries::k2100));
   } else {
     fprintf(stderr, "Unexpected node type: %s\n", t.c_str());
     return false;
@@ -435,7 +457,8 @@ bool ParseNode(F& tok, Tree& tree, SplineData& spline_data,
 }
 }  // namespace
 
-int JxlFromTree(const char* in, const char* out, const char* tree_out) {
+::jxl::Status JxlFromTree(const char* in, const char* out,
+                          const char* tree_out) {
   Tree tree;
   SplineData spline_data;
   CompressParams cparams = {};
@@ -449,7 +472,8 @@ int JxlFromTree(const char* in, const char* out, const char* tree_out) {
   cparams.ec_resampling = 1;
   cparams.modular_group_size_shift = 3;
   cparams.colorspace = 0;
-  CodecInOut io;
+  JxlMemoryManager* memory_manager = jpegxl::tools::NoMemoryManager();
+  CodecInOut io{memory_manager};
   int have_next = JXL_FALSE;
 
   std::istream* f = &std::cin;
@@ -474,11 +498,13 @@ int JxlFromTree(const char* in, const char* out, const char* tree_out) {
     PrintTree(tree, tree_out);
   }
   JXL_ASSIGN_OR_RETURN(
-      Image3F image,
-      Image3F::Create(width * cparams.resampling, height * cparams.resampling));
-  io.SetFromImage(std::move(image), ColorEncoding::SRGB());
-  io.SetSize((width + x0) * cparams.resampling,
-             (height + y0) * cparams.resampling);
+      Image3F image, Image3F::Create(memory_manager, width * cparams.resampling,
+                                     height * cparams.resampling));
+  JXL_RETURN_IF_ERROR(
+      io.SetFromImage(std::move(image), io.metadata.m.color_encoding));
+  JXL_RETURN_IF_ERROR(io.SetSize((width + x0) * cparams.resampling,
+                                 (height + y0) * cparams.resampling));
+
   io.metadata.m.color_encoding.DecideIfWantICC(*JxlGetDefaultCms());
   cparams.options.zero_tokens = true;
   cparams.palette_colors = 0;
@@ -487,11 +513,12 @@ int JxlFromTree(const char* in, const char* out, const char* tree_out) {
   cparams.patches = jxl::Override::kOff;
   cparams.already_downsampled = true;
   cparams.custom_fixed_tree = tree;
-  cparams.custom_splines = SplinesFromSplineData(spline_data);
-  PaddedBytes compressed;
+  JXL_ASSIGN_OR_RETURN(cparams.custom_splines,
+                       SplinesFromSplineData(spline_data));
+  PaddedBytes compressed{memory_manager};
 
-  io.CheckMetadata();
-  BitWriter writer;
+  JXL_RETURN_IF_ERROR(io.CheckMetadata());
+  BitWriter writer{memory_manager};
 
   std::unique_ptr<CodecMetadata> metadata = jxl::make_unique<CodecMetadata>();
   *metadata = io.metadata;
@@ -508,7 +535,8 @@ int JxlFromTree(const char* in, const char* out, const char* tree_out) {
       metadata->m.extra_channel_info.emplace_back();
       auto& eci = metadata->m.extra_channel_info.back();
       eci.type = jxl::ExtraChannel::kOptional;
-      JXL_ASSIGN_OR_DIE(ImageF ch, ImageF::Create(io.xsize(), io.ysize()));
+      JXL_ASSIGN_OR_RETURN(
+          ImageF ch, ImageF::Create(memory_manager, io.xsize(), io.ysize()));
       io.frames[0].extra_channels().emplace_back(std::move(ch));
     }
   }
@@ -525,9 +553,9 @@ int JxlFromTree(const char* in, const char* out, const char* tree_out) {
     io.frames[0].origin.y0 = y0;
     info.clamp = false;
 
-    JXL_RETURN_IF_ERROR(jxl::EncodeFrame(cparams, info, metadata.get(),
-                                         io.frames[0], *JxlGetDefaultCms(),
-                                         nullptr, &writer, nullptr));
+    JXL_RETURN_IF_ERROR(jxl::EncodeFrame(
+        memory_manager, cparams, info, metadata.get(), io.frames[0],
+        *JxlGetDefaultCms(), nullptr, &writer, nullptr));
     if (!have_next) break;
     tree.clear();
     spline_data.splines.clear();
@@ -538,8 +566,10 @@ int JxlFromTree(const char* in, const char* out, const char* tree_out) {
       return 1;
     }
     cparams.custom_fixed_tree = tree;
-    JXL_ASSIGN_OR_RETURN(Image3F image, Image3F::Create(width, height));
-    io.SetFromImage(std::move(image), ColorEncoding::SRGB());
+    JXL_ASSIGN_OR_RETURN(Image3F image,
+                         Image3F::Create(memory_manager, width, height));
+    JXL_RETURN_IF_ERROR(
+        io.SetFromImage(std::move(image), ColorEncoding::SRGB()));
     io.frames[0].blend = true;
   }
 
@@ -547,10 +577,10 @@ int JxlFromTree(const char* in, const char* out, const char* tree_out) {
 
   if (!WriteFile(out, compressed)) {
     fprintf(stderr, "Failed to write to \"%s\"\n", out);
-    return 1;
+    return JXL_FAILURE("Failed to write output");
   }
 
-  return 0;
+  return true;
 }
 }  // namespace tools
 }  // namespace jpegxl
@@ -561,6 +591,11 @@ int main(int argc, char** argv) {
     fprintf(stderr, "Usage: %s tree_in.txt out.jxl [tree_drawing]\n", argv[0]);
     return 1;
   }
-  return jpegxl::tools::JxlFromTree(argv[1], argv[2],
-                                    argc < 4 ? nullptr : argv[3]);
+  jxl::Status result = jpegxl::tools::JxlFromTree(argv[1], argv[2],
+                                                  argc < 4 ? nullptr : argv[3]);
+  if (!result) {
+    fprintf(stderr, "FAILURE\n");
+    return EXIT_FAILURE;
+  }
+  return EXIT_SUCCESS;
 }
