@@ -5,19 +5,65 @@
 
 // This example prints information from the main codestream header.
 
-#include <inttypes.h>
+#include <jxl/compressed_icc.h>
+#include <jxl/decode.h>
+#include <jxl/gain_map.h>
+
+// NB: this is a .c file, C++ headers are not allowed.
+#include <inttypes.h>  // PRIu64
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "jxl/decode.h"
+static void PrintColorEncoding(const JxlColorEncoding* color_encoding) {
+  const char* const cs_string[4] = {"RGB", "Grayscale", "XYB", "Unknown"};
+  const char* const wp_string[12] = {"", "D65", "Custom", "", "",  "",
+                                     "", "",    "",       "", "E", "P3"};
+  const char* const pr_string[12] = {"", "sRGB", "Custom",   "", "",  "", "",
+                                     "", "",     "Rec.2100", "", "P3"};
+  const char* const tf_string[19] = {
+      "", "709", "Unknown", "",     "", "", "",   "",    "Linear", "",
+      "", "",    "",        "sRGB", "", "", "PQ", "DCI", "HLG"};
+  const char* const ri_string[4] = {"Perceptual", "Relative", "Saturation",
+                                    "Absolute"};
+  printf("%s, ", cs_string[(*color_encoding).color_space]);
+  printf("%s, ", wp_string[(*color_encoding).white_point]);
+  if ((*color_encoding).white_point == JXL_WHITE_POINT_CUSTOM) {
+    printf("white_point(x=%f,y=%f), ", (*color_encoding).white_point_xy[0],
+           (*color_encoding).white_point_xy[1]);
+  }
+  if ((*color_encoding).color_space == JXL_COLOR_SPACE_RGB ||
+      (*color_encoding).color_space == JXL_COLOR_SPACE_UNKNOWN) {
+    printf("%s primaries", pr_string[(*color_encoding).primaries]);
+    if ((*color_encoding).primaries == JXL_PRIMARIES_CUSTOM) {
+      printf(": red(x=%f,y=%f),", (*color_encoding).primaries_red_xy[0],
+             (*color_encoding).primaries_red_xy[1]);
+      printf("  green(x=%f,y=%f),", (*color_encoding).primaries_green_xy[0],
+             (*color_encoding).primaries_green_xy[1]);
+      printf("  blue(x=%f,y=%f)", (*color_encoding).primaries_blue_xy[0],
+             (*color_encoding).primaries_blue_xy[1]);
+    } else
+      printf(", ");
+  }
+  if ((*color_encoding).transfer_function == JXL_TRANSFER_FUNCTION_GAMMA) {
+    printf("gamma(%f) transfer function, ", (*color_encoding).gamma);
+  } else {
+    printf("%s transfer function, ",
+           tf_string[(*color_encoding).transfer_function]);
+  }
+  printf("rendering intent: %s", ri_string[(*color_encoding).rendering_intent]);
+}
 
-int PrintBasicInfo(FILE* file, int verbose) {
+static int PrintBasicInfo(FILE* file, int verbose) {
   uint8_t* data = NULL;
   size_t data_size = 0;
   // In how large chunks to read from the file and try decoding the basic info.
   const size_t chunk_size = 2048;
+  uint8_t* box_data = NULL;
+  size_t box_size = 0;
+  size_t box_index = 0;
+  JxlBoxType box_type = {0};
 
   JxlDecoder* dec = JxlDecoderCreate(NULL);
   if (!dec) {
@@ -28,9 +74,10 @@ int PrintBasicInfo(FILE* file, int verbose) {
   JxlDecoderSetKeepOrientation(dec, 1);
   JxlDecoderSetCoalescing(dec, JXL_FALSE);
 
-  if (JXL_DEC_SUCCESS != JxlDecoderSubscribeEvents(
-                             dec, JXL_DEC_BASIC_INFO | JXL_DEC_COLOR_ENCODING |
-                                      JXL_DEC_FRAME | JXL_DEC_BOX)) {
+  if (JXL_DEC_SUCCESS !=
+      JxlDecoderSubscribeEvents(
+          dec, JXL_DEC_BASIC_INFO | JXL_DEC_COLOR_ENCODING | JXL_DEC_FRAME |
+                   JXL_DEC_BOX | JXL_DEC_BOX_COMPLETE)) {
     fprintf(stderr, "JxlDecoderSubscribeEvents failed\n");
     JxlDecoderDestroy(dec);
     return 0;
@@ -90,7 +137,7 @@ int PrintBasicInfo(FILE* file, int verbose) {
       if (info.exponent_bits_per_sample) {
         printf("float (%d exponent bits) ", info.exponent_bits_per_sample);
       }
-      int cmyk = 0, alpha = 0;
+      int cmyk = 0;
       const char* const ec_type_names[] = {
           "Alpha",     "Depth",     "Spotcolor", "Selection", "Black",
           "CFA",       "Thermal",   "Reserved0", "Reserved1", "Reserved2",
@@ -105,17 +152,12 @@ int PrintBasicInfo(FILE* file, int verbose) {
           break;
         }
         if (extra.type == JXL_CHANNEL_BLACK) cmyk = 1;
-        if (extra.type == JXL_CHANNEL_ALPHA) alpha = 1;
       }
-      if (info.num_color_channels == 1)
+      if (info.num_color_channels == 1) {
         printf("Grayscale");
-      else {
+      } else {
         if (cmyk) {
-          printf("CMYK");
-          cmyk = 0;
-        } else if (alpha) {
-          printf("RGBA");
-          alpha = 0;
+          printf("CMY");
         } else {
           printf("RGB");
         }
@@ -126,15 +168,6 @@ int PrintBasicInfo(FILE* file, int verbose) {
           fprintf(stderr, "JxlDecoderGetExtraChannelInfo failed\n");
           break;
         }
-        if (extra.type == JXL_CHANNEL_BLACK && cmyk == 0) {
-          cmyk = 1;
-          continue;
-        }
-        if (extra.type == JXL_CHANNEL_ALPHA && alpha == 0) {
-          alpha = 1;
-          continue;
-        }
-
         printf("+%s", (extra.type < ec_type_names_size
                            ? ec_type_names[extra.type]
                            : "Unknown, please update your libjxl"));
@@ -175,11 +208,12 @@ int PrintBasicInfo(FILE* file, int verbose) {
             printf("  name: %s\n", name);
             free(name);
           }
-          if (extra.type == JXL_CHANNEL_ALPHA)
+          if (extra.type == JXL_CHANNEL_ALPHA) {
             printf("  alpha_premultiplied: %d (%s)\n",
                    extra.alpha_premultiplied,
                    extra.alpha_premultiplied ? "Premultiplied"
                                              : "Non-premultiplied");
+          }
           if (extra.type == JXL_CHANNEL_SPOT_COLOR) {
             printf("  spot_color: (%f, %f, %f) with opacity %f\n",
                    extra.spot_color[0], extra.spot_color[1],
@@ -229,59 +263,20 @@ int PrintBasicInfo(FILE* file, int verbose) {
         fprintf(stderr, "Invalid orientation\n");
       }
     } else if (status == JXL_DEC_COLOR_ENCODING) {
-      JxlPixelFormat format = {4, JXL_TYPE_FLOAT, JXL_LITTLE_ENDIAN, 0};
       printf("Color space: ");
 
       JxlColorEncoding color_encoding;
       if (JXL_DEC_SUCCESS ==
-          JxlDecoderGetColorAsEncodedProfile(dec, &format,
-                                             JXL_COLOR_PROFILE_TARGET_ORIGINAL,
-                                             &color_encoding)) {
-        const char* const cs_string[4] = {"RGB", "Grayscale", "XYB", "Unknown"};
-        const char* const wp_string[12] = {"", "D65", "Custom", "", "",  "",
-                                           "", "",    "",       "", "E", "P3"};
-        const char* const pr_string[12] = {
-            "", "sRGB", "Custom", "", "", "", "", "", "", "Rec.2100", "", "P3"};
-        const char* const tf_string[19] = {
-            "", "709", "Unknown", "",     "", "", "",   "",    "Linear", "",
-            "", "",    "",        "sRGB", "", "", "PQ", "DCI", "HLG"};
-        const char* const ri_string[4] = {"Perceptual", "Relative",
-                                          "Saturation", "Absolute"};
-        printf("%s, ", cs_string[color_encoding.color_space]);
-        printf("%s, ", wp_string[color_encoding.white_point]);
-        if (color_encoding.white_point == JXL_WHITE_POINT_CUSTOM) {
-          printf("white_point(x=%f,y=%f), ", color_encoding.white_point_xy[0],
-                 color_encoding.white_point_xy[1]);
-        }
-        if (color_encoding.color_space == JXL_COLOR_SPACE_RGB ||
-            color_encoding.color_space == JXL_COLOR_SPACE_UNKNOWN) {
-          printf("%s primaries", pr_string[color_encoding.primaries]);
-          if (color_encoding.primaries == JXL_PRIMARIES_CUSTOM) {
-            printf(": red(x=%f,y=%f),", color_encoding.primaries_red_xy[0],
-                   color_encoding.primaries_red_xy[1]);
-            printf("  green(x=%f,y=%f),", color_encoding.primaries_green_xy[0],
-                   color_encoding.primaries_green_xy[1]);
-            printf("  blue(x=%f,y=%f)", color_encoding.primaries_blue_xy[0],
-                   color_encoding.primaries_blue_xy[1]);
-          } else
-            printf(", ");
-        }
-        if (color_encoding.transfer_function == JXL_TRANSFER_FUNCTION_GAMMA) {
-          printf("gamma(%f) transfer function, ", color_encoding.gamma);
-        } else {
-          printf("%s transfer function, ",
-                 tf_string[color_encoding.transfer_function]);
-        }
-        printf("rendering intent: %s\n",
-               ri_string[color_encoding.rendering_intent]);
-
+          JxlDecoderGetColorAsEncodedProfile(
+              dec, JXL_COLOR_PROFILE_TARGET_ORIGINAL, &color_encoding)) {
+        PrintColorEncoding(&color_encoding);
+        puts("");
       } else {
         // The profile is not in JPEG XL encoded form, get as ICC profile
         // instead.
         size_t profile_size;
         if (JXL_DEC_SUCCESS !=
-            JxlDecoderGetICCProfileSize(dec, &format,
-                                        JXL_COLOR_PROFILE_TARGET_ORIGINAL,
+            JxlDecoderGetICCProfileSize(dec, JXL_COLOR_PROFILE_TARGET_ORIGINAL,
                                         &profile_size)) {
           fprintf(stderr, "JxlDecoderGetICCProfileSize failed\n");
           continue;
@@ -292,10 +287,9 @@ int PrintBasicInfo(FILE* file, int verbose) {
           continue;
         }
         uint8_t* profile = (uint8_t*)malloc(profile_size);
-        if (JXL_DEC_SUCCESS !=
-            JxlDecoderGetColorAsICCProfile(dec, &format,
-                                           JXL_COLOR_PROFILE_TARGET_ORIGINAL,
-                                           profile, profile_size)) {
+        if (JXL_DEC_SUCCESS != JxlDecoderGetColorAsICCProfile(
+                                   dec, JXL_COLOR_PROFILE_TARGET_ORIGINAL,
+                                   profile, profile_size)) {
           fprintf(stderr, "JxlDecoderGetColorAsICCProfile failed\n");
           free(profile);
           continue;
@@ -326,11 +320,11 @@ int PrintBasicInfo(FILE* file, int verbose) {
       } else {
         printf("full image size");
       }
-
-      float ms = frame_header.duration * 1000.f *
-                 info.animation.tps_denominator / info.animation.tps_numerator;
-      total_duration += ms;
       if (info.have_animation) {
+        float ms = frame_header.duration * 1000.f *
+                   info.animation.tps_denominator /
+                   info.animation.tps_numerator;
+        total_duration += ms;
         printf(", duration: %.1f ms", ms);
         if (info.animation.have_timecodes) {
           printf(", time code: %X", frame_header.timecode);
@@ -349,37 +343,106 @@ int PrintBasicInfo(FILE* file, int verbose) {
       }
       printf("\n");
     } else if (status == JXL_DEC_BOX) {
-      JxlBoxType type;
       uint64_t size;
-      JxlDecoderGetBoxType(dec, type, JXL_FALSE);
+      uint64_t contents_size;
+      JxlDecoderGetBoxType(dec, box_type, JXL_FALSE);
       JxlDecoderGetBoxSizeRaw(dec, &size);
+      JxlDecoderGetBoxSizeContents(dec, &contents_size);
       if (verbose) {
-        printf("box: type: \"%c%c%c%c\" size: %" PRIu64 "\n", type[0], type[1],
-               type[2], type[3], (uint64_t)size);
+        printf("box: type: \"%.4s\" size: %" PRIu64 ", contents size: %" PRIu64
+               "\n",
+               box_type, size, contents_size);
       }
-      if (!strncmp(type, "JXL ", 4)) {
+      if (!strncmp(box_type, "JXL ", 4)) {
         printf("JPEG XL file format container (ISO/IEC 18181-2)\n");
-      } else if (!strncmp(type, "ftyp", 4)) {
-      } else if (!strncmp(type, "jxlc", 4)) {
-      } else if (!strncmp(type, "jxlp", 4)) {
-      } else if (!strncmp(type, "jxll", 4)) {
-      } else if (!strncmp(type, "jxli", 4)) {
+      } else if (!strncmp(box_type, "ftyp", 4)) {
+      } else if (!strncmp(box_type, "jxlc", 4)) {
+      } else if (!strncmp(box_type, "jxlp", 4)) {
+      } else if (!strncmp(box_type, "jxll", 4)) {
+      } else if (!strncmp(box_type, "jxli", 4)) {
         printf("Frame index box present\n");
-      } else if (!strncmp(type, "jbrd", 4)) {
+      } else if (!strncmp(box_type, "jbrd", 4)) {
         printf("JPEG bitstream reconstruction data available\n");
-      } else if (!strncmp(type, "jumb", 4) || !strncmp(type, "Exif", 4) ||
-                 !strncmp(type, "xml ", 4)) {
-        printf("Uncompressed %c%c%c%c metadata: %" PRIu64 " bytes\n", type[0],
-               type[1], type[2], type[3], (uint64_t)size);
+      } else if (!strncmp(box_type, "jumb", 4) ||
+                 !strncmp(box_type, "Exif", 4) ||
+                 !strncmp(box_type, "xml ", 4)) {
+        printf("Uncompressed %.4s metadata: %" PRIu64 " bytes\n", box_type,
+               size);
 
-      } else if (!strncmp(type, "brob", 4)) {
-        JxlDecoderGetBoxType(dec, type, JXL_TRUE);
-        printf("Brotli-compressed %c%c%c%c metadata: %" PRIu64
+      } else if (!strncmp(box_type, "brob", 4)) {
+        JxlDecoderGetBoxType(dec, box_type, JXL_TRUE);
+        printf("Brotli-compressed %.4s metadata: %" PRIu64
                " compressed bytes\n",
-               type[0], type[1], type[2], type[3], (uint64_t)size);
+               box_type, size);
+      } else if (!strncmp(box_type, "jhgm", 4)) {
+        box_data = malloc(chunk_size);
+        box_size = chunk_size;
+        JxlDecoderSetBoxBuffer(dec, box_data, box_size);
       } else {
-        printf("unknown box: type: \"%c%c%c%c\" size: %" PRIu64 "\n", type[0],
-               type[1], type[2], type[3], (uint64_t)size);
+        printf("unknown box: type: \"%.4s\" size: %" PRIu64 "\n", box_type,
+               size);
+      }
+    } else if (status == JXL_DEC_BOX_NEED_MORE_OUTPUT) {
+      const size_t remaining = JxlDecoderReleaseBoxBuffer(dec);
+      box_size += chunk_size;
+      box_index += chunk_size - remaining;
+      void* temp = realloc(box_data, box_size);
+      if (temp == NULL) {
+        free(box_data);
+        box_data = NULL;
+        box_size = 0;
+        box_index = 0;
+        fprintf(stderr, "Memory reallocation failed\n");
+        break;
+      }
+      box_data = temp;
+      JxlDecoderSetBoxBuffer(dec, box_data + box_index, box_size - box_index);
+    } else if (status == JXL_DEC_BOX_COMPLETE) {
+      if (!strncmp(box_type, "jhgm", 4)) {
+        size_t remaining = JxlDecoderReleaseBoxBuffer(dec);
+        box_size -= remaining;
+        JxlGainMapBundle gain_map_bundle;
+        size_t bytes_read;
+        if (!JxlGainMapReadBundle(&gain_map_bundle, box_data, box_size,
+                                  &bytes_read)) {
+          fprintf(stderr, "Invalid gain map box found\n");
+        } else {
+          uint8_t* icc = NULL;
+          size_t icc_size = 0;
+          JxlMemoryManager manager = {
+              .opaque = NULL, .alloc = NULL, .free = NULL};
+          if (gain_map_bundle.alt_icc_size > 0 &&
+              !JxlICCProfileDecode(&manager, gain_map_bundle.alt_icc,
+                                   gain_map_bundle.alt_icc_size, &icc,
+                                   &icc_size)) {
+            fprintf(stderr,
+                    "Invalid gain map box found (ICC profile does not "
+                    "decompress)\n");
+          }
+          printf("Gain map (jhgm) box: version = %u",
+                 gain_map_bundle.jhgm_version);
+          if (gain_map_bundle.has_color_encoding) {
+            printf(", color encoding = ");
+            PrintColorEncoding(&gain_map_bundle.color_encoding);
+          }
+          if (icc_size > 0) {
+            printf(", %lu-byte ICC profile", (unsigned long)icc_size);
+          }
+          printf(", %u-byte gain map, %u-byte metadata\n",
+                 gain_map_bundle.gain_map_size,
+                 gain_map_bundle.gain_map_metadata_size);
+          free(icc);
+        }
+        free(box_data);
+        box_data = NULL;
+        box_size = 0;
+        box_index = 0;
+      } else {
+        fprintf(
+            stderr,
+            "Unexpected JXL_DEC_BOX_COMPLETE event received for box \"%.4s\"\n",
+            box_type);
+        continue;
       }
     } else {
       fprintf(stderr, "Unexpected decoder status\n");
@@ -399,9 +462,10 @@ int PrintBasicInfo(FILE* file, int verbose) {
 
 static void print_usage(const char* name) {
   fprintf(stderr,
-          "Usage: %s [-v] INPUT\n"
-          "  INPUT      input JPEG XL image filename(s)\n"
-          "  -v         more verbose output\n",
+          "Usage: %s [-v] [-h] INPUT\n"
+          "  INPUT                  input JPEG XL image filename(s)\n"
+          "  -v (or --verbose)      more verbose output\n"
+          "  -h (or --help or -?)   this help)\n",
           name);
 }
 
@@ -421,41 +485,46 @@ static int print_basic_info_filename(const char* jxl_filename, int verbose) {
   return 0;
 }
 
-int main(int argc, char* argv[]) {
-  int verbose = 0, status = 0;
-  const char* const name = argv[0];
-
-  for (int i = 1; i < argc; i++) {
-    const char* const* help_opts =
-        (const char* const[]){"--help", "-h", "-?", NULL};
-    while (*help_opts) {
-      if (!strcmp(*help_opts++, argv[i])) {
-        print_usage(name);
-        return 0;
-      }
+static int is_flag(const char* arg, const char* const* opts) {
+  for (int i = 0; opts[i] != NULL; i++) {
+    if (!strcmp(opts[i], arg)) {
+      return 1;
     }
   }
+  return 0;
+}
 
+int main(int argc, char* argv[]) {
+  int verbose = 0;
+  int status = 0;
+  const char* const name = argv[0];
+  const char* const* help_opts =
+      (const char* const[]){"--help", "-h", "-?", NULL};
   const char* const* verbose_opts =
       (const char* const[]){"--verbose", "-v", NULL};
-  /* argc >= 2 gate prevents segfault on argc = 1 */
-  while (argc >= 2 && *verbose_opts) {
-    if (!strcmp(*verbose_opts++, argv[1])) {
-      verbose = 1;
-      argc--;
-      argv++;
-      break;
-    }
-  }
-
   if (argc < 2) {
     print_usage(name);
     return 2;
   }
 
-  while (argc-- >= 2) {
-    status |= print_basic_info_filename(*++argv, verbose);
+  // First pass: Check for flags
+  for (int i = 1; i < argc; i++) {
+    if (!verbose && is_flag(argv[i], verbose_opts)) {
+      verbose = 1;
+    }
+    if (is_flag(argv[i], help_opts)) {
+      print_usage(name);
+      return 0;
+    }
   }
 
+  // Second pass: print info
+  while (argc-- >= 2) {
+    if (is_flag(*(argv + 1), verbose_opts) || is_flag(*(argv + 1), help_opts)) {
+      ++argv;
+    } else {
+      status |= print_basic_info_filename(*++argv, verbose);
+    }
+  }
   return status;
 }
